// commondefs.pas
// This file is part of the DendoPlayer.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit CommonDefs;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StrUtils, rtlconsts,
  DendoDB;

const
  DownloadBufferCapacity = 2 * 1024 * 1024; // 2MiB

var
  ConfigFilePath: TFileName;
const
  ConfigFileName   = 'dendoplayer.ini';
  DatabaseFileName = 'dendodatabase.csv';
  DPSection        = 'dendoplayer';
  MuseSection      = 'Muse';
  PFSection        = 'PlayerForm';
  DDSection        = 'DownloadDatabaseDialog';
  DLSection        = 'DownloadLzhDialog';
  SDSection        = 'SearchDialog';
  HDSection        = 'HeaderWidthsDialog';

{ Dendo genre types }

type
  TGenreType = (
    gtPopular, gtClassical, gtMovie, gtTelevision, gtAnimation, gyChorus,
    gtWind   , gtDisney   , gtFolk , gtGame      , gtChristmas, gtOther
  );

const
  TGenreName: array[TGenreType] of String = (
    'Popular', 'Classical', 'Movie', 'Television', 'Animation', 'Chorus',
    'Wind'   , 'Disney'   , 'Folk' , 'Game'      , 'Christmas', 'Other'
  );

  TGenreQuery: array[TGenreType] of String[6] = (
    '%82%6F', '%82%62', '%82%72', '%82%73', '%82%60', '%82%74',
    '%82%76', '%82%63', '%82%65', '%82%66', '%82%77', '%82%6E'
  );

{ Header definitions }

type
  THeaderType = (htState, htStar, htTitle, htGenre, htAuthor, htDate,
    htTime, htMember, htSize, htFileName);
const
  HeaderTitles: array[THeaderType] of String =
    ('', '', 'Title', 'Genre', 'Author', 'Date', 'Time', 'Mem.', 'Size', 'File');
  HeaderNames: array[THeaderType] of String =
    ('Status', 'Star', 'Title', 'Genre', 'Author', 'Date', 'Time', 'Member', 'Size', 'File');
  HeaderDefaultWidths: array[THeaderType] of Integer =
    (25, 25, 200, 75, 100, 90, 60, 40, 70, 100);

const
  FieldTable: array[THeaderType] of TDendoFieldType = (
    dftState   , // htState
    dftStar    , // htStar
    dftTitle   , // htTitle
    dftGenre   , // htGenre
    dftAuthor  , // htAuthor
    dftDate    , // htDate
    dftTime    , // htTime
    dftMember  , // htMember
    dftSize    , // htSize
    dftFileName  // htFileName
  );

  TextAlignmentTable: array[THeaderType] of TAlignment = (
    taCenter       , // htState
    taCenter       , // htStar
    taLeftJustify  , // htTitle
    taLeftJustify  , // htGenre
    taLeftJustify  , // htAuthor
    taLeftJustify  , // htDate
    taRightJustify , // htTime
    taRightJustify , // htMember
    taRightJustify , // htSize
    taLeftJustify    // htFileName
  );

{ miscellaneous }

// get after last '/'
function GetFileNameFromURL(const URL: String): String;
// get after last '='
function GetLzhIDFromURL(const URL: String): String;
// return entire content as one String
function StreamToString(Stream: TStream): String;
// case insensitive Pos()
function TextPos(const SubStr: String; const Source: String): SizeInt;
// clip string between FromStr and ToStr (case insensitive)
function StrClip(const Source: String; const FromStr: String; const ToStr: String): String;
// '123456789' -> '123,456,789'
function CommaSepText(const Str: String): String;
// case insensitive
function IsIncludeText(const SubStr: String; const Source: String): Boolean;
// format file size string
function GetFileSizeStr(const Str: String): String;

{ TDownloadStream }
// TMemoryStream with Capacity property :-)

type
  TDownloadSTream = class(TStream)
  private
    FCapacity: Longint;
    FMemory: Pointer;
    FSize, FPosition: Longint;
  protected
    function Realloc(var NewCapacity: Longint): Pointer; virtual;
    procedure SetCapacity(NewCapacity: Longint);
    procedure SetPointer(Ptr: Pointer; ASize: Longint);
  public
    property Capacity: Longint read FCapacity write SetCapacity;
    property Memory: Pointer read FMemory;
    destructor Destroy(); override;
    procedure Clear();
    function GetSize(): Int64; override;
    procedure LoadFromStream(Stream: TStream);
    procedure LoadFromFile(const FileName: String);
    function Read(var Buffer; Count: Longint): Longint; override;
    procedure SaveToFile(const FileName: String);
    procedure SaveToStream(Stream: TStream);
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    procedure SetSize(NewSize: Longint); override;
    function Write(const Buffer; Count: Longint): Longint; override;
  end;

implementation

{ miscellaneous }

function GetAfterChar(const Ch: Char; const Source: String): String;
var
  Index: Integer;
begin
  Result := '';
  Index := RPos(Ch, Source);
  if 0 < Index then
    Result := Copy(Source, Index + 1, Length(Source) - Index);
end;

function GetFileNameFromURL(const URL: String): String;
begin
  Result := GetAfterChar('/', URL);
end;

function GetLzhIDFromURL(const URL: String): String;
begin
  Result := GetAfterChar('=', URL);
end;

function StreamToString(Stream: TStream): String;
var
  Size: Int64;
//  P: PByte;
begin
  Size := Stream.Size;
  SetLength(Result, Size);
  Stream.Position := 0;
  Stream.ReadBuffer(Pointer(Result)^, Size);
//  P := Pointer(Result) + Size;
//  P^ := 0;
end;

function TextPos(const SubStr: String; const Source: String): SizeInt;
var
  LSubStr, LSource: String;
begin
  LSubStr := LowerCase(SubStr);
  LSource := LowerCase(Source);
  Result := Pos(LSubStr, LSource);
end;

function StrClip(const Source: String; const FromStr: String;
  const ToStr: String): String;
var
  Res: String;
  Index: Integer;
begin
  Result := '';
  Res := Source;
  Index := TextPos(FromStr, Res);
  if 0 = Index then exit;
  Res := Copy(Res, Index + Length(FromStr), Length(Res));
  Index := TextPos(ToStr, Res);
  if 0 = Index then exit;
  Result := Copy(Res, 1, Index - 1);
end;

function CommaSepText(const Str: String): String;
var
  i, cnt: Integer;
begin
  Result := '';
  cnt := 0;
  for i := Length(Str) downto 1 do
  begin
    Result := Str[i] + Result;
    if 1 = i then break;
    Inc(cnt);
    if 3 = cnt then
    begin
      Result := ',' + Result;
      cnt := 0;
    end;
  end;
end;

function IsIncludeText(const SubStr: String; const Source: String): Boolean;
begin
  Result := 0 < TextPos(SubStr, Source);
end;

function GetFileSizeStr(const Str: String): String;
var
  S: String;
  i: Integer;
begin
  S := '';
  for i := 1 to Length(Str) do
    if Str[i] in ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'] then
      S := S + Str[i];
  Result := PadLeft(CommaSepText(S), 8);
end;

{ TDownloadSTream }

const
  TMSGrow = 102400; { Use 100k blocks. }

function TDownloadSTream.Realloc(var NewCapacity: Longint): Pointer;
begin
  if NewCapacity < 0 then
    NewCapacity := 0
  else
  begin
    // if growing, grow at least a quarter
    if (NewCapacity > FCapacity) and
       (NewCapacity < (5 * FCapacity) div 4) then
      NewCapacity := (5 * FCapacity) div 4;
    // round off to block size.
    NewCapacity := (NewCapacity + (TMSGrow - 1)) and not (TMSGrow - 1);
  end;
  // Only now check !
  if NewCapacity = FCapacity then
    Result := FMemory
  else
  begin
    Result := Reallocmem(FMemory, Newcapacity);
    if (Result = nil) and (Newcapacity > 0) then
      raise EStreamError.Create(SMemoryStreamError);
  end;
end;

procedure TDownloadSTream.SetCapacity(NewCapacity: Longint);
begin
  SetPointer(Realloc(NewCapacity), Fsize);
  FCapacity := NewCapacity;
end;

procedure TDownloadSTream.SetPointer(Ptr: Pointer; ASize: Longint);
begin
  FMemory := Ptr;
  FSize := ASize;
end;

destructor TDownloadSTream.Destroy();
begin
  Clear();
  inherited;
end;

procedure TDownloadSTream.Clear();
begin
  FSize := 0;
  FPosition := 0;
  SetCapacity(0);
end;

function TDownloadSTream.GetSize(): Int64;
begin
  Result := FSize;
end;

procedure TDownloadSTream.LoadFromStream(Stream: TStream);
begin
  Stream.Position := 0;
  SetSize(Stream.Size);
  if FSize > 0 then Stream.ReadBuffer(FMemory^, FSize);
end;

procedure TDownloadSTream.LoadFromFile(const FileName: string);
var
  S: TFileStream;
begin
  S := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(S);
  finally
    S.Free();
  end;
end;

function TDownloadSTream.Read(var Buffer; Count: Longint): Longint;
begin
  Result := 0;
  if (FSize > 0) and (FPosition < Fsize) then
  begin
    Result := FSize - FPosition;
    if Result > Count then Result := Count;
    Move ((FMemory + FPosition)^, Buffer, Result);
    FPosition := Fposition + Result;
  end;
end;

procedure TDownloadSTream.SaveToFile(const FileName: String);
var
  S: TFileStream;
begin
  S := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(S);
  finally
    S.Free();
  end;
end;

procedure TDownloadSTream.SaveToStream(Stream: TStream);
begin
  if FSize > 0 then Stream.WriteBuffer(FMemory^, FSize);
end;

function TDownloadSTream.Seek(Offset: Longint; Origin: Word): Longint;
begin
  case Origin of
    soFromBeginning: FPosition := Offset;
    soFromEnd      : FPosition := FSize + Offset;
    soFromCurrent  : FpoSition := FPosition + Offset;
  end;
  Result := FPosition;
end;

procedure TDownloadSTream.SetSize(NewSize: Longint);
begin
  SetCapacity(NewSize);
  FSize := NewSize;
  if FPosition > FSize then
    FPosition := FSize;
end;

function TDownloadSTream.Write(const Buffer; Count: Longint): Longint;
var
  NewPos: Longint;
begin
  if Count = 0 then
    exit(0);
  NewPos := FPosition + Count;
  if NewPos > Fsize then
    begin
    if NewPos > FCapacity then
      SetCapacity(NewPos);
    FSize:=NewPos;
    end;
  System.Move(Buffer, (FMemory+FPosition)^, Count);
  FPosition := NewPos;
  Result := Count;
end;

end.
