// dldatadlg.pas - download Dendo database.
// This file is part of the DendoPlayer.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit DlDataDlg;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, IniFiles, StrUtils,
  lNet, lhttp, lHTTPUtil,
  DendoDB, CommonDefs, SUConv;

const
  QuerySize = '50';
  DatabaseCgiQueryFmt = 'http://hpcgi3.nifty.com/~atomic/db/database.cgi'
    + '?did=data2&tid=list2&word=2&print=' + QuerySize
    + '&equal3=%S;p=%S';

type

  { TDownloadDatabaseDialog }

  TDownloadDatabaseDialog = class(TForm)
    CancelButton: TButton;
    MessageLabel: TLabel;
    StartTimer: TTimer;
    DisplayTimer: TTimer;
    procedure CancelButtonClick(Sender: TObject);
    procedure DisplayTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StartTimerTimer(Sender: TObject);
  private
    FDendoDatabase: TDendoDatabase;
  protected
    Cancel: Boolean;
    CurrentGenre: TGenreType;
    Downloading: Boolean;
    DownloadedSize: Integer;
    DownloadStream: TDownloadStream;
    IndicateLabels: array[TGenreType] of TLabel;
    MessageLabels: array[TGenreType] of TLabel;
    Success: Boolean;
    procedure HttpClientDisconnect(ASocket: TLSocket);
    procedure HttpClientDoneInput(ASocket: TLHTTPClientSocket);
    function HttpClientInput(ASocket: TLHTTPClientSocket; ABuffer: PChar;
      ASize: Integer): Integer;
    procedure HttpClientProcessHeaders(ASocket: TLHTTPClientSocket);
    function ParseHtml(): Boolean;
    procedure StartDownload();
  public
    property DendoDatabase: TDendoDatabase read FDendoDatabase;
    function Execute(): Boolean;
  end; 

var
  DownloadDatabaseDialog: TDownloadDatabaseDialog;

implementation

{ TDownloadDatabaseDialog }

procedure TDownloadDatabaseDialog.CancelButtonClick(Sender: TObject);
begin
  Cancel := True;
end;

procedure TDownloadDatabaseDialog.DisplayTimerTimer(Sender: TObject);
begin
  MessageLabels[CurrentGenre].Caption :=
    TGenreName[CurrentGenre] + ' (' +
    CommaSepText(IntToStr(DownloadedSize)) + ' bytes)';
end;

procedure TDownloadDatabaseDialog.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  CloseAction := caHide;
end;

procedure TDownloadDatabaseDialog.FormCreate(Sender: TObject);
var
  Genre: TGenreType;
begin
  DownloadStream := TDownloadStream.Create();
  FDendoDatabase := TDendoDatabase.Create();

  for Genre := Low(TGenreType) to High(TGenreType) do
  begin
    IndicateLabels[Genre] := TLabel.Create(Self);
    with IndicateLabels[Genre] do
    begin
      Caption := '';
      Left := 40;
      Parent := Self;
      ParentFont := True;
      ShowAccelChar := False;
      Top := 56 + 24 * Ord(Genre);
    end;
    MessageLabels[Genre] := TLabel.Create(Self);
    with MessageLabels[Genre] do
    begin
      Caption := TGenreName[Genre];
      Left := 64;
      Parent := Self;
      ParentFont := True;
      ShowAccelChar := False;
      Top := 56 + 24 * Ord(Genre);
    end;
  end;
  
  with TIniFile.Create(ConfigFilePath) do
  try
    Left := ReadInteger(DDSection, 'Left', 0);
    Top  := ReadInteger(DDSection, 'Top' , 0);
  finally
    Free();
  end;
end;

procedure TDownloadDatabaseDialog.FormDestroy(Sender: TObject);
begin
  DendoDatabase.Free();
  DownloadStream.Free();
  with TIniFile.Create(ConfigFilePath) do
  try
    WriteInteger(DDSection, 'Left', Left);
    WriteInteger(DDSection, 'Top' , Top );
  finally
    Free();
  end;
end;

procedure TDownloadDatabaseDialog.StartTimerTimer(Sender: TObject);
begin
  StartTimer.Enabled := False;
  StartDownload();
end;

procedure TDownloadDatabaseDialog.HttpClientDisconnect(ASocket: TLSocket);
begin
  DisplayTimer.Enabled := False;
  Downloading := False;
end;

procedure TDownloadDatabaseDialog.HttpClientDoneInput(
  ASocket: TLHTTPClientSocket);
begin
  Success := True;
  ASocket.Disconnect();
end;

function TDownloadDatabaseDialog.HttpClientInput(ASocket: TLHTTPClientSocket;
  ABuffer: PChar; ASize: Integer): Integer;
begin
  DownloadStream.WriteBuffer(ABuffer^, ASize);
  DownloadedSize += ASize;
  Result := ASize;
end;

procedure TDownloadDatabaseDialog.HttpClientProcessHeaders(ASocket: TLHTTPClientSocket);
begin
  if hsOK = ASocket.ResponseStatus then
    DisplayTimer.Enabled := True
  else
    ASocket.Disconnect();
end;

function TDownloadDatabaseDialog.ParseHtml(): Boolean;
var
  Html, Rec: TStringList;
  Line: String;
  Field: Integer;
  DBRec: TDendoDBRecord;
  l: Integer;
  S: String;
begin
  Result := False;
  Html := TStringList.Create();
  Rec := TStringList.Create();
  l := 0;
  try
    Html.Text := StreamToString(DownloadStream);
    while l < Html.Count do
    begin
      Line := Html[l];
      Inc(l);
      if IsIncludeText('<TABLE border="1" cellspacing="2" cellpadding="2">',
        Line) then break;
    end;
    
    Field := -1;
    while l < Html.Count do
    begin
      Line := Html[l];
      Inc(l);

      if IsIncludeText('</TR>', Line) then // end of record
      begin
        Rec.Clear();
        Field := -1;
        continue;
      end;
      if IsIncludeText('</TABLE>', Line) then break; // go next step

      case Field of
      -1:
        if IsIncludeText('<TR class="l3', Line) then
        begin
          Rec.Add(''); // dftState
          Field := Ord(dftStar);
        end;
      Ord(dftStar):
        if IsIncludeText('<TD nowrap align="left">', Line) then
        begin
          S := StrClip(Line, '<FONT size="-1" color="#a8a8a8">', '</FONT>');
          if (#$81 = S[1]) and (#$40 = S[2]) then // zenkaku space
            S := '  ';
          Rec.Add(SjisToUtf8(S));
          Field := Ord(dftTitle);
        end;
      Ord(dftTitle):
        if IsIncludeText('<CENTER>', Line) then
        begin
          Rec.Add(SjisToUtf8(
            StrClip(Line, '<CENTER>', '</CENTER>')));
          Rec.Add(TGenreName[CurrentGenre]); // dftGenre
          Field := Ord(dftAuthor);
        end;
      Ord(dftAuthor):
        if IsIncludeText('<P align="center">', Line) then
        begin
          Rec.Add(SjisToUtf8(
            StrClip(Line, '<P align="center">', '</P>')));
          Field := Ord(dftDate);
        end;
      Ord(dftDate):
        if IsIncludeText('<P align="center">', Line) then
        begin
          Rec.Add(FormatDateTime('yyyy/mm/dd', StrToDate(
            StrClip(Line, '<P align="center">', '</P>'))));
          Field := Ord(dftTime);
        end;
      Ord(dftTime):
        if IsIncludeText('<P align="center">', Line) then
        begin
          Rec.Add(PadLeft(StrClip(Line, '<P align="center">', '</P>'), 6));
          Field := Ord(dftMember);
        end;
      Ord(dftMember):
        if IsIncludeText('<P align="center">', Line) then
        begin
          Rec.Add(PadLeft(StrClip(Line, '<P align="center">', '</P>'), 2));
          Field := Ord(dftSize);
        end;
      Ord(dftSize):
        if IsIncludeText('<P align="right">', Line) then
        begin
          Rec.Add(GetFileSizeStr(StrClip(Line, '<P align="right">', '</P>')));
          Field := Ord(dftID);
        end;
      Ord(dftID):
        if IsIncludeText('<P align="center">', Line) then
        begin
          Rec.Add(PadLeft(GetLzhIDFromURL(StrClip(Line, '<A href="', '">')), 4));
          Field := Ord(dftFileName);
        end;
      Ord(dftFileName):
        if IsIncludeText('</A></P>', Line) then
        begin
          Rec.Add(StrClip(Line, '      ', '</A></P>'));
          DBRec := TDendoDBRecord.Create();
          DBRec.Text := Rec.CommaText;
          FDendoDatabase.Add(DBRec);
        end;
      end;
    end; // while
    
    while l < Html.Count do
    begin
      Line := Html[l];
      Inc(l);
      if IsIncludeText('NEXT', Line) then
      begin
        Result := True;
        break;
      end;
    end;
  finally
    Rec.Free();
    Html.Free();
  end;
end;

procedure TDownloadDatabaseDialog.StartDownload();
var
  Genre: TGenreType;
  HttpClient: TLHTTPClient;
  URL: String;
  AHost, AURI: String;
  APort: Word;
  Page: Integer;
  Res: Boolean;
begin
  FDendoDatabase.Clear();
  Cancel := False;

  for Genre := Low(TGenreType) to High(TGenreType) do
  begin
    CurrentGenre := Genre;
    DownloadedSize := 0;
    IndicateLabels[Genre].Caption := '>';
    MessageLabels[Genre].Caption := TGenreName[Genre] + ' (0 bytes)';
    Application.ProcessMessages();

    Page := 0;
    while True do
    try
      Downloading := True;
      DownloadStream.Clear();
      DownloadStream.Capacity := DownloadBufferCapacity;
      Success := False;

      URL := Format(DatabaseCgiQueryFmt, [TGenreQuery[Genre], IntToStr(Page)]);
      DecomposeURL(URL, AHost, AURI, APort);
      HttpClient := TLHTTPClient.Create(Self);
      try
        with HttpClient do
        begin
          Host := AHost;
          OnDisConnect     := @HttpClientDisconnect;
          OnDoneInput      := @HttpClientDoneInput;
          OnInput          := @HttpClientInput;
          OnProcessHeaders := @HttpClientProcessHeaders;
          Port := APort;
          URI := AURI;
          SendRequest();
        end;
        while Downloading do
        begin
          HttpClient.CallAction();
          Application.ProcessMessages();
          if Cancel then
          begin
            HttpClient.Disconnect();
            ModalResult := mrCancel;
            exit;
          end;
        end;

        if not Success then
        begin
          ModalResult := mrAbort;
          exit;
        end;

      finally
        DisplayTimer.Enabled := False;
        HttpClient.Free();
      end; // try

      // process buffstream to buffdatabase
      MessageLabels[CurrentGenre].Caption :=
        TGenreName[CurrentGenre] + ' (' +
        CommaSepText(IntToStr(DownloadedSize)) + ' bytes) (Parsing ...)';
      Application.ProcessMessages();

      Res := ParseHtml();

      Application.ProcessMessages();
      if Cancel then
      begin
        ModalResult := mrCancel;
        exit;
      end;

      if not Res then break;

      Inc(Page);

    finally
      DownloadStream.Clear();
    end; // while try

    IndicateLabels[Genre].Caption := 'v';
    MessageLabels[Genre].Caption := TGenreName[Genre];
  end; // for Genre

  FDendoDatabase.SortBy(dftID, False);
  ModalResult := mrOk
end;

function TDownloadDatabaseDialog.Execute(): Boolean;
var
  Genre: TGenreType;
begin
  for Genre := Low(TGenreType) to High(TGenreType) do
  begin
    IndicateLabels[Genre].Caption := '';
    MessageLabels[Genre].Caption := TGenreName[Genre];
  end;
  StartTimer.Enabled := True;
  Result := (mrOk = ShowModal());
end;

initialization
  {$I dldatadlg.lrs}

end.
