// dllzhdlg.pas - download .lzh file.
// This file is part of the DendoPlayer.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit DlLzhDlg;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, IniFiles,
  lNet, lhttp, lHTTPUtil,
  CommonDefs;

const
  DrCgiURL = 'http://atomic.x0.com/marumigi/dlrank/dlranklog.cgi';
  DrCgiParam = '?dl=';
const
  HRefFromStr = '<a href="';
  HRefToStr = '"';

type

  { TDownloadLzhDialog }

  TDownloadLzhDialog = class(TForm)
    CancelButton: TButton;
    MessageLabel: TLabel;
    StartTimer: TTimer;
    procedure CancelButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StartTimerTimer(Sender: TObject);
  private
    FDownloadStream: TDownloadStream;
  protected
    Cancel: Boolean;
    Downloading: Boolean;
    FileName: TFileName;
    HtmlHttpClient: TLHTTPClient;
    LzhHttpClient: TLHTTPClient;
    LzhID: String; // ID number string to CGI query
    Success: Boolean;
    procedure HttpClientDisconnect(ASocket: TLSocket);
    procedure HttpClientDoneInput(ASocket: TLHTTPClientSocket);
    function HttpClientInput(ASocket: TLHTTPClientSocket; ABuffer: PChar;
      ASize: Integer): Integer;
    procedure HtmlHttpClientProcessHeaders(ASocket: TLHTTPClientSocket);
    procedure LzhHttpClientProcessHeaders(ASocket: TLHTTPClientSocket);
    procedure StartDownload();
  public
    property DownloadStream: TDownloadStream read FDownloadStream;
    function Execute(const ALzhID: String; const AFileName: TFileName): Boolean;
  end; 

var
  DownloadLzhDialog: TDownloadLzhDialog;

implementation

{ TDownloadLzhDialog }

procedure TDownloadLzhDialog.CancelButtonClick(Sender: TObject);
begin
  Cancel := True;
end;

procedure TDownloadLzhDialog.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  CloseAction := caHide;
end;

procedure TDownloadLzhDialog.FormCreate(Sender: TObject);
begin
  FDownloadStream := TDownloadStream.Create();
  with TIniFile.Create(ConfigFilePath) do
  try
    Left := ReadInteger(DLSection, 'Left', 0);
    Top  := ReadInteger(DLSection, 'Top' , 0);
  finally
    Free();
  end;
end;

procedure TDownloadLzhDialog.FormDestroy(Sender: TObject);
begin
  FDownloadStream.Free();
  with TIniFile.Create(ConfigFilePath) do
  try
    WriteInteger(DLSection, 'Left', Left);
    WriteInteger(DLSection, 'Top' , Top );
  finally
    Free();
  end;
end;

procedure TDownloadLzhDialog.StartTimerTimer(Sender: TObject);
begin
  StartTimer.Enabled := False;
  StartDownload();
end;

procedure TDownloadLzhDialog.HttpClientDisconnect(ASocket: TLSocket);
begin
  Downloading := False;
end;

procedure TDownloadLzhDialog.HttpClientDoneInput(ASocket: TLHTTPClientSocket);
begin
  Success := True;
  ASocket.Disconnect();
end;

function TDownloadLzhDialog.HttpClientInput(ASocket: TLHTTPClientSocket;
  ABuffer: PChar; ASize: Integer): Integer;
begin
  FDownloadStream.WriteBuffer(ABuffer^, ASize);
  Result := ASize;
end;

procedure TDownloadLzhDialog.HtmlHttpClientProcessHeaders(ASocket: TLHTTPClientSocket);
begin
  if hsFound <> ASocket.ResponseStatus then
    ASocket.Disconnect();
end;

procedure TDownloadLzhDialog.LzhHttpClientProcessHeaders(ASocket: TLHTTPClientSocket);
begin
  if hsOK <> ASocket.ResponseStatus then
    ASocket.Disconnect();
end;

procedure TDownloadLzhDialog.StartDownload();
var
  URL: String;
  AHost, AURI: String;
  APort: Word;
  Phase: Integer;
  CurrentHttpClient: TLHTTPClient;
begin
  HtmlHttpClient := TLHTTPClient.Create(Self);
  LzhHttpClient := TLHTTPClient.Create(Self);
  try
    Cancel := False;

    for Phase := 1 to 2 do
    begin
      case Phase of
        1: CurrentHttpClient := HtmlHttpClient;
        2: CurrentHttpClient := LzhHttpClient;
      end;
      
      case Phase of
        1: begin
             URL := DrCgiURL + DrCgiParam + LzhID;
           end;
        2: begin
             // Get HRef from html
             URL := StrClip(StreamToString(DownloadStream), HRefFromStr, HRefToStr);
             if 0 <> CompareText(FileName, GetFileNameFromURL(URL)) then
             begin
               ShowMessage('Database mismatch occurred.'#$0D#$0A'Update database, please!');
               Success := False;
               exit;
             end;
           end;
      end;

      DownloadStream.Clear();
      DownloadStream.Capacity := DownloadBufferCapacity;
      Downloading := True;
      Success := False;
      case Phase of
        1: MessageLabel.Caption:='Downloading HTML data ...';
        2: MessageLabel.Caption:='Downloading '+ FileName + '...';
      end;
      Application.ProcessMessages();

      DecomposeURL(URL, AHost, AURI, APort);
      with CurrentHttpClient do
      begin
        OnDisconnect := @HttpClientDisconnect;
        OnDoneInput := @HttpClientDoneInput;
        OnInput := @HttpClientInput;
        case Phase of
          1: OnProcessHeaders := @HtmlHttpClientProcessHeaders;
          2: OnProcessHeaders := @LzhHttpClientProcessHeaders;
        end;
        Host := AHost;
        Port := APort;
        URI  := AURI;
        SendRequest();
      end;
      while Downloading do
      begin
        CurrentHttpClient.CallAction();
        Application.ProcessMessages();
        if Cancel then
        begin
          CurrentHttpClient.Disconnect();
          exit;
        end;
      end;
      if not Success then exit;
    end;

  finally
    LzhHttpClient.Free();
    HtmlHttpClient.Free();
    if Success then
      ModalResult := mrOk
    else if Cancel then
      ModalResult := mrCancel
    else
      ModalResult := mrAbort;
  end;
end;

function TDownloadLzhDialog.Execute(const ALzhID: String;
  const AFileName: TFileName): Boolean;
begin
  FileName := AFileName;
  LzhID := TrimLeft(ALzhID);
  StartTimer.Enabled := True;
  Result := (mrOk = ShowModal());
end;

initialization
  {$I dllzhdlg.lrs}

end.
