// extract.pas - extract .lzh file.
//     Windows: Use unlha32.dll
//     UNIX   : Use LHa for Unix
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit Extract;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  {$IFDEF WINDOWS}
  Windows
  {$ELSE}
  Process
  {$ENDIF}
  ;

function ExtractLzhFile(const FileName: TFileName;
  const Directory: TFileName): Boolean;

implementation

{$IFDEF WINDOWS}
const
  UNLHA32_DLL = 'unlha32';
function UnlhaGetRunning(): BOOL;
  stdcall; external UNLHA32_DLL name 'UnlhaGetRunning';
function Unlha(_hwnd: HWND; szCmdLine, szOutput: PChar; wSize: Integer): Integer;
  stdcall; external UNLHA32_DLL name 'Unlha';

function ExtractLzhFile(const FileName: TFileName;
  const Directory: TFileName): Boolean;
var
  Cmd: String;
begin
  // wait for other extraction done
  while UnlhaGetRunning() do Sleep(100);

  Cmd := 'e "' + FileName + '" "'
           + IncludeTrailingPathDelimiter(Directory) + '"';
  Result := (0 = Unlha(0, PChar(Cmd), nil, 0));
end;
{$ELSE}
function ExtractLzhFile(const FileName: TFileName;
  const Directory: TFileName): Boolean;
var
  Process: TProcess;
begin
  Process := TProcess.Create(nil);
  try
    with Process do
    begin
      CommandLine := 'lha eq "' + FileName+ '"';
      CurrentDirectory := Directory;
      Options := [poWaitOnExit];
      Execute();
      Result := (0 = ExitStatus);
    end;
  finally
    Process.Free();
  end;
end;
{$ENDIF}

end.
