// playerform.pas - DendoPlayer main form.
// This file is part of the DendoPlayer.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit PlayerForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Menus,
  PairSplitter, ComCtrls, ExtCtrls, StdCtrls, FileCtrl, process, LCLType,
  IniFiles,
  DlDataDlg, DlLzhDlg, SearchDlg, VerDlg,
  DendoDB, MuseDefs, Extract, CommonDefs, SUConv;

const
  VersionNumber    = '0.9.3';

type

  { TDendoPlayerForm }

  TDendoPlayerForm = class(TForm)
    DendoPlayerMainMenu: TMainMenu;
      FileMenu: TMenuItem;
        QuitCommand: TMenuItem;
      DataMenu: TMenuItem;
        UpdateDatabaseCommand: TMenuItem;
        Bar1: TMenuItem;
        SearchCommand: TMenuItem;
        RestoreDatabaseCommand: TMenuItem;
        Bar2: TMenuItem;
        ForceDownloadCommand: TMenuItem;
        ForceExtractCommand: TMenuItem;
      PlayMenu: TMenuItem;
        PlayCommand: TMenuItem;
      OptionMenu: TMenuItem;
        MusePathCommand: TMenuItem;
        MuseOptionsMenu: TMenuItem;
      HelpMenu: TMenuItem;
        VersionCommand: TMenuItem;
    DendoListBoxPopupMenu: TPopupMenu;
      DownloadAndExtractPopupCommand: TMenuItem;
      BarPopup1: TMenuItem;
      SearchPopupCommand: TMenuItem;
      RestoreDatabasePopupCommand: TMenuItem;
      BarPopup2: TMenuItem;
      ForceDownloadPopupCommand: TMenuItem;
      ForceExtractPopupCommand: TMenuItem;
    FileListBoxPopupMenu: TPopupMenu;
      PlayPopupCommand: TMenuItem;
    VerticalPairSplitter: TPairSplitter;
      UpperSide: TPairSplitterSide;
        DendoHeaderControl: THeaderControl;
        DendoListBox: TListBox;
      LowerSide: TPairSplitterSide;
        HorizontalPairSplitter: TPairSplitter;
          LeftSide: TPairSplitterSide;
            FileListBox: TFileListBox;
          RightSide: TPairSplitterSide;
            TextMemo: TMemo;
    ClearStatusTimer: TTimer;
    DataDirDialog: TSelectDirectoryDialog;
    MusePathDialog: TOpenDialog;
    MuseProcess: TProcess;
    StatusBar: TStatusBar;
    StartTimer: TTimer;
    procedure ClearStatusTimerTimer(Sender: TObject);
    procedure DendoHeaderControlSectionClick(
      HeaderControl: TCustomHeaderControl; Section: THeaderSection);
    procedure DendoListBoxClick(Sender: TObject);
    procedure DendoListBoxDblClick(Sender: TObject);
    procedure DendoListBoxDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    procedure DendoListBoxKeyPress(Sender: TObject; var Key: char);
    procedure FileListBoxChange(Sender: TObject);
    procedure FileListBoxDblClick(Sender: TObject);
    procedure FileListBoxKeyPress(Sender: TObject; var Key: char);
    procedure ForceDownloadCommandClick(Sender: TObject);
    procedure ForceExtractCommandClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MusePathCommandClick(Sender: TObject);
    procedure PlayCommandClick(Sender: TObject);
    procedure QuitCommandClick(Sender: TObject);
    procedure RestoreDatabaseCommandClick(Sender: TObject);
    procedure SearchCommandClick(Sender: TObject);
    procedure StartTimerTimer(Sender: TObject);
    procedure UpdateDatabaseCommandClick(Sender: TObject);
    procedure VersionCommandClick(Sender: TObject);
  private
    FMuseOptions: TMuseOptions;
    FMusePath: TFileName;
    procedure CreateHeaders();
    procedure CreateMenus();
    procedure LoadDatabase();
    procedure LoadFromIniFile();
    procedure SaveToIniFile();
  protected
    ConfigDir: TFileName;
    DatabasePath: TFileName;
    DataDir: TFileName;
    DendoDatabase: TDendoDatabase;
    LastAscending: Boolean;
    LastSortedHeader: THeaderType;
    ListDatabase: TDendoDatabase;
    LzhDir: TFileName;
    MusBaseDir: TFileName;
    MuseOptionCommands: array[TMuseOption] of TMenuItem;
    SelectedFileName: String;
    function FindMusePath(): Boolean;
    function GetHeaderSection(Hdr: THeaderType): THeaderSection;
    function GetHeaderWidths(Hdr: THeaderType): Integer;
    function GetMusDir(FileName: TFileName): TFileName;
    function GetSelectedFileName(): String;
    procedure MuseOptionCommandsClick(Sender: TObject);
    procedure SetDendoListBoxIndexBy(FileName: String);
    procedure SetFileListBoxDir(Dir: TFileName);
    procedure SetHeaderWidths(Hdr: THeaderType; AWidth: Integer);
    procedure SetMuseOptions(Value: TMuseOptions);
    procedure SetMusePath(Value: TFilename);
    procedure SetState(FileName: String; State: TDendoStateType);
    procedure SetStatusText(AText: String);
  public
    procedure DownloadLzh(Force: Boolean);
    procedure ExtractLzh();
    property HeaderWidths[Hdr: THeaderType]: Integer read GetHeaderWidths write SetHeaderWidths;
    property MuseOptions: TMuseOptions read FMuseOptions write SetMuseOptions;
    property MusePath: TFilename read FMusePath write SetMusePath;
    procedure PlayMus();
    procedure RestoreDatabase();
    procedure Search();
    procedure SortListBy(HeaderType: THeaderType);
    procedure UpdateDatabase();
  end;

var
  DendoPlayerForm: TDendoPlayerForm;

implementation

{ TDendoPlayerForm }

{ Generated by IDE }

procedure TDendoPlayerForm.ClearStatusTimerTimer(Sender: TObject);
begin
  ClearStatusTimer.Enabled := False;
  StatusBar.SimpleText := '';
end;

procedure TDendoPlayerForm.DendoHeaderControlSectionClick(
  HeaderControl: TCustomHeaderControl; Section: THeaderSection);
begin
  SortListBy(THeaderType(Section.ID));
end;

procedure TDendoPlayerForm.DendoListBoxClick(Sender: TObject);
var
  FileName: String;
  MusDir: TFileName;
begin
  FileName := GetSelectedFileName();
  MusDir := GetMusDir(FileName);
  if not DirectoryExists(MusDir) then
    CreateDir(MusDir);
  SetFileListBoxDir(MusDir);
end;

procedure TDendoPlayerForm.DendoListBoxDblClick(Sender: TObject);
begin
  DownloadLzh(False);
end;

procedure TDendoPlayerForm.DendoListBoxDrawItem(Control: TWinControl;
  Index: Integer; ARect: TRect; State: TOwnerDrawState);
var
  DrawBox: TListBox;
  DrawCanvas: TCanvas;
  DrawRect: TRect;
  OldBrushStyle: TBrushStyle;
  OldTextStyle: TTextStyle;
  Hdr: THeaderType;
  HeaderSection: THeaderSection;
  ItemStr: String;
begin
  DrawBox := TListBox(Control);
  DrawCanvas := DrawBox.Canvas;
  DrawCanvas.FillRect(ARect);

  with DrawBox do
    if (Index < 0) or (Items.Count <= Index) then exit;

  OldBrushStyle := DrawCanvas.Brush.Style;
  OldTextStyle := DrawCanvas.TextStyle;
  DrawCanvas.Brush.Style := bsClear;
  DrawCanvas.TextStyle.Layout := tlCenter;

  for Hdr := Low(THeaderType) to High(THeaderType) do
  begin
    HeaderSection := GetHeaderSection(Hdr);
    ItemStr := ListDatabase[Index][FieldTable[Hdr]];
    DrawCanvas.TextStyle.Alignment := TextAlignmentTable[Hdr];
    DrawRect := Rect(HeaderSection.Left + 2,
                     ARect.Top,
                     HeaderSection.Right - 6,
                     ARect.Bottom);
    DrawCanvas.TextRect(DrawRect, DrawRect.Left, DrawRect.Top, ItemStr);
  end;

  DrawCanvas.Brush.Style := OldBrushStyle;
  DrawCanvas.TextStyle := OldTextStyle;
end;

procedure TDendoPlayerForm.DendoListBoxKeyPress(Sender: TObject; var Key: char);
begin
  if #$0D = Key then // CR
    DownloadLzh(False);
end;

procedure TDendoPlayerForm.FileListBoxChange(Sender: TObject);
var
  FileName: TFileName;
  Ext: String;
  S: TStringList;
begin
  TextMemo.Clear();
  FileName := FileListBox.FileName;
  if not FileExists(FileName) then exit;
  Ext := ExtractFileExt(FileName);
  if (0 <> CompareText('.mus', Ext)) and
     (0 <> CompareText('.txt', Ext)) and
     (0 <> CompareText('.htm', Ext)) and
     (0 <> CompareText('.html', Ext)) then exit;

  S := TStringList.Create();
  try
    S.LoadFromFile(FileName);
    TextMemo.Lines.Text := SjisToUtf8(S.Text);
    TextMemo.SelStart:=0;
    TextMemo.SelLength := 0;
  finally
    S.Free();
  end;
end;

procedure TDendoPlayerForm.FileListBoxDblClick(Sender: TObject);
begin
  PlayMus();
end;

procedure TDendoPlayerForm.FileListBoxKeyPress(Sender: TObject;
  var Key: char);
begin
  if #$0D = Key then // CR
    PlayMus()
end;

procedure TDendoPlayerForm.ForceDownloadCommandClick(Sender: TObject);
begin
  DownloadLzh(True);
end;

procedure TDendoPlayerForm.ForceExtractCommandClick(Sender: TObject);
begin
  ExtractLzh();
end;

procedure TDendoPlayerForm.FormCreate(Sender: TObject);
begin
  ConfigDir := GetAppConfigDir(False);
  if not DirectoryExists(ConfigDir) then
    ForceDirectories(ConfigDir);
  ConfigDir := IncludeTrailingPathDelimiter(ConfigDir);
  ConfigFilePath := ConfigDir + ConfigFileName;
  DatabasePath := ConfigDir + DatabaseFileName;

  DendoDatabase := TDendoDatabase.Create(); // for original
  ListDatabase := TDendoDatabase.Create(); // for DendoListBox

  CreateMenus();
  CreateHeaders();
  LoadFromIniFile();

  if not DirectoryExists(DataDir) then
    if DataDirDialog.Execute() then
      DataDir := DataDirDialog.FileName
    else
    begin
      Close();
      exit;
    end;
  DataDir := IncludeTrailingPathDelimiter(DataDir);
  LzhDir := IncludeTrailingPathDelimiter(DataDir + 'lzh');
  MusBaseDir := IncludeTrailingPathDelimiter(DataDir + 'mus');
  if not DirectoryExists(LzhDir) then
    CreateDir(LzhDir);
  if not DirectoryExists(MusBaseDir) then
    CreateDir(MusBaseDir);

  {$IFDEF WINDOWS}
  MusePathDialog.Filter := 'Muse executable|muse.exe';
  if not FileExists(MusePath) then
    if FileExists(MuseDefaultPath) then
      MusePath := MuseDefaultPath;
  {$ENDIF}
  if not FileExists(MusePath) then FindMusePath();
  
  LastSortedHeader := Low(THeaderType);
  LastAscending := False;
  DateSeparator := '/';
  ShortDateFormat := 'y/m/d';

  DendoHeaderControl.DoubleBuffered := True;
  DendoListBox.DoubleBuffered := True;
  ActiveControl := DendoListBox;

  {$IFDEF WINDOWS} // sometimes fail on Windows (why ?)
  SearchCommand.Enabled := False;
  SearchPopupCommand.Enabled := False;
  {$ENDIF}

  StartTimer.Enabled := True;
end;

procedure TDendoPlayerForm.FormDestroy(Sender: TObject);
begin
  SaveToIniFile();
  ListDatabase.Free();
  DendoDatabase.Free();
end;

procedure TDendoPlayerForm.MusePathCommandClick(Sender: TObject);
begin
  FindMusePath();
end;

procedure TDendoPlayerForm.PlayCommandClick(Sender: TObject);
begin
  PlayMus();
end;

procedure TDendoPlayerForm.QuitCommandClick(Sender: TObject);
begin
  Close();
end;

procedure TDendoPlayerForm.RestoreDatabaseCommandClick(Sender: TObject);
begin
  RestoreDatabase();
end;

procedure TDendoPlayerForm.SearchCommandClick(Sender: TObject);
begin
  Search();
end;

procedure TDendoPlayerForm.StartTimerTimer(Sender: TObject);
begin
  StartTimer.Enabled := False;
  LoadDatabase();
end;

procedure TDendoPlayerForm.UpdateDatabaseCommandClick(Sender: TObject);
begin
  UpdateDatabase();
end;

procedure TDendoPlayerForm.VersionCommandClick(Sender: TObject);
begin
  VersionDialog.Execute('Version ' + VersionNumber);
end;

{ private }

procedure TDendoPlayerForm.CreateHeaders();
var
  Hdr: THeaderType;
begin
  for Hdr := Low(THeaderType) to High(THeaderType) do
    with DendoHeaderControl.Sections.Add() do
    begin
      Alignment := taCenter;
      Text := HeaderTitles[Hdr];
    end;
end;

procedure TDendoPlayerForm.CreateMenus();
var
  Opt: TMuseOption;
  OptMenuItem: TMenuItem;
begin
  for Opt := Low(TMuseOption) to High(TMuseOption) do
  begin
    OptMenuItem := TMenuItem.Create(Self);
    with OptMenuItem do
    begin
      Caption := MuseOptionMenuCaptions[Opt];
      OnClick := @MuseOptionCommandsClick;
      ShowAlwaysCheckable := True;
      Tag := Ord(Opt);
    end;
    MuseOptionsMenu.Add(OptMenuItem);
    MuseOptionCommands[Opt] := OptMenuItem;
  end;
end;

procedure TDendoPlayerForm.LoadDatabase();
var
  State: String;
  LzhPath: String;
  i: Integer;
begin
  if FileExists(DatabasePath) then
    DendoDatabase.LoadFromFile(DatabasePath);

  for i := 0 to DendoDatabase.Count - 1 do
  begin
    State := DendoDatabase[i][dftState];
    LzhPath := LzhDir + DendoDatabase[i][dftFileName];
    // clear downloading state
    if DendoStateStrs[dstDownloading] = State then
      DendoDatabase[i][dftState] := DendoStateStrs[dstNotDownloaded];
    // file not exist (deleted ?)
    if (DendoStateStrs[dstDownloaded] = State) and not FileExists(LzhPath) then
      DendoDatabase[i][dftState] := DendoStateStrs[dstNotDownloaded];
    // if manually downloaded file is there
    if FileExists(LzhPath) then
      DendoDatabase[i][dftState] := DendoStateStrs[dstDownloaded];
  end;

  RestoreDatabase();
  SetDendoListBoxIndexBy(SelectedFileName);
end;

procedure TDendoPlayerForm.LoadFromIniFile();
var
  Hdr: THeaderType;
begin
  with TIniFile.Create(ConfigFilePath) do
  try
    WindowState := TWindowState(ReadInteger(PFSection, 'WindowState', Ord(wsNormal)));
    Height      := ReadInteger(PFSection, 'Height', Height);;
    Left        := ReadInteger(PFSection, 'Left', 0);
    Top         := ReadInteger(PFSection, 'Top', 0);
    Width       := ReadInteger(PFSection, 'Width', Width);
    DataDir     := ReadString(DPSection, 'DataDir', '');
    MusePath    := ReadString(MuseSection, 'MusePath', '');
    MuseOptions := TMuseOptions(ReadInteger(MuseSection, 'MuseOptions', 0));
    for Hdr := Low(THeaderType) to High(THeaderType) do
      HeaderWidths[Hdr] :=
        ReadInteger(PFSection, 'HeaderWidth' + IntToStr(Ord(Hdr)),
                    HeaderDefaultWidths[Hdr]);
    VerticalPairSplitter.Position :=
      ReadInteger(PFSection, 'VerticalPosition', 200);
    HorizontalPairSplitter.Position :=
      ReadInteger(PFSection, 'HorizontalPosition', 200);
    SelectedFileName := ReadString(PFSection, 'SelectedFileName', '');
  finally
    Free();
  end;
end;

procedure TDendoPlayerForm.SaveToIniFile();
var
  Hdr: THeaderType;
begin
  with TIniFile.Create(ConfigFilePath) do
  try
    WriteInteger(PFSection, 'Height', Height);
    WriteInteger(PFSection, 'Left', Left);
    WriteInteger(PFSection, 'Top', Top);
    WriteInteger(PFSection, 'Width', Width);
    if wsMinimized <> WindowState then
      WriteInteger(PFSection, 'WindowState', Ord(WindowState));
    WriteString(DPSection, 'DataDir', DataDir);
    WriteString(MuseSection, 'MusePath', MusePath);
    WriteInteger(MuseSection, 'MuseOptions', Longint(MuseOptions));
    for Hdr := Low(THeaderType) to High(THeaderType) do
      WriteInteger(PFSection, 'HeaderWidth' + IntToStr(Ord(Hdr)),
                  HeaderWidths[Hdr]);
    WriteInteger(PFSection, 'VerticalPosition', VerticalPairSplitter.Position);
    WriteInteger(PFSection, 'HorizontalPosition', HorizontalPairSplitter.Position);
    WriteString(PFSection, 'SelectedFileName', GetSelectedFileName());
  finally
    Free();
  end;
end;

{ protected }

function TDendoPlayerForm.FindMusePath(): Boolean;
begin
  Result := MusePathDialog.Execute();
  if Result then
    MusePath := MusePathDialog.FileName
  else
    Result := FileExists(MusePath);
end;

function TDendoPlayerForm.GetHeaderSection(Hdr: THeaderType): THeaderSection;
begin
  Result := DendoHeaderControl.Sections[Ord(Hdr)];
end;

function TDendoPlayerForm.GetHeaderWidths(Hdr: THeaderType): Integer;
begin
  Result := GetHeaderSection(Hdr).Width;
end;

function TDendoPlayerForm.GetMusDir(FileName: TFileName): TFileName;
begin
  if '' = FileName then
    Result := '.'
  else
    Result := MusBaseDir + ChangeFileExt(ExtractFileName(FileName), '');
end;

function TDendoPlayerForm.GetSelectedFileName(): String;
var
  Index: Integer;
begin
  Result := '';
  Index := DendoListBox.ItemIndex;
  if Index < 0 then exit;
  Result := ListDatabase[Index][dftFileName];
end;

procedure TDendoPlayerForm.MuseOptionCommandsClick(Sender: TObject);
var
  Opt: TMuseOption;
begin
  Opt := TMuseOption(TMenuItem(Sender).Tag);
  if Opt in MuseOptions then
    MuseOptions := MuseOptions - [Opt]
  else
    MuseOptions := MuseOptions + [Opt];
end;

procedure TDendoPlayerForm.SetDendoListBoxIndexBy(FileName: String);
var
  Index, TopIndex: Integer;
begin
  if 0 < DendoListBox.Count then
  begin
    Index := ListDatabase.IndexOf(dftFileName, FileName);
    if Index < 0 then Index := 0;
    TopIndex := Index - 1;
    if TopIndex < 0 then TopIndex := 0;
    DendoListBox.ItemIndex := Index;
    DendoListBox.TopIndex := TopIndex;
  end;
  {$IFDEF WINDOWS}DendoListBox.Click();{$ENDIF}
end;

procedure TDendoPlayerForm.SetFileListBoxDir(Dir: TFileName);
begin
  with FileListBox do
  begin
    Directory := '';
    Directory := Dir;
    if 0 < Count then ItemIndex := 0;
  end;
  {$IFDEF WINDOWS}FileListBox.Click();{$ENDIF}
end;

procedure TDendoPlayerForm.SetHeaderWidths(Hdr: THeaderType; AWidth: Integer);
begin
  GetHeaderSection(Hdr).Width := AWidth;
end;

procedure TDendoPlayerForm.SetMuseOptions(Value: TMuseOptions);
var
  Opt: TMuseOption;
begin
  if FMuseOptions = Value then exit;
  FMuseOptions := Value;
  for Opt := Low(TMuseOption) to High(TMuseOption) do
    MuseOptionCommands[Opt].Checked := Opt in Value;
end;

procedure TDendoPlayerForm.SetMusePath(Value: TFilename);
begin
  if FMusePath = Value then Exit;
  FMusePath := Value;
  MusePathDialog.InitialDir := ExtractFileDir(Value);
  MusePathDialog.FileName := ExtractFileName(Value);
end;

procedure TDendoPlayerForm.SetState(FileName: String; State: TDendoStateType);
var
  Index: Integer;
  Str: String;
begin
  Str := DendoStateStrs[State];

  Index := DendoDatabase.IndexOf(dftFileName, FileName);
  DendoDatabase[Index][dftState] := Str;

  Index := ListDatabase.IndexOf(dftFileName, FileName);
  if Index < 0 then exit;
  ListDatabase[Index][dftState] := Str;
  DendoListBox.Items[Index] := Str;
end;

procedure TDendoPlayerForm.SetStatusText(AText: String);
begin
  StatusBar.SimpleText := AText;
  Application.ProcessMessages();
  ClearStatusTimer.Enabled := False;
  ClearStatusTimer.Enabled := True;
end;

{ public }

procedure TDendoPlayerForm.DownloadLzh(Force: Boolean);
var
  Index: Integer;
  ID, FileName: String;
  LzhPath: TFileName;
begin
  Index := DendoListBox.ItemIndex;
  if Index < 0 then exit;

  ID := ListDatabase[Index][dftID];
  FileName := ListDatabase[Index][dftFileName];
  LzhPath := LzhDir + FileName;
  if (not Force) and FileExists(LzhPath) then exit;

  SetStatusText('Downloading Muse data file ...');
  SetState(FileName, dstDownloading);

  if DownloadLzhDialog.Execute(ID, FileName) then
  begin
    DownloadLzhDialog.DownloadStream.SaveToFile(LzhPath);
    SetStatusText('Download ' + FileName + ' success.');
    SetState(FileName, dstDownloaded);
    ExtractLzh();
  end
  else
  begin
    SetStatusText('Download ' + FileName + ' !!! FAILED !!!.');
    SetState(FileName, dstNotDownloaded);
  end;
  DownloadLzhDialog.DownloadStream.Clear();
end;

procedure TDendoPlayerForm.ExtractLzh();
var
  FileName: String;
  LzhPath, MusDir: TFileName;
begin
  FileName := GetSelectedFileName();
  if '' = FileName then exit;

  LzhPath := LzhDir + FileName;
  if not FileExists(LzhPath) then exit;

  MusDir := GetMusDir(FileName);
  if not DirectoryExists(MusDir) then CreateDir(MusDir);

  if not ExtractLzhFile(LzhPath, MusDir) then
    SetStatusText('Extract ' + FileName + ' !!! FAILED !!!.');

  SetFileListBoxDir(MusDir);
end;

procedure TDendoPlayerForm.PlayMus();
var
  Cmd: String;
  Opt: TMuseOption;
  MusFile: TFileName;
begin
  MusFile := FileListBox.FileName;
  if 0 <> CompareText('.mus', ExtractFileExt(MusFile)) then exit;

  if not FileExists(MusePath) then
    if not FindMusePath() then exit;

  SetStatusText('Playing ' + ExtractFileName(MusFile) + ' ...');

  Cmd := '"' + MusePath + '"';
  for Opt := Low(TMuseOption) to High(TMuseOption) do
    if Opt in MuseOptions then
      Cmd := Cmd + ' ' + GetMuseOptionStr(Opt);
  Cmd := Cmd + ' "' + MusFile + '"';
  MuseProcess.CommandLine := Cmd;
  MuseProcess.Execute();
end;

procedure TDendoPlayerForm.RestoreDatabase();
var
  FileName: String;
begin
  FileName := GetSelectedFileName();
  ListDatabase.Assign(DendoDatabase);
  ListDatabase.AssignToStrings(DendoListBox.Items);
  SetDendoListBoxIndexBy(FileName);
  LastAscending := False;
end;

procedure TDendoPlayerForm.Search();
var
  FileName: String;
begin
  FileName := GetSelectedFileName();

  if not SearchDialog.Execute(ListDatabase, DendoDatabase) then exit;

  ListDatabase.AssignToStrings(DendoListBox.Items);
  SetDendoListBoxIndexBy(FileName);
  LastAscending := False;
end;

procedure TDendoPlayerForm.SortListBy(HeaderType: THeaderType);
var
  FileName: String;
  Ascending: Boolean;
begin
  FileName := GetSelectedFileName();
  if '' = FileName then exit;

  if LastSortedHeader = HeaderType then
    Ascending := not LastAscending
  else
    Ascending := True;
  LastAscending := Ascending;
  LastSortedHeader := HeaderType;

  ListDatabase.SortBy(FieldTable[HeaderType], Ascending);
  ListDatabase.AssignToStrings(DendoListBox.Items);
  DendoListBox.Repaint();

  SetDendoListBoxIndexBy(FileName);
end;

procedure TDendoPlayerForm.UpdateDatabase();
begin
  SetStatusText('Updateing dendo database ...');
  if DownloadDatabaseDialog.Execute() then
  begin
    SetStatusText('Now processing data. Wait for a while, please.');
    DownloadDatabaseDialog.DendoDatabase.MergeDB(DendoDatabase);
    DendoDatabase.Assign(DownloadDatabaseDialog.DendoDatabase);
    DendoDatabase.SaveToFile(DatabasePath);
    DendoDatabase.SaveToFile(DataDir + DatabaseFileName);
    RestoreDatabase();
    SetStatusText('Update Dendo database success.');
  end
  else
    SetStatusText('Update Dendo database !!! FAILED !!!.');
end;

initialization
  {$I playerform.lrs}

end.
