// searchdlg.pas
// This file is part of the DendoPlayer.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit SearchDlg;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, IniFiles,
  DendoDB, CommonDefs, SUConv;

type

  { TSearchDialog }

  TSearchDialog = class(TForm)
    StateLabel: TLabel;
    StateComboBox: TComboBox;
    StarLabel: TLabel;
    StarComboBox: TComboBox;
    GenreLabel: TLabel;
    GenreComboBox: TComboBox;
    TitleLabel: TLabel;
    TitleEdit: TEdit;
    AuthorLabel: TLabel;
    AuthorEdit: TEdit;
    ResetButton: TButton;
    OkButton: TButton;
    CancelButton: TButton;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure ResetButtonClick(Sender: TObject);
  private
    DestDatabase: TDendoDatabase;
    SourceDatabase: TDendoDatabase;
  protected
    procedure Search();
  public
    function Execute(Dest, Source: TDendoDatabase): Boolean;
  end;

var
  SearchDialog: TSearchDialog;

implementation

{ TSearchDialog }

procedure TSearchDialog.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  CloseAction := caHide;
end;

procedure TSearchDialog.FormCreate(Sender: TObject);
var
  S: String;
begin
  with TIniFile.Create(ConfigFilePath) do
  try
    Left := ReadInteger(SDSection, 'Left', 0);
    Top  := ReadInteger(SDSection, 'Top' , 0);
  finally
    Free();
  end;

  StarComboBox.Items.Add('(None)');
  S := ''; S += #$81; S += #$9A; // black star
  StarComboBox.Items.Add(SjisToUtf8(S));
  S := ''; S += #$81; S += #$99; // white star
  StarComboBox.Items.Add(SjisToUtf8(S));
  StarComboBox.Items.Add('  '); // without star
  StarComboBox.ItemIndex := 0;
end;

procedure TSearchDialog.FormDestroy(Sender: TObject);
begin
  with TIniFile.Create(ConfigFilePath) do
  try
    WriteInteger(SDSection, 'Left', Left);
    WriteInteger(SDSection, 'Top' , Top );
  finally
    Free();
  end;
end;

procedure TSearchDialog.OkButtonClick(Sender: TObject);
begin
  Search();
  ModalResult := mrOk;
end;

procedure TSearchDialog.ResetButtonClick(Sender: TObject);
begin
  StateComboBox.ItemIndex := 0;
  StarComboBox .ItemIndex := 0;
  GenreComboBox.ItemIndex := 0;
  TitleEdit .Text := '';
  AuthorEdit.Text := '';
end;

procedure TSearchDialog.Search();
var
  Index: Integer;
  IsScanState, IsScanStar, IsScanGenre,
  IsScanTitle, IsScanAuthor: Boolean;
  StateStr, StarStr, GenreStr, TitleStr, AuthorStr: String;
  Rec: TDendoDBRecord;
begin
  IsScanState  := (0 < StateComboBox.ItemIndex);
  IsScanStar   := (0 < StarComboBox .ItemIndex);
  IsScanGenre  := (0 < GenreComboBox.ItemIndex);
  IsScanTitle  := ('' <> TitleEdit .Text);
  IsScanAuthor := ('' <> AuthorEdit.Text);
  StateStr  := StateComboBox.Text;
  StarStr   := StarComboBox .Text;
  GenreStr  := GenreComboBox.Text;
  TitleStr  := TitleEdit    .Text;
  AuthorStr := AuthorEdit   .Text;

  DestDatabase.Clear();

  for Index := 0 to SourceDatabase.Count - 1 do
  begin
    Rec := SourceDatabase[Index];
    if IsScanState then
      if 0 <> CompareStr(StateStr, Rec[dftState]) then
        continue;
    if IsScanStar then
      if 0 <> CompareStr(StarStr, Rec[dftStar]) then
        continue;
    if IsScanGenre then
      if 0 <> CompareStr(GenreStr, Rec[dftGenre]) then
        continue;
    if IsScanTitle then
      if 0 = Pos(TitleStr, Rec[dftTitle]) then
        continue;
    if IsScanAuthor then
      if 0 = Pos(AuthorStr, Rec[dftAuthor]) then
        continue;

    DestDatabase.Add(Rec.Dup());
  end;
end;

function TSearchDialog.Execute(Dest, Source: TDendoDatabase): Boolean;
begin
  DestDatabase   := Dest;
  SourceDatabase := Source;
  ActiveControl := StateComboBox;
  Result := (mrOk = ShowModal());
end;

initialization
  {$I searchdlg.lrs}

end.
