// suconv.pas - Shift-JIS(JIS X 0208) <--> UTF-8 converter.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit SUConv;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils; 

// Shift-JIS <-> UTF-8 convert
function SjisToUtf8(Value: String): String;
function Utf8ToSjis(Value: String): String;

implementation

type
  TUtfChar = array[0 .. 2] of Byte;
  TSjisRowTable = array[$40 .. $FC] of TUtfChar;

var
  Sjis2Utf8Table_81_84: array[$81..$84] of TSjisRowTable;
  Sjis2Utf8Table_88_9F: array[$88..$9F] of TSjisRowTable;
  Sjis2Utf8Table_E0_EA: array[$E0..$EA] of TSjisRowTable;

const
  S2U_81: TSjisRowTable =
    (($E3,$80,$80),($E3,$80,$81),($E3,$80,$82),($EF,$BC,$8C),
     ($EF,$BC,$8E),($E3,$83,$BB),($EF,$BC,$9A),($EF,$BC,$9B),
     ($EF,$BC,$9F),($EF,$BC,$81),($E3,$82,$9B),($E3,$82,$9C),
     ($C2,$B4,$00),($EF,$BD,$80),($C2,$A8,$00),($EF,$BC,$BE),
     ($E2,$80,$BE),($EF,$BC,$BF),($E3,$83,$BD),($E3,$83,$BE),
     ($E3,$82,$9D),($E3,$82,$9E),($E3,$80,$83),($E4,$BB,$9D),
     ($E3,$80,$85),($E3,$80,$86),($E3,$80,$87),($E3,$83,$BC),
     ($E2,$80,$94),($E2,$80,$90),($EF,$BC,$8F),($EF,$BC,$BC),
     ($E3,$80,$9C),($E2,$80,$96),($EF,$BD,$9C),($E2,$80,$A6),
     ($E2,$80,$A5),($E2,$80,$98),($E2,$80,$99),($E2,$80,$9C),
     ($E2,$80,$9D),($EF,$BC,$88),($EF,$BC,$89),($E3,$80,$94),
     ($E3,$80,$95),($EF,$BC,$BB),($EF,$BC,$BD),($EF,$BD,$9B),
     ($EF,$BD,$9D),($E3,$80,$88),($E3,$80,$89),($E3,$80,$8A),
     ($E3,$80,$8B),($E3,$80,$8C),($E3,$80,$8D),($E3,$80,$8E),
     ($E3,$80,$8F),($E3,$80,$90),($E3,$80,$91),($EF,$BC,$8B),
     ($E2,$88,$92),($C2,$B1,$00),($C3,$97,$00),($00,$00,$00),
     ($C3,$B7,$00),($EF,$BC,$9D),($E2,$89,$A0),($EF,$BC,$9C),
     ($EF,$BC,$9E),($E2,$89,$A6),($E2,$89,$A7),($E2,$88,$9E),
     ($E2,$88,$B4),($E2,$99,$82),($E2,$99,$80),($C2,$B0,$00),
     ($E2,$80,$B2),($E2,$80,$B3),($E2,$84,$83),($C2,$A5,$00),
     ($EF,$BC,$84),($C2,$A2,$00),($C2,$A3,$00),($EF,$BC,$85),
     ($EF,$BC,$83),($EF,$BC,$86),($EF,$BC,$8A),($EF,$BC,$A0),
     ($C2,$A7,$00),($E2,$98,$86),($E2,$98,$85),($E2,$97,$8B),
     ($E2,$97,$8F),($E2,$97,$8E),($E2,$97,$87),($E2,$97,$86),
     ($E2,$96,$A1),($E2,$96,$A0),($E2,$96,$B3),($E2,$96,$B2),
     ($E2,$96,$BD),($E2,$96,$BC),($E2,$80,$BB),($E3,$80,$92),
     ($E2,$86,$92),($E2,$86,$90),($E2,$86,$91),($E2,$86,$93),
     ($E3,$80,$93),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($E2,$88,$88),($E2,$88,$8B),($E2,$8A,$86),($E2,$8A,$87),
     ($E2,$8A,$82),($E2,$8A,$83),($E2,$88,$AA),($E2,$88,$A9),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($E2,$88,$A7),($E2,$88,$A8),($C2,$AC,$00),($E2,$87,$92),
     ($E2,$87,$94),($E2,$88,$80),($E2,$88,$83),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($E2,$88,$A0),($E2,$8A,$A5),
     ($E2,$8C,$92),($E2,$88,$82),($E2,$88,$87),($E2,$89,$A1),
     ($E2,$89,$92),($E2,$89,$AA),($E2,$89,$AB),($E2,$88,$9A),
     ($E2,$88,$BD),($E2,$88,$9D),($E2,$88,$B5),($E2,$88,$AB),
     ($E2,$88,$AC),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($E2,$84,$AB),($E2,$80,$B0),($E2,$99,$AF),($E2,$99,$AD),
     ($E2,$99,$AA),($E2,$80,$A0),($E2,$80,$A1),($C2,$B6,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($E2,$97,$AF));

  S2U_82: TSjisRowTable =
    (($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($EF,$BC,$90),
     ($EF,$BC,$91),($EF,$BC,$92),($EF,$BC,$93),($EF,$BC,$94),
     ($EF,$BC,$95),($EF,$BC,$96),($EF,$BC,$97),($EF,$BC,$98),
     ($EF,$BC,$99),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($EF,$BC,$A1),($EF,$BC,$A2),($EF,$BC,$A3),($EF,$BC,$A4),
     ($EF,$BC,$A5),($EF,$BC,$A6),($EF,$BC,$A7),($EF,$BC,$A8),
     ($EF,$BC,$A9),($EF,$BC,$AA),($EF,$BC,$AB),($EF,$BC,$AC),
     ($EF,$BC,$AD),($EF,$BC,$AE),($EF,$BC,$AF),($EF,$BC,$B0),
     ($EF,$BC,$B1),($EF,$BC,$B2),($EF,$BC,$B3),($EF,$BC,$B4),
     ($EF,$BC,$B5),($EF,$BC,$B6),($EF,$BC,$B7),($EF,$BC,$B8),
     ($EF,$BC,$B9),($EF,$BC,$BA),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($EF,$BD,$81),($EF,$BD,$82),($EF,$BD,$83),
     ($EF,$BD,$84),($EF,$BD,$85),($EF,$BD,$86),($EF,$BD,$87),
     ($EF,$BD,$88),($EF,$BD,$89),($EF,$BD,$8A),($EF,$BD,$8B),
     ($EF,$BD,$8C),($EF,$BD,$8D),($EF,$BD,$8E),($EF,$BD,$8F),
     ($EF,$BD,$90),($EF,$BD,$91),($EF,$BD,$92),($EF,$BD,$93),
     ($EF,$BD,$94),($EF,$BD,$95),($EF,$BD,$96),($EF,$BD,$97),
     ($EF,$BD,$98),($EF,$BD,$99),($EF,$BD,$9A),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($E3,$81,$81),
     ($E3,$81,$82),($E3,$81,$83),($E3,$81,$84),($E3,$81,$85),
     ($E3,$81,$86),($E3,$81,$87),($E3,$81,$88),($E3,$81,$89),
     ($E3,$81,$8A),($E3,$81,$8B),($E3,$81,$8C),($E3,$81,$8D),
     ($E3,$81,$8E),($E3,$81,$8F),($E3,$81,$90),($E3,$81,$91),
     ($E3,$81,$92),($E3,$81,$93),($E3,$81,$94),($E3,$81,$95),
     ($E3,$81,$96),($E3,$81,$97),($E3,$81,$98),($E3,$81,$99),
     ($E3,$81,$9A),($E3,$81,$9B),($E3,$81,$9C),($E3,$81,$9D),
     ($E3,$81,$9E),($E3,$81,$9F),($E3,$81,$A0),($E3,$81,$A1),
     ($E3,$81,$A2),($E3,$81,$A3),($E3,$81,$A4),($E3,$81,$A5),
     ($E3,$81,$A6),($E3,$81,$A7),($E3,$81,$A8),($E3,$81,$A9),
     ($E3,$81,$AA),($E3,$81,$AB),($E3,$81,$AC),($E3,$81,$AD),
     ($E3,$81,$AE),($E3,$81,$AF),($E3,$81,$B0),($E3,$81,$B1),
     ($E3,$81,$B2),($E3,$81,$B3),($E3,$81,$B4),($E3,$81,$B5),
     ($E3,$81,$B6),($E3,$81,$B7),($E3,$81,$B8),($E3,$81,$B9),
     ($E3,$81,$BA),($E3,$81,$BB),($E3,$81,$BC),($E3,$81,$BD),
     ($E3,$81,$BE),($E3,$81,$BF),($E3,$82,$80),($E3,$82,$81),
     ($E3,$82,$82),($E3,$82,$83),($E3,$82,$84),($E3,$82,$85),
     ($E3,$82,$86),($E3,$82,$87),($E3,$82,$88),($E3,$82,$89),
     ($E3,$82,$8A),($E3,$82,$8B),($E3,$82,$8C),($E3,$82,$8D),
     ($E3,$82,$8E),($E3,$82,$8F),($E3,$82,$90),($E3,$82,$91),
     ($E3,$82,$92),($E3,$82,$93),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00));

  S2U_83: TSjisRowTable =
    (($E3,$82,$A1),($E3,$82,$A2),($E3,$82,$A3),($E3,$82,$A4),
     ($E3,$82,$A5),($E3,$82,$A6),($E3,$82,$A7),($E3,$82,$A8),
     ($E3,$82,$A9),($E3,$82,$AA),($E3,$82,$AB),($E3,$82,$AC),
     ($E3,$82,$AD),($E3,$82,$AE),($E3,$82,$AF),($E3,$82,$B0),
     ($E3,$82,$B1),($E3,$82,$B2),($E3,$82,$B3),($E3,$82,$B4),
     ($E3,$82,$B5),($E3,$82,$B6),($E3,$82,$B7),($E3,$82,$B8),
     ($E3,$82,$B9),($E3,$82,$BA),($E3,$82,$BB),($E3,$82,$BC),
     ($E3,$82,$BD),($E3,$82,$BE),($E3,$82,$BF),($E3,$83,$80),
     ($E3,$83,$81),($E3,$83,$82),($E3,$83,$83),($E3,$83,$84),
     ($E3,$83,$85),($E3,$83,$86),($E3,$83,$87),($E3,$83,$88),
     ($E3,$83,$89),($E3,$83,$8A),($E3,$83,$8B),($E3,$83,$8C),
     ($E3,$83,$8D),($E3,$83,$8E),($E3,$83,$8F),($E3,$83,$90),
     ($E3,$83,$91),($E3,$83,$92),($E3,$83,$93),($E3,$83,$94),
     ($E3,$83,$95),($E3,$83,$96),($E3,$83,$97),($E3,$83,$98),
     ($E3,$83,$99),($E3,$83,$9A),($E3,$83,$9B),($E3,$83,$9C),
     ($E3,$83,$9D),($E3,$83,$9E),($E3,$83,$9F),($00,$00,$00),
     ($E3,$83,$A0),($E3,$83,$A1),($E3,$83,$A2),($E3,$83,$A3),
     ($E3,$83,$A4),($E3,$83,$A5),($E3,$83,$A6),($E3,$83,$A7),
     ($E3,$83,$A8),($E3,$83,$A9),($E3,$83,$AA),($E3,$83,$AB),
     ($E3,$83,$AC),($E3,$83,$AD),($E3,$83,$AE),($E3,$83,$AF),
     ($E3,$83,$B0),($E3,$83,$B1),($E3,$83,$B2),($E3,$83,$B3),
     ($E3,$83,$B4),($E3,$83,$B5),($E3,$83,$B6),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($CE,$91,$00),
     ($CE,$92,$00),($CE,$93,$00),($CE,$94,$00),($CE,$95,$00),
     ($CE,$96,$00),($CE,$97,$00),($CE,$98,$00),($CE,$99,$00),
     ($CE,$9A,$00),($CE,$9B,$00),($CE,$9C,$00),($CE,$9D,$00),
     ($CE,$9E,$00),($CE,$9F,$00),($CE,$A0,$00),($CE,$A1,$00),
     ($CE,$A3,$00),($CE,$A4,$00),($CE,$A5,$00),($CE,$A6,$00),
     ($CE,$A7,$00),($CE,$A8,$00),($CE,$A9,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($CE,$B1,$00),
     ($CE,$B2,$00),($CE,$B3,$00),($CE,$B4,$00),($CE,$B5,$00),
     ($CE,$B6,$00),($CE,$B7,$00),($CE,$B8,$00),($CE,$B9,$00),
     ($CE,$BA,$00),($CE,$BB,$00),($CE,$BC,$00),($CE,$BD,$00),
     ($CE,$BE,$00),($CE,$BF,$00),($CF,$80,$00),($CF,$81,$00),
     ($CF,$83,$00),($CF,$84,$00),($CF,$85,$00),($CF,$86,$00),
     ($CF,$87,$00),($CF,$88,$00),($CF,$89,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00));

  S2U_84: TSjisRowTable =
    (($D0,$90,$00),($D0,$91,$00),($D0,$92,$00),($D0,$93,$00),
     ($D0,$94,$00),($D0,$95,$00),($D0,$81,$00),($D0,$96,$00),
     ($D0,$97,$00),($D0,$98,$00),($D0,$99,$00),($D0,$9A,$00),
     ($D0,$9B,$00),($D0,$9C,$00),($D0,$9D,$00),($D0,$9E,$00),
     ($D0,$9F,$00),($D0,$A0,$00),($D0,$A1,$00),($D0,$A2,$00),
     ($D0,$A3,$00),($D0,$A4,$00),($D0,$A5,$00),($D0,$A6,$00),
     ($D0,$A7,$00),($D0,$A8,$00),($D0,$A9,$00),($D0,$AA,$00),
     ($D0,$AB,$00),($D0,$AC,$00),($D0,$AD,$00),($D0,$AE,$00),
     ($D0,$AF,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($D0,$B0,$00),($D0,$B1,$00),($D0,$B2,$00),($D0,$B3,$00),
     ($D0,$B4,$00),($D0,$B5,$00),($D1,$91,$00),($D0,$B6,$00),
     ($D0,$B7,$00),($D0,$B8,$00),($D0,$B9,$00),($D0,$BA,$00),
     ($D0,$BB,$00),($D0,$BC,$00),($D0,$BD,$00),($00,$00,$00),
     ($D0,$BE,$00),($D0,$BF,$00),($D1,$80,$00),($D1,$81,$00),
     ($D1,$82,$00),($D1,$83,$00),($D1,$84,$00),($D1,$85,$00),
     ($D1,$86,$00),($D1,$87,$00),($D1,$88,$00),($D1,$89,$00),
     ($D1,$8A,$00),($D1,$8B,$00),($D1,$8C,$00),($D1,$8D,$00),
     ($D1,$8E,$00),($D1,$8F,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($E2,$94,$80),
     ($E2,$94,$82),($E2,$94,$8C),($E2,$94,$90),($E2,$94,$98),
     ($E2,$94,$94),($E2,$94,$9C),($E2,$94,$AC),($E2,$94,$A4),
     ($E2,$94,$B4),($E2,$94,$BC),($E2,$94,$81),($E2,$94,$83),
     ($E2,$94,$8F),($E2,$94,$93),($E2,$94,$9B),($E2,$94,$97),
     ($E2,$94,$A3),($E2,$94,$B3),($E2,$94,$AB),($E2,$94,$BB),
     ($E2,$95,$8B),($E2,$94,$A0),($E2,$94,$AF),($E2,$94,$A8),
     ($E2,$94,$B7),($E2,$94,$BF),($E2,$94,$9D),($E2,$94,$B0),
     ($E2,$94,$A5),($E2,$94,$B8),($E2,$95,$82),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00));

  S2U_88: TSjisRowTable =
    (($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($E4,$BA,$9C),
     ($E5,$94,$96),($E5,$A8,$83),($E9,$98,$BF),($E5,$93,$80),
     ($E6,$84,$9B),($E6,$8C,$A8),($E5,$A7,$B6),($E9,$80,$A2),
     ($E8,$91,$B5),($E8,$8C,$9C),($E7,$A9,$90),($E6,$82,$AA),
     ($E6,$8F,$A1),($E6,$B8,$A5),($E6,$97,$AD),($E8,$91,$A6),
     ($E8,$8A,$A6),($E9,$AF,$B5),($E6,$A2,$93),($E5,$9C,$A7),
     ($E6,$96,$A1),($E6,$89,$B1),($E5,$AE,$9B),($E5,$A7,$90),
     ($E8,$99,$BB),($E9,$A3,$B4),($E7,$B5,$A2),($E7,$B6,$BE),
     ($E9,$AE,$8E),($E6,$88,$96),($E7,$B2,$9F),($E8,$A2,$B7),
     ($E5,$AE,$89),($E5,$BA,$B5),($E6,$8C,$89),($E6,$9A,$97),
     ($E6,$A1,$88),($E9,$97,$87),($E9,$9E,$8D),($E6,$9D,$8F),
     ($E4,$BB,$A5),($E4,$BC,$8A),($E4,$BD,$8D),($E4,$BE,$9D),
     ($E5,$81,$89),($E5,$9B,$B2),($E5,$A4,$B7),($E5,$A7,$94),
     ($E5,$A8,$81),($E5,$B0,$89),($E6,$83,$9F),($E6,$84,$8F),
     ($E6,$85,$B0),($E6,$98,$93),($E6,$A4,$85),($E7,$82,$BA),
     ($E7,$95,$8F),($E7,$95,$B0),($E7,$A7,$BB),($E7,$B6,$AD),
     ($E7,$B7,$AF),($E8,$83,$83),($E8,$90,$8E),($E8,$A1,$A3),
     ($E8,$AC,$82),($E9,$81,$95),($E9,$81,$BA),($E5,$8C,$BB),
     ($E4,$BA,$95),($E4,$BA,$A5),($E5,$9F,$9F),($E8,$82,$B2),
     ($E9,$83,$81),($E7,$A3,$AF),($E4,$B8,$80),($E5,$A3,$B1),
     ($E6,$BA,$A2),($E9,$80,$B8),($E7,$A8,$B2),($E8,$8C,$A8),
     ($E8,$8A,$8B),($E9,$B0,$AF),($E5,$85,$81),($E5,$8D,$B0),
     ($E5,$92,$BD),($E5,$93,$A1),($E5,$9B,$A0),($E5,$A7,$BB),
     ($E5,$BC,$95),($E9,$A3,$B2),($E6,$B7,$AB),($E8,$83,$A4),
     ($E8,$94,$AD));

  S2U_89: TSjisRowTable =
    (($E9,$99,$A2),($E9,$99,$B0),($E9,$9A,$A0),($E9,$9F,$BB),
     ($E5,$90,$8B),($E5,$8F,$B3),($E5,$AE,$87),($E7,$83,$8F),
     ($E7,$BE,$BD),($E8,$BF,$82),($E9,$9B,$A8),($E5,$8D,$AF),
     ($E9,$B5,$9C),($E7,$AA,$BA),($E4,$B8,$91),($E7,$A2,$93),
     ($E8,$87,$BC),($E6,$B8,$A6),($E5,$98,$98),($E5,$94,$84),
     ($E6,$AC,$9D),($E8,$94,$9A),($E9,$B0,$BB),($E5,$A7,$A5),
     ($E5,$8E,$A9),($E6,$B5,$A6),($E7,$93,$9C),($E9,$96,$8F),
     ($E5,$99,$82),($E4,$BA,$91),($E9,$81,$8B),($E9,$9B,$B2),
     ($E8,$8D,$8F),($E9,$A4,$8C),($E5,$8F,$A1),($E5,$96,$B6),
     ($E5,$AC,$B0),($E5,$BD,$B1),($E6,$98,$A0),($E6,$9B,$B3),
     ($E6,$A0,$84),($E6,$B0,$B8),($E6,$B3,$B3),($E6,$B4,$A9),
     ($E7,$91,$9B),($E7,$9B,$88),($E7,$A9,$8E),($E9,$A0,$B4),
     ($E8,$8B,$B1),($E8,$A1,$9B),($E8,$A9,$A0),($E9,$8B,$AD),
     ($E6,$B6,$B2),($E7,$96,$AB),($E7,$9B,$8A),($E9,$A7,$85),
     ($E6,$82,$A6),($E8,$AC,$81),($E8,$B6,$8A),($E9,$96,$B2),
     ($E6,$A6,$8E),($E5,$8E,$AD),($E5,$86,$86),($00,$00,$00),
     ($E5,$9C,$92),($E5,$A0,$B0),($E5,$A5,$84),($E5,$AE,$B4),
     ($E5,$BB,$B6),($E6,$80,$A8),($E6,$8E,$A9),($E6,$8F,$B4),
     ($E6,$B2,$BF),($E6,$BC,$94),($E7,$82,$8E),($E7,$84,$94),
     ($E7,$85,$99),($E7,$87,$95),($E7,$8C,$BF),($E7,$B8,$81),
     ($E8,$89,$B6),($E8,$8B,$91),($E8,$96,$97),($E9,$81,$A0),
     ($E9,$89,$9B),($E9,$B4,$9B),($E5,$A1,$A9),($E6,$96,$BC),
     ($E6,$B1,$9A),($E7,$94,$A5),($E5,$87,$B9),($E5,$A4,$AE),
     ($E5,$A5,$A5),($E5,$BE,$80),($E5,$BF,$9C),($E6,$8A,$BC),
     ($E6,$97,$BA),($E6,$A8,$AA),($E6,$AC,$A7),($E6,$AE,$B4),
     ($E7,$8E,$8B),($E7,$BF,$81),($E8,$A5,$96),($E9,$B4,$AC),
     ($E9,$B4,$8E),($E9,$BB,$84),($E5,$B2,$A1),($E6,$B2,$96),
     ($E8,$8D,$BB),($E5,$84,$84),($E5,$B1,$8B),($E6,$86,$B6),
     ($E8,$87,$86),($E6,$A1,$B6),($E7,$89,$A1),($E4,$B9,$99),
     ($E4,$BF,$BA),($E5,$8D,$B8),($E6,$81,$A9),($E6,$B8,$A9),
     ($E7,$A9,$8F),($E9,$9F,$B3),($E4,$B8,$8B),($E5,$8C,$96),
     ($E4,$BB,$AE),($E4,$BD,$95),($E4,$BC,$BD),($E4,$BE,$A1),
     ($E4,$BD,$B3),($E5,$8A,$A0),($E5,$8F,$AF),($E5,$98,$89),
     ($E5,$A4,$8F),($E5,$AB,$81),($E5,$AE,$B6),($E5,$AF,$A1),
     ($E7,$A7,$91),($E6,$9A,$87),($E6,$9E,$9C),($E6,$9E,$B6),
     ($E6,$AD,$8C),($E6,$B2,$B3),($E7,$81,$AB),($E7,$8F,$82),
     ($E7,$A6,$8D),($E7,$A6,$BE),($E7,$A8,$BC),($E7,$AE,$87),
     ($E8,$8A,$B1),($E8,$8B,$9B),($E8,$8C,$84),($E8,$8D,$B7),
     ($E8,$8F,$AF),($E8,$8F,$93),($E8,$9D,$A6),($E8,$AA,$B2),
     ($E5,$98,$A9),($E8,$B2,$A8),($E8,$BF,$A6),($E9,$81,$8E),
     ($E9,$9C,$9E),($E8,$9A,$8A),($E4,$BF,$84),($E5,$B3,$A8),
     ($E6,$88,$91),($E7,$89,$99),($E7,$94,$BB),($E8,$87,$A5),
     ($E8,$8A,$BD),($E8,$9B,$BE),($E8,$B3,$80),($E9,$9B,$85),
     ($E9,$A4,$93),($E9,$A7,$95),($E4,$BB,$8B),($E4,$BC,$9A),
     ($E8,$A7,$A3),($E5,$9B,$9E),($E5,$A1,$8A),($E5,$A3,$8A),
     ($E5,$BB,$BB),($E5,$BF,$AB),($E6,$80,$AA),($E6,$82,$94),
     ($E6,$81,$A2),($E6,$87,$90),($E6,$88,$92),($E6,$8B,$90),
     ($E6,$94,$B9));

  S2U_8A: TSjisRowTable =
    (($E9,$AD,$81),($E6,$99,$A6),($E6,$A2,$B0),($E6,$B5,$B7),
     ($E7,$81,$B0),($E7,$95,$8C),($E7,$9A,$86),($E7,$B5,$B5),
     ($E8,$8A,$A5),($E8,$9F,$B9),($E9,$96,$8B),($E9,$9A,$8E),
     ($E8,$B2,$9D),($E5,$87,$B1),($E5,$8A,$BE),($E5,$A4,$96),
     ($E5,$92,$B3),($E5,$AE,$B3),($E5,$B4,$96),($E6,$85,$A8),
     ($E6,$A6,$82),($E6,$B6,$AF),($E7,$A2,$8D),($E8,$93,$8B),
     ($E8,$A1,$97),($E8,$A9,$B2),($E9,$8E,$A7),($E9,$AA,$B8),
     ($E6,$B5,$AC),($E9,$A6,$A8),($E8,$9B,$99),($E5,$9E,$A3),
     ($E6,$9F,$BF),($E8,$9B,$8E),($E9,$88,$8E),($E5,$8A,$83),
     ($E5,$9A,$87),($E5,$90,$84),($E5,$BB,$93),($E6,$8B,$A1),
     ($E6,$92,$B9),($E6,$A0,$BC),($E6,$A0,$B8),($E6,$AE,$BB),
     ($E7,$8D,$B2),($E7,$A2,$BA),($E7,$A9,$AB),($E8,$A6,$9A),
     ($E8,$A7,$92),($E8,$B5,$AB),($E8,$BC,$83),($E9,$83,$AD),
     ($E9,$96,$A3),($E9,$9A,$94),($E9,$9D,$A9),($E5,$AD,$A6),
     ($E5,$B2,$B3),($E6,$A5,$BD),($E9,$A1,$8D),($E9,$A1,$8E),
     ($E6,$8E,$9B),($E7,$AC,$A0),($E6,$A8,$AB),($00,$00,$00),
     ($E6,$A9,$BF),($E6,$A2,$B6),($E9,$B0,$8D),($E6,$BD,$9F),
     ($E5,$89,$B2),($E5,$96,$9D),($E6,$81,$B0),($E6,$8B,$AC),
     ($E6,$B4,$BB),($E6,$B8,$87),($E6,$BB,$91),($E8,$91,$9B),
     ($E8,$A4,$90),($E8,$BD,$84),($E4,$B8,$94),($E9,$B0,$B9),
     ($E5,$8F,$B6),($E6,$A4,$9B),($E6,$A8,$BA),($E9,$9E,$84),
     ($E6,$A0,$AA),($E5,$85,$9C),($E7,$AB,$83),($E8,$92,$B2),
     ($E9,$87,$9C),($E9,$8E,$8C),($E5,$99,$9B),($E9,$B4,$A8),
     ($E6,$A0,$A2),($E8,$8C,$85),($E8,$90,$B1),($E7,$B2,$A5),
     ($E5,$88,$88),($E8,$8B,$85),($E7,$93,$A6),($E4,$B9,$BE),
     ($E4,$BE,$83),($E5,$86,$A0),($E5,$AF,$92),($E5,$88,$8A),
     ($E5,$8B,$98),($E5,$8B,$A7),($E5,$B7,$BB),($E5,$96,$9A),
     ($E5,$A0,$AA),($E5,$A7,$A6),($E5,$AE,$8C),($E5,$AE,$98),
     ($E5,$AF,$9B),($E5,$B9,$B2),($E5,$B9,$B9),($E6,$82,$A3),
     ($E6,$84,$9F),($E6,$85,$A3),($E6,$86,$BE),($E6,$8F,$9B),
     ($E6,$95,$A2),($E6,$9F,$91),($E6,$A1,$93),($E6,$A3,$BA),
     ($E6,$AC,$BE),($E6,$AD,$93),($E6,$B1,$97),($E6,$BC,$A2),
     ($E6,$BE,$97),($E6,$BD,$85),($E7,$92,$B0),($E7,$94,$98),
     ($E7,$9B,$A3),($E7,$9C,$8B),($E7,$AB,$BF),($E7,$AE,$A1),
     ($E7,$B0,$A1),($E7,$B7,$A9),($E7,$BC,$B6),($E7,$BF,$B0),
     ($E8,$82,$9D),($E8,$89,$A6),($E8,$8E,$9E),($E8,$A6,$B3),
     ($E8,$AB,$8C),($E8,$B2,$AB),($E9,$82,$84),($E9,$91,$91),
     ($E9,$96,$93),($E9,$96,$91),($E9,$96,$A2),($E9,$99,$A5),
     ($E9,$9F,$93),($E9,$A4,$A8),($E8,$88,$98),($E4,$B8,$B8),
     ($E5,$90,$AB),($E5,$B2,$B8),($E5,$B7,$8C),($E7,$8E,$A9),
     ($E7,$99,$8C),($E7,$9C,$BC),($E5,$B2,$A9),($E7,$BF,$AB),
     ($E8,$B4,$8B),($E9,$9B,$81),($E9,$A0,$91),($E9,$A1,$94),
     ($E9,$A1,$98),($E4,$BC,$81),($E4,$BC,$8E),($E5,$8D,$B1),
     ($E5,$96,$9C),($E5,$99,$A8),($E5,$9F,$BA),($E5,$A5,$87),
     ($E5,$AC,$89),($E5,$AF,$84),($E5,$B2,$90),($E5,$B8,$8C),
     ($E5,$B9,$BE),($E5,$BF,$8C),($E6,$8F,$AE),($E6,$9C,$BA),
     ($E6,$97,$97),($E6,$97,$A2),($E6,$9C,$9F),($E6,$A3,$8B),
     ($E6,$A3,$84));

  S2U_8B: TSjisRowTable =
    (($E6,$A9,$9F),($E5,$B8,$B0),($E6,$AF,$85),($E6,$B0,$97),
     ($E6,$B1,$BD),($E7,$95,$BF),($E7,$A5,$88),($E5,$AD,$A3),
     ($E7,$A8,$80),($E7,$B4,$80),($E5,$BE,$BD),($E8,$A6,$8F),
     ($E8,$A8,$98),($E8,$B2,$B4),($E8,$B5,$B7),($E8,$BB,$8C),
     ($E8,$BC,$9D),($E9,$A3,$A2),($E9,$A8,$8E),($E9,$AC,$BC),
     ($E4,$BA,$80),($E5,$81,$BD),($E5,$84,$80),($E5,$A6,$93),
     ($E5,$AE,$9C),($E6,$88,$AF),($E6,$8A,$80),($E6,$93,$AC),
     ($E6,$AC,$BA),($E7,$8A,$A0),($E7,$96,$91),($E7,$A5,$87),
     ($E7,$BE,$A9),($E8,$9F,$BB),($E8,$AA,$BC),($E8,$AD,$B0),
     ($E6,$8E,$AC),($E8,$8F,$8A),($E9,$9E,$A0),($E5,$90,$89),
     ($E5,$90,$83),($E5,$96,$AB),($E6,$A1,$94),($E6,$A9,$98),
     ($E8,$A9,$B0),($E7,$A0,$A7),($E6,$9D,$B5),($E9,$BB,$8D),
     ($E5,$8D,$B4),($E5,$AE,$A2),($E8,$84,$9A),($E8,$99,$90),
     ($E9,$80,$86),($E4,$B8,$98),($E4,$B9,$85),($E4,$BB,$87),
     ($E4,$BC,$91),($E5,$8F,$8A),($E5,$90,$B8),($E5,$AE,$AE),
     ($E5,$BC,$93),($E6,$80,$A5),($E6,$95,$91),($00,$00,$00),
     ($E6,$9C,$BD),($E6,$B1,$82),($E6,$B1,$B2),($E6,$B3,$A3),
     ($E7,$81,$B8),($E7,$90,$83),($E7,$A9,$B6),($E7,$AA,$AE),
     ($E7,$AC,$88),($E7,$B4,$9A),($E7,$B3,$BE),($E7,$B5,$A6),
     ($E6,$97,$A7),($E7,$89,$9B),($E5,$8E,$BB),($E5,$B1,$85),
     ($E5,$B7,$A8),($E6,$8B,$92),($E6,$8B,$A0),($E6,$8C,$99),
     ($E6,$B8,$A0),($E8,$99,$9A),($E8,$A8,$B1),($E8,$B7,$9D),
     ($E9,$8B,$B8),($E6,$BC,$81),($E7,$A6,$A6),($E9,$AD,$9A),
     ($E4,$BA,$A8),($E4,$BA,$AB),($E4,$BA,$AC),($E4,$BE,$9B),
     ($E4,$BE,$A0),($E5,$83,$91),($E5,$85,$87),($E7,$AB,$B6),
     ($E5,$85,$B1),($E5,$87,$B6),($E5,$8D,$94),($E5,$8C,$A1),
     ($E5,$8D,$BF),($E5,$8F,$AB),($E5,$96,$AC),($E5,$A2,$83),
     ($E5,$B3,$A1),($E5,$BC,$B7),($E5,$BD,$8A),($E6,$80,$AF),
     ($E6,$81,$90),($E6,$81,$AD),($E6,$8C,$9F),($E6,$95,$99),
     ($E6,$A9,$8B),($E6,$B3,$81),($E7,$8B,$82),($E7,$8B,$AD),
     ($E7,$9F,$AF),($E8,$83,$B8),($E8,$84,$85),($E8,$88,$88),
     ($E8,$95,$8E),($E9,$83,$B7),($E9,$8F,$A1),($E9,$9F,$BF),
     ($E9,$A5,$97),($E9,$A9,$9A),($E4,$BB,$B0),($E5,$87,$9D),
     ($E5,$B0,$AD),($E6,$9A,$81),($E6,$A5,$AD),($E5,$B1,$80),
     ($E6,$9B,$B2),($E6,$A5,$B5),($E7,$8E,$89),($E6,$A1,$90),
     ($E7,$B2,$81),($E5,$83,$85),($E5,$8B,$A4),($E5,$9D,$87),
     ($E5,$B7,$BE),($E9,$8C,$A6),($E6,$96,$A4),($E6,$AC,$A3),
     ($E6,$AC,$BD),($E7,$90,$B4),($E7,$A6,$81),($E7,$A6,$BD),
     ($E7,$AD,$8B),($E7,$B7,$8A),($E8,$8A,$B9),($E8,$8F,$8C),
     ($E8,$A1,$BF),($E8,$A5,$9F),($E8,$AC,$B9),($E8,$BF,$91),
     ($E9,$87,$91),($E5,$90,$9F),($E9,$8A,$80),($E4,$B9,$9D),
     ($E5,$80,$B6),($E5,$8F,$A5),($E5,$8C,$BA),($E7,$8B,$97),
     ($E7,$8E,$96),($E7,$9F,$A9),($E8,$8B,$A6),($E8,$BA,$AF),
     ($E9,$A7,$86),($E9,$A7,$88),($E9,$A7,$92),($E5,$85,$B7),
     ($E6,$84,$9A),($E8,$99,$9E),($E5,$96,$B0),($E7,$A9,$BA),
     ($E5,$81,$B6),($E5,$AF,$93),($E9,$81,$87),($E9,$9A,$85),
     ($E4,$B8,$B2),($E6,$AB,$9B),($E9,$87,$A7),($E5,$B1,$91),
     ($E5,$B1,$88));

  S2U_8C: TSjisRowTable =
    (($E6,$8E,$98),($E7,$AA,$9F),($E6,$B2,$93),($E9,$9D,$B4),
     ($E8,$BD,$A1),($E7,$AA,$AA),($E7,$86,$8A),($E9,$9A,$88),
     ($E7,$B2,$82),($E6,$A0,$97),($E7,$B9,$B0),($E6,$A1,$91),
     ($E9,$8D,$AC),($E5,$8B,$B2),($E5,$90,$9B),($E8,$96,$AB),
     ($E8,$A8,$93),($E7,$BE,$A4),($E8,$BB,$8D),($E9,$83,$A1),
     ($E5,$8D,$A6),($E8,$A2,$88),($E7,$A5,$81),($E4,$BF,$82),
     ($E5,$82,$BE),($E5,$88,$91),($E5,$85,$84),($E5,$95,$93),
     ($E5,$9C,$AD),($E7,$8F,$AA),($E5,$9E,$8B),($E5,$A5,$91),
     ($E5,$BD,$A2),($E5,$BE,$84),($E6,$81,$B5),($E6,$85,$B6),
     ($E6,$85,$A7),($E6,$86,$A9),($E6,$8E,$B2),($E6,$90,$BA),
     ($E6,$95,$AC),($E6,$99,$AF),($E6,$A1,$82),($E6,$B8,$93),
     ($E7,$95,$A6),($E7,$A8,$BD),($E7,$B3,$BB),($E7,$B5,$8C),
     ($E7,$B6,$99),($E7,$B9,$8B),($E7,$BD,$AB),($E8,$8C,$8E),
     ($E8,$8D,$8A),($E8,$9B,$8D),($E8,$A8,$88),($E8,$A9,$A3),
     ($E8,$AD,$A6),($E8,$BB,$BD),($E9,$A0,$9A),($E9,$B6,$8F),
     ($E8,$8A,$B8),($E8,$BF,$8E),($E9,$AF,$A8),($00,$00,$00),
     ($E5,$8A,$87),($E6,$88,$9F),($E6,$92,$83),($E6,$BF,$80),
     ($E9,$9A,$99),($E6,$A1,$81),($E5,$82,$91),($E6,$AC,$A0),
     ($E6,$B1,$BA),($E6,$BD,$94),($E7,$A9,$B4),($E7,$B5,$90),
     ($E8,$A1,$80),($E8,$A8,$A3),($E6,$9C,$88),($E4,$BB,$B6),
     ($E5,$80,$B9),($E5,$80,$A6),($E5,$81,$A5),($E5,$85,$BC),
     ($E5,$88,$B8),($E5,$89,$A3),($E5,$96,$A7),($E5,$9C,$8F),
     ($E5,$A0,$85),($E5,$AB,$8C),($E5,$BB,$BA),($E6,$86,$B2),
     ($E6,$87,$B8),($E6,$8B,$B3),($E6,$8D,$B2),($E6,$A4,$9C),
     ($E6,$A8,$A9),($E7,$89,$BD),($E7,$8A,$AC),($E7,$8C,$AE),
     ($E7,$A0,$94),($E7,$A1,$AF),($E7,$B5,$B9),($E7,$9C,$8C),
     ($E8,$82,$A9),($E8,$A6,$8B),($E8,$AC,$99),($E8,$B3,$A2),
     ($E8,$BB,$92),($E9,$81,$A3),($E9,$8D,$B5),($E9,$99,$BA),
     ($E9,$A1,$95),($E9,$A8,$93),($E9,$B9,$B8),($E5,$85,$83),
     ($E5,$8E,$9F),($E5,$8E,$B3),($E5,$B9,$BB),($E5,$BC,$A6),
     ($E6,$B8,$9B),($E6,$BA,$90),($E7,$8E,$84),($E7,$8F,$BE),
     ($E7,$B5,$83),($E8,$88,$B7),($E8,$A8,$80),($E8,$AB,$BA),
     ($E9,$99,$90),($E4,$B9,$8E),($E5,$80,$8B),($E5,$8F,$A4),
     ($E5,$91,$BC),($E5,$9B,$BA),($E5,$A7,$91),($E5,$AD,$A4),
     ($E5,$B7,$B1),($E5,$BA,$AB),($E5,$BC,$A7),($E6,$88,$B8),
     ($E6,$95,$85),($E6,$9E,$AF),($E6,$B9,$96),($E7,$8B,$90),
     ($E7,$B3,$8A),($E8,$A2,$B4),($E8,$82,$A1),($E8,$83,$A1),
     ($E8,$8F,$B0),($E8,$99,$8E),($E8,$AA,$87),($E8,$B7,$A8),
     ($E9,$88,$B7),($E9,$9B,$87),($E9,$A1,$A7),($E9,$BC,$93),
     ($E4,$BA,$94),($E4,$BA,$92),($E4,$BC,$8D),($E5,$8D,$88),
     ($E5,$91,$89),($E5,$90,$BE),($E5,$A8,$AF),($E5,$BE,$8C),
     ($E5,$BE,$A1),($E6,$82,$9F),($E6,$A2,$A7),($E6,$AA,$8E),
     ($E7,$91,$9A),($E7,$A2,$81),($E8,$AA,$9E),($E8,$AA,$A4),
     ($E8,$AD,$B7),($E9,$86,$90),($E4,$B9,$9E),($E9,$AF,$89),
     ($E4,$BA,$A4),($E4,$BD,$BC),($E4,$BE,$AF),($E5,$80,$99),
     ($E5,$80,$96),($E5,$85,$89),($E5,$85,$AC),($E5,$8A,$9F),
     ($E5,$8A,$B9),($E5,$8B,$BE),($E5,$8E,$9A),($E5,$8F,$A3),
     ($E5,$90,$91));

  S2U_8D: TSjisRowTable =
    (($E5,$90,$8E),($E5,$96,$89),($E5,$9D,$91),($E5,$9E,$A2),
     ($E5,$A5,$BD),($E5,$AD,$94),($E5,$AD,$9D),($E5,$AE,$8F),
     ($E5,$B7,$A5),($E5,$B7,$A7),($E5,$B7,$B7),($E5,$B9,$B8),
     ($E5,$BA,$83),($E5,$BA,$9A),($E5,$BA,$B7),($E5,$BC,$98),
     ($E6,$81,$92),($E6,$85,$8C),($E6,$8A,$97),($E6,$8B,$98),
     ($E6,$8E,$A7),($E6,$94,$BB),($E6,$98,$82),($E6,$99,$83),
     ($E6,$9B,$B4),($E6,$9D,$AD),($E6,$A0,$A1),($E6,$A2,$97),
     ($E6,$A7,$8B),($E6,$B1,$9F),($E6,$B4,$AA),($E6,$B5,$A9),
     ($E6,$B8,$AF),($E6,$BA,$9D),($E7,$94,$B2),($E7,$9A,$87),
     ($E7,$A1,$AC),($E7,$A8,$BF),($E7,$B3,$A0),($E7,$B4,$85),
     ($E7,$B4,$98),($E7,$B5,$9E),($E7,$B6,$B1),($E8,$80,$95),
     ($E8,$80,$83),($E8,$82,$AF),($E8,$82,$B1),($E8,$85,$94),
     ($E8,$86,$8F),($E8,$88,$AA),($E8,$8D,$92),($E8,$A1,$8C),
     ($E8,$A1,$A1),($E8,$AC,$9B),($E8,$B2,$A2),($E8,$B3,$BC),
     ($E9,$83,$8A),($E9,$85,$B5),($E9,$89,$B1),($E7,$A0,$BF),
     ($E9,$8B,$BC),($E9,$96,$A4),($E9,$99,$8D),($00,$00,$00),
     ($E9,$A0,$85),($E9,$A6,$99),($E9,$AB,$98),($E9,$B4,$BB),
     ($E5,$89,$9B),($E5,$8A,$AB),($E5,$8F,$B7),($E5,$90,$88),
     ($E5,$A3,$95),($E6,$8B,$B7),($E6,$BF,$A0),($E8,$B1,$AA),
     ($E8,$BD,$9F),($E9,$BA,$B9),($E5,$85,$8B),($E5,$88,$BB),
     ($E5,$91,$8A),($E5,$9B,$BD),($E7,$A9,$80),($E9,$85,$B7),
     ($E9,$B5,$A0),($E9,$BB,$92),($E7,$8D,$84),($E6,$BC,$89),
     ($E8,$85,$B0),($E7,$94,$91),($E5,$BF,$BD),($E6,$83,$9A),
     ($E9,$AA,$A8),($E7,$8B,$9B),($E8,$BE,$BC),($E6,$AD,$A4),
     ($E9,$A0,$83),($E4,$BB,$8A),($E5,$9B,$B0),($E5,$9D,$A4),
     ($E5,$A2,$BE),($E5,$A9,$9A),($E6,$81,$A8),($E6,$87,$87),
     ($E6,$98,$8F),($E6,$98,$86),($E6,$A0,$B9),($E6,$A2,$B1),
     ($E6,$B7,$B7),($E7,$97,$95),($E7,$B4,$BA),($E8,$89,$AE),
     ($E9,$AD,$82),($E4,$BA,$9B),($E4,$BD,$90),($E5,$8F,$89),
     ($E5,$94,$86),($E5,$B5,$AF),($E5,$B7,$A6),($E5,$B7,$AE),
     ($E6,$9F,$BB),($E6,$B2,$99),($E7,$91,$B3),($E7,$A0,$82),
     ($E8,$A9,$90),($E9,$8E,$96),($E8,$A3,$9F),($E5,$9D,$90),
     ($E5,$BA,$A7),($E6,$8C,$AB),($E5,$82,$B5),($E5,$82,$AC),
     ($E5,$86,$8D),($E6,$9C,$80),($E5,$93,$89),($E5,$A1,$9E),
     ($E5,$A6,$BB),($E5,$AE,$B0),($E5,$BD,$A9),($E6,$89,$8D),
     ($E6,$8E,$A1),($E6,$A0,$BD),($E6,$AD,$B3),($E6,$B8,$88),
     ($E7,$81,$BD),($E9,$87,$87),($E7,$8A,$80),($E7,$A0,$95),
     ($E7,$A0,$A6),($E7,$A5,$AD),($E6,$96,$8E),($E7,$B4,$B0),
     ($E8,$8F,$9C),($E8,$A3,$81),($E8,$BC,$89),($E9,$9A,$9B),
     ($E5,$89,$A4),($E5,$9C,$A8),($E6,$9D,$90),($E7,$BD,$AA),
     ($E8,$B2,$A1),($E5,$86,$B4),($E5,$9D,$82),($E9,$98,$AA),
     ($E5,$A0,$BA),($E6,$A6,$8A),($E8,$82,$B4),($E5,$92,$B2),
     ($E5,$B4,$8E),($E5,$9F,$BC),($E7,$A2,$95),($E9,$B7,$BA),
     ($E4,$BD,$9C),($E5,$89,$8A),($E5,$92,$8B),($E6,$90,$BE),
     ($E6,$98,$A8),($E6,$9C,$94),($E6,$9F,$B5),($E7,$AA,$84),
     ($E7,$AD,$96),($E7,$B4,$A2),($E9,$8C,$AF),($E6,$A1,$9C),
     ($E9,$AE,$AD),($E7,$AC,$B9),($E5,$8C,$99),($E5,$86,$8A),
     ($E5,$88,$B7));

  S2U_8E: TSjisRowTable =
    (($E5,$AF,$9F),($E6,$8B,$B6),($E6,$92,$AE),($E6,$93,$A6),
     ($E6,$9C,$AD),($E6,$AE,$BA),($E8,$96,$A9),($E9,$9B,$91),
     ($E7,$9A,$90),($E9,$AF,$96),($E6,$8D,$8C),($E9,$8C,$86),
     ($E9,$AE,$AB),($E7,$9A,$BF),($E6,$99,$92),($E4,$B8,$89),
     ($E5,$82,$98),($E5,$8F,$82),($E5,$B1,$B1),($E6,$83,$A8),
     ($E6,$92,$92),($E6,$95,$A3),($E6,$A1,$9F),($E7,$87,$A6),
     ($E7,$8F,$8A),($E7,$94,$A3),($E7,$AE,$97),($E7,$BA,$82),
     ($E8,$9A,$95),($E8,$AE,$83),($E8,$B3,$9B),($E9,$85,$B8),
     ($E9,$A4,$90),($E6,$96,$AC),($E6,$9A,$AB),($E6,$AE,$8B),
     ($E4,$BB,$95),($E4,$BB,$94),($E4,$BC,$BA),($E4,$BD,$BF),
     ($E5,$88,$BA),($E5,$8F,$B8),($E5,$8F,$B2),($E5,$97,$A3),
     ($E5,$9B,$9B),($E5,$A3,$AB),($E5,$A7,$8B),($E5,$A7,$89),
     ($E5,$A7,$BF),($E5,$AD,$90),($E5,$B1,$8D),($E5,$B8,$82),
     ($E5,$B8,$AB),($E5,$BF,$97),($E6,$80,$9D),($E6,$8C,$87),
     ($E6,$94,$AF),($E5,$AD,$9C),($E6,$96,$AF),($E6,$96,$BD),
     ($E6,$97,$A8),($E6,$9E,$9D),($E6,$AD,$A2),($00,$00,$00),
     ($E6,$AD,$BB),($E6,$B0,$8F),($E7,$8D,$85),($E7,$A5,$89),
     ($E7,$A7,$81),($E7,$B3,$B8),($E7,$B4,$99),($E7,$B4,$AB),
     ($E8,$82,$A2),($E8,$84,$82),($E8,$87,$B3),($E8,$A6,$96),
     ($E8,$A9,$9E),($E8,$A9,$A9),($E8,$A9,$A6),($E8,$AA,$8C),
     ($E8,$AB,$AE),($E8,$B3,$87),($E8,$B3,$9C),($E9,$9B,$8C),
     ($E9,$A3,$BC),($E6,$AD,$AF),($E4,$BA,$8B),($E4,$BC,$BC),
     ($E4,$BE,$8D),($E5,$85,$90),($E5,$AD,$97),($E5,$AF,$BA),
     ($E6,$85,$88),($E6,$8C,$81),($E6,$99,$82),($E6,$AC,$A1),
     ($E6,$BB,$8B),($E6,$B2,$BB),($E7,$88,$BE),($E7,$92,$BD),
     ($E7,$97,$94),($E7,$A3,$81),($E7,$A4,$BA),($E8,$80,$8C),
     ($E8,$80,$B3),($E8,$87,$AA),($E8,$92,$94),($E8,$BE,$9E),
     ($E6,$B1,$90),($E9,$B9,$BF),($E5,$BC,$8F),($E8,$AD,$98),
     ($E9,$B4,$AB),($E7,$AB,$BA),($E8,$BB,$B8),($E5,$AE,$8D),
     ($E9,$9B,$AB),($E4,$B8,$83),($E5,$8F,$B1),($E5,$9F,$B7),
     ($E5,$A4,$B1),($E5,$AB,$89),($E5,$AE,$A4),($E6,$82,$89),
     ($E6,$B9,$BF),($E6,$BC,$86),($E7,$96,$BE),($E8,$B3,$AA),
     ($E5,$AE,$9F),($E8,$94,$80),($E7,$AF,$A0),($E5,$81,$B2),
     ($E6,$9F,$B4),($E8,$8A,$9D),($E5,$B1,$A1),($E8,$95,$8A),
     ($E7,$B8,$9E),($E8,$88,$8E),($E5,$86,$99),($E5,$B0,$84),
     ($E6,$8D,$A8),($E8,$B5,$A6),($E6,$96,$9C),($E7,$85,$AE),
     ($E7,$A4,$BE),($E7,$B4,$97),($E8,$80,$85),($E8,$AC,$9D),
     ($E8,$BB,$8A),($E9,$81,$AE),($E8,$9B,$87),($E9,$82,$AA),
     ($E5,$80,$9F),($E5,$8B,$BA),($E5,$B0,$BA),($E6,$9D,$93),
     ($E7,$81,$BC),($E7,$88,$B5),($E9,$85,$8C),($E9,$87,$88),
     ($E9,$8C,$AB),($E8,$8B,$A5),($E5,$AF,$82),($E5,$BC,$B1),
     ($E6,$83,$B9),($E4,$B8,$BB),($E5,$8F,$96),($E5,$AE,$88),
     ($E6,$89,$8B),($E6,$9C,$B1),($E6,$AE,$8A),($E7,$8B,$A9),
     ($E7,$8F,$A0),($E7,$A8,$AE),($E8,$85,$AB),($E8,$B6,$A3),
     ($E9,$85,$92),($E9,$A6,$96),($E5,$84,$92),($E5,$8F,$97),
     ($E5,$91,$AA),($E5,$AF,$BF),($E6,$8E,$88),($E6,$A8,$B9),
     ($E7,$B6,$AC),($E9,$9C,$80),($E5,$9B,$9A),($E5,$8F,$8E),
     ($E5,$91,$A8));

  S2U_8F: TSjisRowTable =
    (($E5,$AE,$97),($E5,$B0,$B1),($E5,$B7,$9E),($E4,$BF,$AE),
     ($E6,$84,$81),($E6,$8B,$BE),($E6,$B4,$B2),($E7,$A7,$80),
     ($E7,$A7,$8B),($E7,$B5,$82),($E7,$B9,$8D),($E7,$BF,$92),
     ($E8,$87,$AD),($E8,$88,$9F),($E8,$92,$90),($E8,$A1,$86),
     ($E8,$A5,$B2),($E8,$AE,$90),($E8,$B9,$B4),($E8,$BC,$AF),
     ($E9,$80,$B1),($E9,$85,$8B),($E9,$85,$AC),($E9,$9B,$86),
     ($E9,$86,$9C),($E4,$BB,$80),($E4,$BD,$8F),($E5,$85,$85),
     ($E5,$8D,$81),($E5,$BE,$93),($E6,$88,$8E),($E6,$9F,$94),
     ($E6,$B1,$81),($E6,$B8,$8B),($E7,$8D,$A3),($E7,$B8,$A6),
     ($E9,$87,$8D),($E9,$8A,$83),($E5,$8F,$94),($E5,$A4,$99),
     ($E5,$AE,$BF),($E6,$B7,$91),($E7,$A5,$9D),($E7,$B8,$AE),
     ($E7,$B2,$9B),($E5,$A1,$BE),($E7,$86,$9F),($E5,$87,$BA),
     ($E8,$A1,$93),($E8,$BF,$B0),($E4,$BF,$8A),($E5,$B3,$BB),
     ($E6,$98,$A5),($E7,$9E,$AC),($E7,$AB,$A3),($E8,$88,$9C),
     ($E9,$A7,$BF),($E5,$87,$86),($E5,$BE,$AA),($E6,$97,$AC),
     ($E6,$A5,$AF),($E6,$AE,$89),($E6,$B7,$B3),($00,$00,$00),
     ($E6,$BA,$96),($E6,$BD,$A4),($E7,$9B,$BE),($E7,$B4,$94),
     ($E5,$B7,$A1),($E9,$81,$B5),($E9,$86,$87),($E9,$A0,$86),
     ($E5,$87,$A6),($E5,$88,$9D),($E6,$89,$80),($E6,$9A,$91),
     ($E6,$9B,$99),($E6,$B8,$9A),($E5,$BA,$B6),($E7,$B7,$92),
     ($E7,$BD,$B2),($E6,$9B,$B8),($E8,$96,$AF),($E8,$97,$B7),
     ($E8,$AB,$B8),($E5,$8A,$A9),($E5,$8F,$99),($E5,$A5,$B3),
     ($E5,$BA,$8F),($E5,$BE,$90),($E6,$81,$95),($E9,$8B,$A4),
     ($E9,$99,$A4),($E5,$82,$B7),($E5,$84,$9F),($E5,$8B,$9D),
     ($E5,$8C,$A0),($E5,$8D,$87),($E5,$8F,$AC),($E5,$93,$A8),
     ($E5,$95,$86),($E5,$94,$B1),($E5,$98,$97),($E5,$A5,$A8),
     ($E5,$A6,$BE),($E5,$A8,$BC),($E5,$AE,$B5),($E5,$B0,$86),
     ($E5,$B0,$8F),($E5,$B0,$91),($E5,$B0,$9A),($E5,$BA,$84),
     ($E5,$BA,$8A),($E5,$BB,$A0),($E5,$BD,$B0),($E6,$89,$BF),
     ($E6,$8A,$84),($E6,$8B,$9B),($E6,$8E,$8C),($E6,$8D,$B7),
     ($E6,$98,$87),($E6,$98,$8C),($E6,$98,$AD),($E6,$99,$B6),
     ($E6,$9D,$BE),($E6,$A2,$A2),($E6,$A8,$9F),($E6,$A8,$B5),
     ($E6,$B2,$BC),($E6,$B6,$88),($E6,$B8,$89),($E6,$B9,$98),
     ($E7,$84,$BC),($E7,$84,$A6),($E7,$85,$A7),($E7,$97,$87),
     ($E7,$9C,$81),($E7,$A1,$9D),($E7,$A4,$81),($E7,$A5,$A5),
     ($E7,$A7,$B0),($E7,$AB,$A0),($E7,$AC,$91),($E7,$B2,$A7),
     ($E7,$B4,$B9),($E8,$82,$96),($E8,$8F,$96),($E8,$92,$8B),
     ($E8,$95,$89),($E8,$A1,$9D),($E8,$A3,$B3),($E8,$A8,$9F),
     ($E8,$A8,$BC),($E8,$A9,$94),($E8,$A9,$B3),($E8,$B1,$A1),
     ($E8,$B3,$9E),($E9,$86,$A4),($E9,$89,$A6),($E9,$8D,$BE),
     ($E9,$90,$98),($E9,$9A,$9C),($E9,$9E,$98),($E4,$B8,$8A),
     ($E4,$B8,$88),($E4,$B8,$9E),($E4,$B9,$97),($E5,$86,$97),
     ($E5,$89,$B0),($E5,$9F,$8E),($E5,$A0,$B4),($E5,$A3,$8C),
     ($E5,$AC,$A2),($E5,$B8,$B8),($E6,$83,$85),($E6,$93,$BE),
     ($E6,$9D,$A1),($E6,$9D,$96),($E6,$B5,$84),($E7,$8A,$B6),
     ($E7,$95,$B3),($E7,$A9,$A3),($E8,$92,$B8),($E8,$AD,$B2),
     ($E9,$86,$B8),($E9,$8C,$A0),($E5,$98,$B1),($E5,$9F,$B4),
     ($E9,$A3,$BE));

  S2U_90: TSjisRowTable =
    (($E6,$8B,$AD),($E6,$A4,$8D),($E6,$AE,$96),($E7,$87,$AD),
     ($E7,$B9,$94),($E8,$81,$B7),($E8,$89,$B2),($E8,$A7,$A6),
     ($E9,$A3,$9F),($E8,$9D,$95),($E8,$BE,$B1),($E5,$B0,$BB),
     ($E4,$BC,$B8),($E4,$BF,$A1),($E4,$BE,$B5),($E5,$94,$87),
     ($E5,$A8,$A0),($E5,$AF,$9D),($E5,$AF,$A9),($E5,$BF,$83),
     ($E6,$85,$8E),($E6,$8C,$AF),($E6,$96,$B0),($E6,$99,$8B),
     ($E6,$A3,$AE),($E6,$A6,$9B),($E6,$B5,$B8),($E6,$B7,$B1),
     ($E7,$94,$B3),($E7,$96,$B9),($E7,$9C,$9F),($E7,$A5,$9E),
     ($E7,$A7,$A6),($E7,$B4,$B3),($E8,$87,$A3),($E8,$8A,$AF),
     ($E8,$96,$AA),($E8,$A6,$AA),($E8,$A8,$BA),($E8,$BA,$AB),
     ($E8,$BE,$9B),($E9,$80,$B2),($E9,$87,$9D),($E9,$9C,$87),
     ($E4,$BA,$BA),($E4,$BB,$81),($E5,$88,$83),($E5,$A1,$B5),
     ($E5,$A3,$AC),($E5,$B0,$8B),($E7,$94,$9A),($E5,$B0,$BD),
     ($E8,$85,$8E),($E8,$A8,$8A),($E8,$BF,$85),($E9,$99,$A3),
     ($E9,$9D,$AD),($E7,$AC,$A5),($E8,$AB,$8F),($E9,$A0,$88),
     ($E9,$85,$A2),($E5,$9B,$B3),($E5,$8E,$A8),($00,$00,$00),
     ($E9,$80,$97),($E5,$90,$B9),($E5,$9E,$82),($E5,$B8,$A5),
     ($E6,$8E,$A8),($E6,$B0,$B4),($E7,$82,$8A),($E7,$9D,$A1),
     ($E7,$B2,$8B),($E7,$BF,$A0),($E8,$A1,$B0),($E9,$81,$82),
     ($E9,$85,$94),($E9,$8C,$90),($E9,$8C,$98),($E9,$9A,$8F),
     ($E7,$91,$9E),($E9,$AB,$84),($E5,$B4,$87),($E5,$B5,$A9),
     ($E6,$95,$B0),($E6,$9E,$A2),($E8,$B6,$A8),($E9,$9B,$9B),
     ($E6,$8D,$AE),($E6,$9D,$89),($E6,$A4,$99),($E8,$8F,$85),
     ($E9,$A0,$97),($E9,$9B,$80),($E8,$A3,$BE),($E6,$BE,$84),
     ($E6,$91,$BA),($E5,$AF,$B8),($E4,$B8,$96),($E7,$80,$AC),
     ($E7,$95,$9D),($E6,$98,$AF),($E5,$87,$84),($E5,$88,$B6),
     ($E5,$8B,$A2),($E5,$A7,$93),($E5,$BE,$81),($E6,$80,$A7),
     ($E6,$88,$90),($E6,$94,$BF),($E6,$95,$B4),($E6,$98,$9F),
     ($E6,$99,$B4),($E6,$A3,$B2),($E6,$A0,$96),($E6,$AD,$A3),
     ($E6,$B8,$85),($E7,$89,$B2),($E7,$94,$9F),($E7,$9B,$9B),
     ($E7,$B2,$BE),($E8,$81,$96),($E5,$A3,$B0),($E8,$A3,$BD),
     ($E8,$A5,$BF),($E8,$AA,$A0),($E8,$AA,$93),($E8,$AB,$8B),
     ($E9,$80,$9D),($E9,$86,$92),($E9,$9D,$92),($E9,$9D,$99),
     ($E6,$96,$89),($E7,$A8,$8E),($E8,$84,$86),($E9,$9A,$BB),
     ($E5,$B8,$AD),($E6,$83,$9C),($E6,$88,$9A),($E6,$96,$A5),
     ($E6,$98,$94),($E6,$9E,$90),($E7,$9F,$B3),($E7,$A9,$8D),
     ($E7,$B1,$8D),($E7,$B8,$BE),($E8,$84,$8A),($E8,$B2,$AC),
     ($E8,$B5,$A4),($E8,$B7,$A1),($E8,$B9,$9F),($E7,$A2,$A9),
     ($E5,$88,$87),($E6,$8B,$99),($E6,$8E,$A5),($E6,$91,$82),
     ($E6,$8A,$98),($E8,$A8,$AD),($E7,$AA,$83),($E7,$AF,$80),
     ($E8,$AA,$AC),($E9,$9B,$AA),($E7,$B5,$B6),($E8,$88,$8C),
     ($E8,$9D,$89),($E4,$BB,$99),($E5,$85,$88),($E5,$8D,$83),
     ($E5,$8D,$A0),($E5,$AE,$A3),($E5,$B0,$82),($E5,$B0,$96),
     ($E5,$B7,$9D),($E6,$88,$A6),($E6,$89,$87),($E6,$92,$B0),
     ($E6,$A0,$93),($E6,$A0,$B4),($E6,$B3,$89),($E6,$B5,$85),
     ($E6,$B4,$97),($E6,$9F,$93),($E6,$BD,$9C),($E7,$85,$8E),
     ($E7,$85,$BD),($E6,$97,$8B),($E7,$A9,$BF),($E7,$AE,$AD),
     ($E7,$B7,$9A));

  S2U_91: TSjisRowTable =
    (($E7,$B9,$8A),($E7,$BE,$A8),($E8,$85,$BA),($E8,$88,$9B),
     ($E8,$88,$B9),($E8,$96,$A6),($E8,$A9,$AE),($E8,$B3,$8E),
     ($E8,$B7,$B5),($E9,$81,$B8),($E9,$81,$B7),($E9,$8A,$AD),
     ($E9,$8A,$91),($E9,$96,$83),($E9,$AE,$AE),($E5,$89,$8D),
     ($E5,$96,$84),($E6,$BC,$B8),($E7,$84,$B6),($E5,$85,$A8),
     ($E7,$A6,$85),($E7,$B9,$95),($E8,$86,$B3),($E7,$B3,$8E),
     ($E5,$99,$8C),($E5,$A1,$91),($E5,$B2,$A8),($E6,$8E,$AA),
     ($E6,$9B,$BE),($E6,$9B,$BD),($E6,$A5,$9A),($E7,$8B,$99),
     ($E7,$96,$8F),($E7,$96,$8E),($E7,$A4,$8E),($E7,$A5,$96),
     ($E7,$A7,$9F),($E7,$B2,$97),($E7,$B4,$A0),($E7,$B5,$84),
     ($E8,$98,$87),($E8,$A8,$B4),($E9,$98,$BB),($E9,$81,$A1),
     ($E9,$BC,$A0),($E5,$83,$A7),($E5,$89,$B5),($E5,$8F,$8C),
     ($E5,$8F,$A2),($E5,$80,$89),($E5,$96,$AA),($E5,$A3,$AE),
     ($E5,$A5,$8F),($E7,$88,$BD),($E5,$AE,$8B),($E5,$B1,$A4),
     ($E5,$8C,$9D),($E6,$83,$A3),($E6,$83,$B3),($E6,$8D,$9C),
     ($E6,$8E,$83),($E6,$8C,$BF),($E6,$8E,$BB),($00,$00,$00),
     ($E6,$93,$8D),($E6,$97,$A9),($E6,$9B,$B9),($E5,$B7,$A3),
     ($E6,$A7,$8D),($E6,$A7,$BD),($E6,$BC,$95),($E7,$87,$A5),
     ($E4,$BA,$89),($E7,$97,$A9),($E7,$9B,$B8),($E7,$AA,$93),
     ($E7,$B3,$9F),($E7,$B7,$8F),($E7,$B6,$9C),($E8,$81,$A1),
     ($E8,$8D,$89),($E8,$8D,$98),($E8,$91,$AC),($E8,$92,$BC),
     ($E8,$97,$BB),($E8,$A3,$85),($E8,$B5,$B0),($E9,$80,$81),
     ($E9,$81,$AD),($E9,$8E,$97),($E9,$9C,$9C),($E9,$A8,$92),
     ($E5,$83,$8F),($E5,$A2,$97),($E6,$86,$8E),($E8,$87,$93),
     ($E8,$94,$B5),($E8,$B4,$88),($E9,$80,$A0),($E4,$BF,$83),
     ($E5,$81,$B4),($E5,$89,$87),($E5,$8D,$B3),($E6,$81,$AF),
     ($E6,$8D,$89),($E6,$9D,$9F),($E6,$B8,$AC),($E8,$B6,$B3),
     ($E9,$80,$9F),($E4,$BF,$97),($E5,$B1,$9E),($E8,$B3,$8A),
     ($E6,$97,$8F),($E7,$B6,$9A),($E5,$8D,$92),($E8,$A2,$96),
     ($E5,$85,$B6),($E6,$8F,$83),($E5,$AD,$98),($E5,$AD,$AB),
     ($E5,$B0,$8A),($E6,$90,$8D),($E6,$9D,$91),($E9,$81,$9C),
     ($E4,$BB,$96),($E5,$A4,$9A),($E5,$A4,$AA),($E6,$B1,$B0),
     ($E8,$A9,$91),($E5,$94,$BE),($E5,$A0,$95),($E5,$A6,$A5),
     ($E6,$83,$B0),($E6,$89,$93),($E6,$9F,$81),($E8,$88,$B5),
     ($E6,$A5,$95),($E9,$99,$80),($E9,$A7,$84),($E9,$A8,$A8),
     ($E4,$BD,$93),($E5,$A0,$86),($E5,$AF,$BE),($E8,$80,$90),
     ($E5,$B2,$B1),($E5,$B8,$AF),($E5,$BE,$85),($E6,$80,$A0),
     ($E6,$85,$8B),($E6,$88,$B4),($E6,$9B,$BF),($E6,$B3,$B0),
     ($E6,$BB,$9E),($E8,$83,$8E),($E8,$85,$BF),($E8,$8B,$94),
     ($E8,$A2,$8B),($E8,$B2,$B8),($E9,$80,$80),($E9,$80,$AE),
     ($E9,$9A,$8A),($E9,$BB,$9B),($E9,$AF,$9B),($E4,$BB,$A3),
     ($E5,$8F,$B0),($E5,$A4,$A7),($E7,$AC,$AC),($E9,$86,$8D),
     ($E9,$A1,$8C),($E9,$B7,$B9),($E6,$BB,$9D),($E7,$80,$A7),
     ($E5,$8D,$93),($E5,$95,$84),($E5,$AE,$85),($E6,$89,$98),
     ($E6,$8A,$9E),($E6,$8B,$93),($E6,$B2,$A2),($E6,$BF,$AF),
     ($E7,$90,$A2),($E8,$A8,$97),($E9,$90,$B8),($E6,$BF,$81),
     ($E8,$AB,$BE),($E8,$8C,$B8),($E5,$87,$A7),($E8,$9B,$B8),
     ($E5,$8F,$AA));

  S2U_92: TSjisRowTable =
    (($E5,$8F,$A9),($E4,$BD,$86),($E9,$81,$94),($E8,$BE,$B0),
     ($E5,$A5,$AA),($E8,$84,$B1),($E5,$B7,$BD),($E7,$AB,$AA),
     ($E8,$BE,$BF),($E6,$A3,$9A),($E8,$B0,$B7),($E7,$8B,$B8),
     ($E9,$B1,$88),($E6,$A8,$BD),($E8,$AA,$B0),($E4,$B8,$B9),
     ($E5,$8D,$98),($E5,$98,$86),($E5,$9D,$A6),($E6,$8B,$85),
     ($E6,$8E,$A2),($E6,$97,$A6),($E6,$AD,$8E),($E6,$B7,$A1),
     ($E6,$B9,$9B),($E7,$82,$AD),($E7,$9F,$AD),($E7,$AB,$AF),
     ($E7,$AE,$AA),($E7,$B6,$BB),($E8,$80,$BD),($E8,$83,$86),
     ($E8,$9B,$8B),($E8,$AA,$95),($E9,$8D,$9B),($E5,$9B,$A3),
     ($E5,$A3,$87),($E5,$BC,$BE),($E6,$96,$AD),($E6,$9A,$96),
     ($E6,$AA,$80),($E6,$AE,$B5),($E7,$94,$B7),($E8,$AB,$87),
     ($E5,$80,$A4),($E7,$9F,$A5),($E5,$9C,$B0),($E5,$BC,$9B),
     ($E6,$81,$A5),($E6,$99,$BA),($E6,$B1,$A0),($E7,$97,$B4),
     ($E7,$A8,$9A),($E7,$BD,$AE),($E8,$87,$B4),($E8,$9C,$98),
     ($E9,$81,$85),($E9,$A6,$B3),($E7,$AF,$89),($E7,$95,$9C),
     ($E7,$AB,$B9),($E7,$AD,$91),($E8,$93,$84),($00,$00,$00),
     ($E9,$80,$90),($E7,$A7,$A9),($E7,$AA,$92),($E8,$8C,$B6),
     ($E5,$AB,$A1),($E7,$9D,$80),($E4,$B8,$AD),($E4,$BB,$B2),
     ($E5,$AE,$99),($E5,$BF,$A0),($E6,$8A,$BD),($E6,$98,$BC),
     ($E6,$9F,$B1),($E6,$B3,$A8),($E8,$99,$AB),($E8,$A1,$B7),
     ($E8,$A8,$BB),($E9,$85,$8E),($E9,$8B,$B3),($E9,$A7,$90),
     ($E6,$A8,$97),($E7,$80,$A6),($E7,$8C,$AA),($E8,$8B,$A7),
     ($E8,$91,$97),($E8,$B2,$AF),($E4,$B8,$81),($E5,$85,$86),
     ($E5,$87,$8B),($E5,$96,$8B),($E5,$AF,$B5),($E5,$B8,$96),
     ($E5,$B8,$B3),($E5,$BA,$81),($E5,$BC,$94),($E5,$BC,$B5),
     ($E5,$BD,$AB),($E5,$BE,$B4),($E6,$87,$B2),($E6,$8C,$91),
     ($E6,$9A,$A2),($E6,$9C,$9D),($E6,$BD,$AE),($E7,$89,$92),
     ($E7,$94,$BA),($E7,$9C,$BA),($E8,$81,$B4),($E8,$84,$B9),
     ($E8,$85,$B8),($E8,$9D,$B6),($E8,$AA,$BF),($E8,$AB,$9C),
     ($E8,$B6,$85),($E8,$B7,$B3),($E9,$8A,$9A),($E9,$95,$B7),
     ($E9,$A0,$82),($E9,$B3,$A5),($E5,$8B,$85),($E6,$8D,$97),
     ($E7,$9B,$B4),($E6,$9C,$95),($E6,$B2,$88),($E7,$8F,$8D),
     ($E8,$B3,$83),($E9,$8E,$AE),($E9,$99,$B3),($E6,$B4,$A5),
     ($E5,$A2,$9C),($E6,$A4,$8E),($E6,$A7,$8C),($E8,$BF,$BD),
     ($E9,$8E,$9A),($E7,$97,$9B),($E9,$80,$9A),($E5,$A1,$9A),
     ($E6,$A0,$82),($E6,$8E,$B4),($E6,$A7,$BB),($E4,$BD,$83),
     ($E6,$BC,$AC),($E6,$9F,$98),($E8,$BE,$BB),($E8,$94,$A6),
     ($E7,$B6,$B4),($E9,$8D,$94),($E6,$A4,$BF),($E6,$BD,$B0),
     ($E5,$9D,$AA),($E5,$A3,$B7),($E5,$AC,$AC),($E7,$B4,$AC),
     ($E7,$88,$AA),($E5,$90,$8A),($E9,$87,$A3),($E9,$B6,$B4),
     ($E4,$BA,$AD),($E4,$BD,$8E),($E5,$81,$9C),($E5,$81,$B5),
     ($E5,$89,$83),($E8,$B2,$9E),($E5,$91,$88),($E5,$A0,$A4),
     ($E5,$AE,$9A),($E5,$B8,$9D),($E5,$BA,$95),($E5,$BA,$AD),
     ($E5,$BB,$B7),($E5,$BC,$9F),($E6,$82,$8C),($E6,$8A,$B5),
     ($E6,$8C,$BA),($E6,$8F,$90),($E6,$A2,$AF),($E6,$B1,$80),
     ($E7,$A2,$87),($E7,$A6,$8E),($E7,$A8,$8B),($E7,$B7,$A0),
     ($E8,$89,$87),($E8,$A8,$82),($E8,$AB,$A6),($E8,$B9,$84),
     ($E9,$80,$93));

  S2U_93: TSjisRowTable =
    (($E9,$82,$B8),($E9,$84,$AD),($E9,$87,$98),($E9,$BC,$8E),
     ($E6,$B3,$A5),($E6,$91,$98),($E6,$93,$A2),($E6,$95,$B5),
     ($E6,$BB,$B4),($E7,$9A,$84),($E7,$AC,$9B),($E9,$81,$A9),
     ($E9,$8F,$91),($E6,$BA,$BA),($E5,$93,$B2),($E5,$BE,$B9),
     ($E6,$92,$A4),($E8,$BD,$8D),($E8,$BF,$AD),($E9,$89,$84),
     ($E5,$85,$B8),($E5,$A1,$AB),($E5,$A4,$A9),($E5,$B1,$95),
     ($E5,$BA,$97),($E6,$B7,$BB),($E7,$BA,$8F),($E7,$94,$9C),
     ($E8,$B2,$BC),($E8,$BB,$A2),($E9,$A1,$9B),($E7,$82,$B9),
     ($E4,$BC,$9D),($E6,$AE,$BF),($E6,$BE,$B1),($E7,$94,$B0),
     ($E9,$9B,$BB),($E5,$85,$8E),($E5,$90,$90),($E5,$A0,$B5),
     ($E5,$A1,$97),($E5,$A6,$AC),($E5,$B1,$A0),($E5,$BE,$92),
     ($E6,$96,$97),($E6,$9D,$9C),($E6,$B8,$A1),($E7,$99,$BB),
     ($E8,$8F,$9F),($E8,$B3,$AD),($E9,$80,$94),($E9,$83,$BD),
     ($E9,$8D,$8D),($E7,$A0,$A5),($E7,$A0,$BA),($E5,$8A,$AA),
     ($E5,$BA,$A6),($E5,$9C,$9F),($E5,$A5,$B4),($E6,$80,$92),
     ($E5,$80,$92),($E5,$85,$9A),($E5,$86,$AC),($00,$00,$00),
     ($E5,$87,$8D),($E5,$88,$80),($E5,$94,$90),($E5,$A1,$94),
     ($E5,$A1,$98),($E5,$A5,$97),($E5,$AE,$95),($E5,$B3,$B6),
     ($E5,$B6,$8B),($E6,$82,$BC),($E6,$8A,$95),($E6,$90,$AD),
     ($E6,$9D,$B1),($E6,$A1,$83),($E6,$A2,$BC),($E6,$A3,$9F),
     ($E7,$9B,$97),($E6,$B7,$98),($E6,$B9,$AF),($E6,$B6,$9B),
     ($E7,$81,$AF),($E7,$87,$88),($E5,$BD,$93),($E7,$97,$98),
     ($E7,$A5,$B7),($E7,$AD,$89),($E7,$AD,$94),($E7,$AD,$92),
     ($E7,$B3,$96),($E7,$B5,$B1),($E5,$88,$B0),($E8,$91,$A3),
     ($E8,$95,$A9),($E8,$97,$A4),($E8,$A8,$8E),($E8,$AC,$84),
     ($E8,$B1,$86),($E8,$B8,$8F),($E9,$80,$83),($E9,$80,$8F),
     ($E9,$90,$99),($E9,$99,$B6),($E9,$A0,$AD),($E9,$A8,$B0),
     ($E9,$97,$98),($E5,$83,$8D),($E5,$8B,$95),($E5,$90,$8C),
     ($E5,$A0,$82),($E5,$B0,$8E),($E6,$86,$A7),($E6,$92,$9E),
     ($E6,$B4,$9E),($E7,$9E,$B3),($E7,$AB,$A5),($E8,$83,$B4),
     ($E8,$90,$84),($E9,$81,$93),($E9,$8A,$85),($E5,$B3,$A0),
     ($E9,$B4,$87),($E5,$8C,$BF),($E5,$BE,$97),($E5,$BE,$B3),
     ($E6,$B6,$9C),($E7,$89,$B9),($E7,$9D,$A3),($E7,$A6,$BF),
     ($E7,$AF,$A4),($E6,$AF,$92),($E7,$8B,$AC),($E8,$AA,$AD),
     ($E6,$A0,$83),($E6,$A9,$A1),($E5,$87,$B8),($E7,$AA,$81),
     ($E6,$A4,$B4),($E5,$B1,$8A),($E9,$B3,$B6),($E8,$8B,$AB),
     ($E5,$AF,$85),($E9,$85,$89),($E7,$80,$9E),($E5,$99,$B8),
     ($E5,$B1,$AF),($E6,$83,$87),($E6,$95,$A6),($E6,$B2,$8C),
     ($E8,$B1,$9A),($E9,$81,$81),($E9,$A0,$93),($E5,$91,$91),
     ($E6,$9B,$87),($E9,$88,$8D),($E5,$A5,$88),($E9,$82,$A3),
     ($E5,$86,$85),($E4,$B9,$8D),($E5,$87,$AA),($E8,$96,$99),
     ($E8,$AC,$8E),($E7,$81,$98),($E6,$8D,$BA),($E9,$8D,$8B),
     ($E6,$A5,$A2),($E9,$A6,$B4),($E7,$B8,$84),($E7,$95,$B7),
     ($E5,$8D,$97),($E6,$A5,$A0),($E8,$BB,$9F),($E9,$9B,$A3),
     ($E6,$B1,$9D),($E4,$BA,$8C),($E5,$B0,$BC),($E5,$BC,$90),
     ($E8,$BF,$A9),($E5,$8C,$82),($E8,$B3,$91),($E8,$82,$89),
     ($E8,$99,$B9),($E5,$BB,$BF),($E6,$97,$A5),($E4,$B9,$B3),
     ($E5,$85,$A5));

  S2U_94: TSjisRowTable =
    (($E5,$A6,$82),($E5,$B0,$BF),($E9,$9F,$AE),($E4,$BB,$BB),
     ($E5,$A6,$8A),($E5,$BF,$8D),($E8,$AA,$8D),($E6,$BF,$A1),
     ($E7,$A6,$B0),($E7,$A5,$A2),($E5,$AF,$A7),($E8,$91,$B1),
     ($E7,$8C,$AB),($E7,$86,$B1),($E5,$B9,$B4),($E5,$BF,$B5),
     ($E6,$8D,$BB),($E6,$92,$9A),($E7,$87,$83),($E7,$B2,$98),
     ($E4,$B9,$83),($E5,$BB,$BC),($E4,$B9,$8B),($E5,$9F,$9C),
     ($E5,$9A,$A2),($E6,$82,$A9),($E6,$BF,$83),($E7,$B4,$8D),
     ($E8,$83,$BD),($E8,$84,$B3),($E8,$86,$BF),($E8,$BE,$B2),
     ($E8,$A6,$97),($E8,$9A,$A4),($E5,$B7,$B4),($E6,$8A,$8A),
     ($E6,$92,$AD),($E8,$A6,$87),($E6,$9D,$B7),($E6,$B3,$A2),
     ($E6,$B4,$BE),($E7,$90,$B6),($E7,$A0,$B4),($E5,$A9,$86),
     ($E7,$BD,$B5),($E8,$8A,$AD),($E9,$A6,$AC),($E4,$BF,$B3),
     ($E5,$BB,$83),($E6,$8B,$9D),($E6,$8E,$92),($E6,$95,$97),
     ($E6,$9D,$AF),($E7,$9B,$83),($E7,$89,$8C),($E8,$83,$8C),
     ($E8,$82,$BA),($E8,$BC,$A9),($E9,$85,$8D),($E5,$80,$8D),
     ($E5,$9F,$B9),($E5,$AA,$92),($E6,$A2,$85),($00,$00,$00),
     ($E6,$A5,$B3),($E7,$85,$A4),($E7,$8B,$BD),($E8,$B2,$B7),
     ($E5,$A3,$B2),($E8,$B3,$A0),($E9,$99,$AA),($E9,$80,$99),
     ($E8,$9D,$BF),($E7,$A7,$A4),($E7,$9F,$A7),($E8,$90,$A9),
     ($E4,$BC,$AF),($E5,$89,$A5),($E5,$8D,$9A),($E6,$8B,$8D),
     ($E6,$9F,$8F),($E6,$B3,$8A),($E7,$99,$BD),($E7,$AE,$94),
     ($E7,$B2,$95),($E8,$88,$B6),($E8,$96,$84),($E8,$BF,$AB),
     ($E6,$9B,$9D),($E6,$BC,$A0),($E7,$88,$86),($E7,$B8,$9B),
     ($E8,$8E,$AB),($E9,$A7,$81),($E9,$BA,$A6),($E5,$87,$BD),
     ($E7,$AE,$B1),($E7,$A1,$B2),($E7,$AE,$B8),($E8,$82,$87),
     ($E7,$AD,$88),($E6,$AB,$A8),($E5,$B9,$A1),($E8,$82,$8C),
     ($E7,$95,$91),($E7,$95,$A0),($E5,$85,$AB),($E9,$89,$A2),
     ($E6,$BA,$8C),($E7,$99,$BA),($E9,$86,$97),($E9,$AB,$AA),
     ($E4,$BC,$90),($E7,$BD,$B0),($E6,$8A,$9C),($E7,$AD,$8F),
     ($E9,$96,$A5),($E9,$B3,$A9),($E5,$99,$BA),($E5,$A1,$99),
     ($E8,$9B,$A4),($E9,$9A,$BC),($E4,$BC,$B4),($E5,$88,$A4),
     ($E5,$8D,$8A),($E5,$8F,$8D),($E5,$8F,$9B),($E5,$B8,$86),
     ($E6,$90,$AC),($E6,$96,$91),($E6,$9D,$BF),($E6,$B0,$BE),
     ($E6,$B1,$8E),($E7,$89,$88),($E7,$8A,$AF),($E7,$8F,$AD),
     ($E7,$95,$94),($E7,$B9,$81),($E8,$88,$AC),($E8,$97,$A9),
     ($E8,$B2,$A9),($E7,$AF,$84),($E9,$87,$86),($E7,$85,$A9),
     ($E9,$A0,$92),($E9,$A3,$AF),($E6,$8C,$BD),($E6,$99,$A9),
     ($E7,$95,$AA),($E7,$9B,$A4),($E7,$A3,$90),($E8,$95,$83),
     ($E8,$9B,$AE),($E5,$8C,$AA),($E5,$8D,$91),($E5,$90,$A6),
     ($E5,$A6,$83),($E5,$BA,$87),($E5,$BD,$BC),($E6,$82,$B2),
     ($E6,$89,$89),($E6,$89,$B9),($E6,$8A,$AB),($E6,$96,$90),
     ($E6,$AF,$94),($E6,$B3,$8C),($E7,$96,$B2),($E7,$9A,$AE),
     ($E7,$A2,$91),($E7,$A7,$98),($E7,$B7,$8B),($E7,$BD,$B7),
     ($E8,$82,$A5),($E8,$A2,$AB),($E8,$AA,$B9),($E8,$B2,$BB),
     ($E9,$81,$BF),($E9,$9D,$9E),($E9,$A3,$9B),($E6,$A8,$8B),
     ($E7,$B0,$B8),($E5,$82,$99),($E5,$B0,$BE),($E5,$BE,$AE),
     ($E6,$9E,$87),($E6,$AF,$98),($E7,$90,$B5),($E7,$9C,$89),
     ($E7,$BE,$8E));

  S2U_95: TSjisRowTable =
    (($E9,$BC,$BB),($E6,$9F,$8A),($E7,$A8,$97),($E5,$8C,$B9),
     ($E7,$96,$8B),($E9,$AB,$AD),($E5,$BD,$A6),($E8,$86,$9D),
     ($E8,$8F,$B1),($E8,$82,$98),($E5,$BC,$BC),($E5,$BF,$85),
     ($E7,$95,$A2),($E7,$AD,$86),($E9,$80,$BC),($E6,$A1,$A7),
     ($E5,$A7,$AB),($E5,$AA,$9B),($E7,$B4,$90),($E7,$99,$BE),
     ($E8,$AC,$AC),($E4,$BF,$B5),($E5,$BD,$AA),($E6,$A8,$99),
     ($E6,$B0,$B7),($E6,$BC,$82),($E7,$93,$A2),($E7,$A5,$A8),
     ($E8,$A1,$A8),($E8,$A9,$95),($E8,$B1,$B9),($E5,$BB,$9F),
     ($E6,$8F,$8F),($E7,$97,$85),($E7,$A7,$92),($E8,$8B,$97),
     ($E9,$8C,$A8),($E9,$8B,$B2),($E8,$92,$9C),($E8,$9B,$AD),
     ($E9,$B0,$AD),($E5,$93,$81),($E5,$BD,$AC),($E6,$96,$8C),
     ($E6,$B5,$9C),($E7,$80,$95),($E8,$B2,$A7),($E8,$B3,$93),
     ($E9,$A0,$BB),($E6,$95,$8F),($E7,$93,$B6),($E4,$B8,$8D),
     ($E4,$BB,$98),($E5,$9F,$A0),($E5,$A4,$AB),($E5,$A9,$A6),
     ($E5,$AF,$8C),($E5,$86,$A8),($E5,$B8,$83),($E5,$BA,$9C),
     ($E6,$80,$96),($E6,$89,$B6),($E6,$95,$B7),($00,$00,$00),
     ($E6,$96,$A7),($E6,$99,$AE),($E6,$B5,$AE),($E7,$88,$B6),
     ($E7,$AC,$A6),($E8,$85,$90),($E8,$86,$9A),($E8,$8A,$99),
     ($E8,$AD,$9C),($E8,$B2,$A0),($E8,$B3,$A6),($E8,$B5,$B4),
     ($E9,$98,$9C),($E9,$99,$84),($E4,$BE,$AE),($E6,$92,$AB),
     ($E6,$AD,$A6),($E8,$88,$9E),($E8,$91,$A1),($E8,$95,$AA),
     ($E9,$83,$A8),($E5,$B0,$81),($E6,$A5,$93),($E9,$A2,$A8),
     ($E8,$91,$BA),($E8,$95,$97),($E4,$BC,$8F),($E5,$89,$AF),
     ($E5,$BE,$A9),($E5,$B9,$85),($E6,$9C,$8D),($E7,$A6,$8F),
     ($E8,$85,$B9),($E8,$A4,$87),($E8,$A6,$86),($E6,$B7,$B5),
     ($E5,$BC,$97),($E6,$89,$95),($E6,$B2,$B8),($E4,$BB,$8F),
     ($E7,$89,$A9),($E9,$AE,$92),($E5,$88,$86),($E5,$90,$BB),
     ($E5,$99,$B4),($E5,$A2,$B3),($E6,$86,$A4),($E6,$89,$AE),
     ($E7,$84,$9A),($E5,$A5,$AE),($E7,$B2,$89),($E7,$B3,$9E),
     ($E7,$B4,$9B),($E9,$9B,$B0),($E6,$96,$87),($E8,$81,$9E),
     ($E4,$B8,$99),($E4,$BD,$B5),($E5,$85,$B5),($E5,$A1,$80),
     ($E5,$B9,$A3),($E5,$B9,$B3),($E5,$BC,$8A),($E6,$9F,$84),
     ($E4,$B8,$A6),($E8,$94,$BD),($E9,$96,$89),($E9,$99,$9B),
     ($E7,$B1,$B3),($E9,$A0,$81),($E5,$83,$BB),($E5,$A3,$81),
     ($E7,$99,$96),($E7,$A2,$A7),($E5,$88,$A5),($E7,$9E,$A5),
     ($E8,$94,$91),($E7,$AE,$86),($E5,$81,$8F),($E5,$A4,$89),
     ($E7,$89,$87),($E7,$AF,$87),($E7,$B7,$A8),($E8,$BE,$BA),
     ($E8,$BF,$94),($E9,$81,$8D),($E4,$BE,$BF),($E5,$8B,$89),
     ($E5,$A8,$A9),($E5,$BC,$81),($E9,$9E,$AD),($E4,$BF,$9D),
     ($E8,$88,$97),($E9,$8B,$AA),($E5,$9C,$83),($E6,$8D,$95),
     ($E6,$AD,$A9),($E7,$94,$AB),($E8,$A3,$9C),($E8,$BC,$94),
     ($E7,$A9,$82),($E5,$8B,$9F),($E5,$A2,$93),($E6,$85,$95),
     ($E6,$88,$8A),($E6,$9A,$AE),($E6,$AF,$8D),($E7,$B0,$BF),
     ($E8,$8F,$A9),($E5,$80,$A3),($E4,$BF,$B8),($E5,$8C,$85),
     ($E5,$91,$86),($E5,$A0,$B1),($E5,$A5,$89),($E5,$AE,$9D),
     ($E5,$B3,$B0),($E5,$B3,$AF),($E5,$B4,$A9),($E5,$BA,$96),
     ($E6,$8A,$B1),($E6,$8D,$A7),($E6,$94,$BE),($E6,$96,$B9),
     ($E6,$9C,$8B));

  S2U_96: TSjisRowTable =
    (($E6,$B3,$95),($E6,$B3,$A1),($E7,$83,$B9),($E7,$A0,$B2),
     ($E7,$B8,$AB),($E8,$83,$9E),($E8,$8A,$B3),($E8,$90,$8C),
     ($E8,$93,$AC),($E8,$9C,$82),($E8,$A4,$92),($E8,$A8,$AA),
     ($E8,$B1,$8A),($E9,$82,$A6),($E9,$8B,$92),($E9,$A3,$BD),
     ($E9,$B3,$B3),($E9,$B5,$AC),($E4,$B9,$8F),($E4,$BA,$A1),
     ($E5,$82,$8D),($E5,$89,$96),($E5,$9D,$8A),($E5,$A6,$A8),
     ($E5,$B8,$BD),($E5,$BF,$98),($E5,$BF,$99),($E6,$88,$BF),
     ($E6,$9A,$B4),($E6,$9C,$9B),($E6,$9F,$90),($E6,$A3,$92),
     ($E5,$86,$92),($E7,$B4,$A1),($E8,$82,$AA),($E8,$86,$A8),
     ($E8,$AC,$80),($E8,$B2,$8C),($E8,$B2,$BF),($E9,$89,$BE),
     ($E9,$98,$B2),($E5,$90,$A0),($E9,$A0,$AC),($E5,$8C,$97),
     ($E5,$83,$95),($E5,$8D,$9C),($E5,$A2,$A8),($E6,$92,$B2),
     ($E6,$9C,$B4),($E7,$89,$A7),($E7,$9D,$A6),($E7,$A9,$86),
     ($E9,$87,$A6),($E5,$8B,$83),($E6,$B2,$A1),($E6,$AE,$86),
     ($E5,$A0,$80),($E5,$B9,$8C),($E5,$A5,$94),($E6,$9C,$AC),
     ($E7,$BF,$BB),($E5,$87,$A1),($E7,$9B,$86),($00,$00,$00),
     ($E6,$91,$A9),($E7,$A3,$A8),($E9,$AD,$94),($E9,$BA,$BB),
     ($E5,$9F,$8B),($E5,$A6,$B9),($E6,$98,$A7),($E6,$9E,$9A),
     ($E6,$AF,$8E),($E5,$93,$A9),($E6,$A7,$99),($E5,$B9,$95),
     ($E8,$86,$9C),($E6,$9E,$95),($E9,$AE,$AA),($E6,$9F,$BE),
     ($E9,$B1,$92),($E6,$A1,$9D),($E4,$BA,$A6),($E4,$BF,$A3),
     ($E5,$8F,$88),($E6,$8A,$B9),($E6,$9C,$AB),($E6,$B2,$AB),
     ($E8,$BF,$84),($E4,$BE,$AD),($E7,$B9,$AD),($E9,$BA,$BF),
     ($E4,$B8,$87),($E6,$85,$A2),($E6,$BA,$80),($E6,$BC,$AB),
     ($E8,$94,$93),($E5,$91,$B3),($E6,$9C,$AA),($E9,$AD,$85),
     ($E5,$B7,$B3),($E7,$AE,$95),($E5,$B2,$AC),($E5,$AF,$86),
     ($E8,$9C,$9C),($E6,$B9,$8A),($E8,$93,$91),($E7,$A8,$94),
     ($E8,$84,$88),($E5,$A6,$99),($E7,$B2,$8D),($E6,$B0,$91),
     ($E7,$9C,$A0),($E5,$8B,$99),($E5,$A4,$A2),($E7,$84,$A1),
     ($E7,$89,$9F),($E7,$9F,$9B),($E9,$9C,$A7),($E9,$B5,$A1),
     ($E6,$A4,$8B),($E5,$A9,$BF),($E5,$A8,$98),($E5,$86,$A5),
     ($E5,$90,$8D),($E5,$91,$BD),($E6,$98,$8E),($E7,$9B,$9F),
     ($E8,$BF,$B7),($E9,$8A,$98),($E9,$B3,$B4),($E5,$A7,$AA),
     ($E7,$89,$9D),($E6,$BB,$85),($E5,$85,$8D),($E6,$A3,$89),
     ($E7,$B6,$BF),($E7,$B7,$AC),($E9,$9D,$A2),($E9,$BA,$BA),
     ($E6,$91,$B8),($E6,$A8,$A1),($E8,$8C,$82),($E5,$A6,$84),
     ($E5,$AD,$9F),($E6,$AF,$9B),($E7,$8C,$9B),($E7,$9B,$B2),
     ($E7,$B6,$B2),($E8,$80,$97),($E8,$92,$99),($E5,$84,$B2),
     ($E6,$9C,$A8),($E9,$BB,$99),($E7,$9B,$AE),($E6,$9D,$A2),
     ($E5,$8B,$BF),($E9,$A4,$85),($E5,$B0,$A4),($E6,$88,$BB),
     ($E7,$B1,$BE),($E8,$B2,$B0),($E5,$95,$8F),($E6,$82,$B6),
     ($E7,$B4,$8B),($E9,$96,$80),($E5,$8C,$81),($E4,$B9,$9F),
     ($E5,$86,$B6),($E5,$A4,$9C),($E7,$88,$BA),($E8,$80,$B6),
     ($E9,$87,$8E),($E5,$BC,$A5),($E7,$9F,$A2),($E5,$8E,$84),
     ($E5,$BD,$B9),($E7,$B4,$84),($E8,$96,$AC),($E8,$A8,$B3),
     ($E8,$BA,$8D),($E9,$9D,$96),($E6,$9F,$B3),($E8,$96,$AE),
     ($E9,$91,$93),($E6,$84,$89),($E6,$84,$88),($E6,$B2,$B9),
     ($E7,$99,$92));

  S2U_97: TSjisRowTable =
    (($E8,$AB,$AD),($E8,$BC,$B8),($E5,$94,$AF),($E4,$BD,$91),
     ($E5,$84,$AA),($E5,$8B,$87),($E5,$8F,$8B),($E5,$AE,$A5),
     ($E5,$B9,$BD),($E6,$82,$A0),($E6,$86,$82),($E6,$8F,$96),
     ($E6,$9C,$89),($E6,$9F,$9A),($E6,$B9,$A7),($E6,$B6,$8C),
     ($E7,$8C,$B6),($E7,$8C,$B7),($E7,$94,$B1),($E7,$A5,$90),
     ($E8,$A3,$95),($E8,$AA,$98),($E9,$81,$8A),($E9,$82,$91),
     ($E9,$83,$B5),($E9,$9B,$84),($E8,$9E,$8D),($E5,$A4,$95),
     ($E4,$BA,$88),($E4,$BD,$99),($E4,$B8,$8E),($E8,$AA,$89),
     ($E8,$BC,$BF),($E9,$A0,$90),($E5,$82,$AD),($E5,$B9,$BC),
     ($E5,$A6,$96),($E5,$AE,$B9),($E5,$BA,$B8),($E6,$8F,$9A),
     ($E6,$8F,$BA),($E6,$93,$81),($E6,$9B,$9C),($E6,$A5,$8A),
     ($E6,$A7,$98),($E6,$B4,$8B),($E6,$BA,$B6),($E7,$86,$94),
     ($E7,$94,$A8),($E7,$AA,$AF),($E7,$BE,$8A),($E8,$80,$80),
     ($E8,$91,$89),($E8,$93,$89),($E8,$A6,$81),($E8,$AC,$A1),
     ($E8,$B8,$8A),($E9,$81,$A5),($E9,$99,$BD),($E9,$A4,$8A),
     ($E6,$85,$BE),($E6,$8A,$91),($E6,$AC,$B2),($00,$00,$00),
     ($E6,$B2,$83),($E6,$B5,$B4),($E7,$BF,$8C),($E7,$BF,$BC),
     ($E6,$B7,$80),($E7,$BE,$85),($E8,$9E,$BA),($E8,$A3,$B8),
     ($E6,$9D,$A5),($E8,$8E,$B1),($E9,$A0,$BC),($E9,$9B,$B7),
     ($E6,$B4,$9B),($E7,$B5,$A1),($E8,$90,$BD),($E9,$85,$AA),
     ($E4,$B9,$B1),($E5,$8D,$B5),($E5,$B5,$90),($E6,$AC,$84),
     ($E6,$BF,$AB),($E8,$97,$8D),($E8,$98,$AD),($E8,$A6,$A7),
     ($E5,$88,$A9),($E5,$90,$8F),($E5,$B1,$A5),($E6,$9D,$8E),
     ($E6,$A2,$A8),($E7,$90,$86),($E7,$92,$83),($E7,$97,$A2),
     ($E8,$A3,$8F),($E8,$A3,$A1),($E9,$87,$8C),($E9,$9B,$A2),
     ($E9,$99,$B8),($E5,$BE,$8B),($E7,$8E,$87),($E7,$AB,$8B),
     ($E8,$91,$8E),($E6,$8E,$A0),($E7,$95,$A5),($E5,$8A,$89),
     ($E6,$B5,$81),($E6,$BA,$9C),($E7,$90,$89),($E7,$95,$99),
     ($E7,$A1,$AB),($E7,$B2,$92),($E9,$9A,$86),($E7,$AB,$9C),
     ($E9,$BE,$8D),($E4,$BE,$B6),($E6,$85,$AE),($E6,$97,$85),
     ($E8,$99,$9C),($E4,$BA,$86),($E4,$BA,$AE),($E5,$83,$9A),
     ($E4,$B8,$A1),($E5,$87,$8C),($E5,$AF,$AE),($E6,$96,$99),
     ($E6,$A2,$81),($E6,$B6,$BC),($E7,$8C,$9F),($E7,$99,$82),
     ($E7,$9E,$AD),($E7,$A8,$9C),($E7,$B3,$A7),($E8,$89,$AF),
     ($E8,$AB,$92),($E9,$81,$BC),($E9,$87,$8F),($E9,$99,$B5),
     ($E9,$A0,$98),($E5,$8A,$9B),($E7,$B7,$91),($E5,$80,$AB),
     ($E5,$8E,$98),($E6,$9E,$97),($E6,$B7,$8B),($E7,$87,$90),
     ($E7,$90,$B3),($E8,$87,$A8),($E8,$BC,$AA),($E9,$9A,$A3),
     ($E9,$B1,$97),($E9,$BA,$9F),($E7,$91,$A0),($E5,$A1,$81),
     ($E6,$B6,$99),($E7,$B4,$AF),($E9,$A1,$9E),($E4,$BB,$A4),
     ($E4,$BC,$B6),($E4,$BE,$8B),($E5,$86,$B7),($E5,$8A,$B1),
     ($E5,$B6,$BA),($E6,$80,$9C),($E7,$8E,$B2),($E7,$A4,$BC),
     ($E8,$8B,$93),($E9,$88,$B4),($E9,$9A,$B7),($E9,$9B,$B6),
     ($E9,$9C,$8A),($E9,$BA,$97),($E9,$BD,$A2),($E6,$9A,$A6),
     ($E6,$AD,$B4),($E5,$88,$97),($E5,$8A,$A3),($E7,$83,$88),
     ($E8,$A3,$82),($E5,$BB,$89),($E6,$81,$8B),($E6,$86,$90),
     ($E6,$BC,$A3),($E7,$85,$89),($E7,$B0,$BE),($E7,$B7,$B4),
     ($E8,$81,$AF));

  S2U_98: TSjisRowTable =
    (($E8,$93,$AE),($E9,$80,$A3),($E9,$8C,$AC),($E5,$91,$82),
     ($E9,$AD,$AF),($E6,$AB,$93),($E7,$82,$89),($E8,$B3,$82),
     ($E8,$B7,$AF),($E9,$9C,$B2),($E5,$8A,$B4),($E5,$A9,$81),
     ($E5,$BB,$8A),($E5,$BC,$84),($E6,$9C,$97),($E6,$A5,$BC),
     ($E6,$A6,$94),($E6,$B5,$AA),($E6,$BC,$8F),($E7,$89,$A2),
     ($E7,$8B,$BC),($E7,$AF,$AD),($E8,$80,$81),($E8,$81,$BE),
     ($E8,$9D,$8B),($E9,$83,$8E),($E5,$85,$AD),($E9,$BA,$93),
     ($E7,$A6,$84),($E8,$82,$8B),($E9,$8C,$B2),($E8,$AB,$96),
     ($E5,$80,$AD),($E5,$92,$8C),($E8,$A9,$B1),($E6,$AD,$AA),
     ($E8,$B3,$84),($E8,$84,$87),($E6,$83,$91),($E6,$9E,$A0),
     ($E9,$B7,$B2),($E4,$BA,$99),($E4,$BA,$98),($E9,$B0,$90),
     ($E8,$A9,$AB),($E8,$97,$81),($E8,$95,$A8),($E6,$A4,$80),
     ($E6,$B9,$BE),($E7,$A2,$97),($E8,$85,$95),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($E5,$BC,$8C),
     ($E4,$B8,$90),($E4,$B8,$95),($E4,$B8,$AA),($E4,$B8,$B1),
     ($E4,$B8,$B6),($E4,$B8,$BC),($E4,$B8,$BF),($E4,$B9,$82),
     ($E4,$B9,$96),($E4,$B9,$98),($E4,$BA,$82),($E4,$BA,$85),
     ($E8,$B1,$AB),($E4,$BA,$8A),($E8,$88,$92),($E5,$BC,$8D),
     ($E4,$BA,$8E),($E4,$BA,$9E),($E4,$BA,$9F),($E4,$BA,$A0),
     ($E4,$BA,$A2),($E4,$BA,$B0),($E4,$BA,$B3),($E4,$BA,$B6),
     ($E4,$BB,$8E),($E4,$BB,$8D),($E4,$BB,$84),($E4,$BB,$86),
     ($E4,$BB,$82),($E4,$BB,$97),($E4,$BB,$9E),($E4,$BB,$AD),
     ($E4,$BB,$9F),($E4,$BB,$B7),($E4,$BC,$89),($E4,$BD,$9A),
     ($E4,$BC,$B0),($E4,$BD,$9B),($E4,$BD,$9D),($E4,$BD,$97),
     ($E4,$BD,$87),($E4,$BD,$B6),($E4,$BE,$88),($E4,$BE,$8F),
     ($E4,$BE,$98),($E4,$BD,$BB),($E4,$BD,$A9),($E4,$BD,$B0),
     ($E4,$BE,$91),($E4,$BD,$AF),($E4,$BE,$86),($E4,$BE,$96),
     ($E5,$84,$98),($E4,$BF,$94),($E4,$BF,$9F),($E4,$BF,$8E),
     ($E4,$BF,$98),($E4,$BF,$9B),($E4,$BF,$91),($E4,$BF,$9A),
     ($E4,$BF,$90),($E4,$BF,$A4),($E4,$BF,$A5),($E5,$80,$9A),
     ($E5,$80,$A8),($E5,$80,$94),($E5,$80,$AA),($E5,$80,$A5),
     ($E5,$80,$85),($E4,$BC,$9C),($E4,$BF,$B6),($E5,$80,$A1),
     ($E5,$80,$A9),($E5,$80,$AC),($E4,$BF,$BE),($E4,$BF,$AF),
     ($E5,$80,$91),($E5,$80,$86),($E5,$81,$83),($E5,$81,$87),
     ($E6,$9C,$83),($E5,$81,$95),($E5,$81,$90),($E5,$81,$88),
     ($E5,$81,$9A),($E5,$81,$96),($E5,$81,$AC),($E5,$81,$B8),
     ($E5,$82,$80),($E5,$82,$9A),($E5,$82,$85),($E5,$82,$B4),
     ($E5,$82,$B2));

  S2U_99: TSjisRowTable =
    (($E5,$83,$89),($E5,$83,$8A),($E5,$82,$B3),($E5,$83,$82),
     ($E5,$83,$96),($E5,$83,$9E),($E5,$83,$A5),($E5,$83,$AD),
     ($E5,$83,$A3),($E5,$83,$AE),($E5,$83,$B9),($E5,$83,$B5),
     ($E5,$84,$89),($E5,$84,$81),($E5,$84,$82),($E5,$84,$96),
     ($E5,$84,$95),($E5,$84,$94),($E5,$84,$9A),($E5,$84,$A1),
     ($E5,$84,$BA),($E5,$84,$B7),($E5,$84,$BC),($E5,$84,$BB),
     ($E5,$84,$BF),($E5,$85,$80),($E5,$85,$92),($E5,$85,$8C),
     ($E5,$85,$94),($E5,$85,$A2),($E7,$AB,$B8),($E5,$85,$A9),
     ($E5,$85,$AA),($E5,$85,$AE),($E5,$86,$80),($E5,$86,$82),
     ($E5,$9B,$98),($E5,$86,$8C),($E5,$86,$89),($E5,$86,$8F),
     ($E5,$86,$91),($E5,$86,$93),($E5,$86,$95),($E5,$86,$96),
     ($E5,$86,$A4),($E5,$86,$A6),($E5,$86,$A2),($E5,$86,$A9),
     ($E5,$86,$AA),($E5,$86,$AB),($E5,$86,$B3),($E5,$86,$B1),
     ($E5,$86,$B2),($E5,$86,$B0),($E5,$86,$B5),($E5,$86,$BD),
     ($E5,$87,$85),($E5,$87,$89),($E5,$87,$9B),($E5,$87,$A0),
     ($E8,$99,$95),($E5,$87,$A9),($E5,$87,$AD),($00,$00,$00),
     ($E5,$87,$B0),($E5,$87,$B5),($E5,$87,$BE),($E5,$88,$84),
     ($E5,$88,$8B),($E5,$88,$94),($E5,$88,$8E),($E5,$88,$A7),
     ($E5,$88,$AA),($E5,$88,$AE),($E5,$88,$B3),($E5,$88,$B9),
     ($E5,$89,$8F),($E5,$89,$84),($E5,$89,$8B),($E5,$89,$8C),
     ($E5,$89,$9E),($E5,$89,$94),($E5,$89,$AA),($E5,$89,$B4),
     ($E5,$89,$A9),($E5,$89,$B3),($E5,$89,$BF),($E5,$89,$BD),
     ($E5,$8A,$8D),($E5,$8A,$94),($E5,$8A,$92),($E5,$89,$B1),
     ($E5,$8A,$88),($E5,$8A,$91),($E8,$BE,$A8),($E8,$BE,$A7),
     ($E5,$8A,$AC),($E5,$8A,$AD),($E5,$8A,$BC),($E5,$8A,$B5),
     ($E5,$8B,$81),($E5,$8B,$8D),($E5,$8B,$97),($E5,$8B,$9E),
     ($E5,$8B,$A3),($E5,$8B,$A6),($E9,$A3,$AD),($E5,$8B,$A0),
     ($E5,$8B,$B3),($E5,$8B,$B5),($E5,$8B,$B8),($E5,$8B,$B9),
     ($E5,$8C,$86),($E5,$8C,$88),($E7,$94,$B8),($E5,$8C,$8D),
     ($E5,$8C,$90),($E5,$8C,$8F),($E5,$8C,$95),($E5,$8C,$9A),
     ($E5,$8C,$A3),($E5,$8C,$AF),($E5,$8C,$B1),($E5,$8C,$B3),
     ($E5,$8C,$B8),($E5,$8D,$80),($E5,$8D,$86),($E5,$8D,$85),
     ($E4,$B8,$97),($E5,$8D,$89),($E5,$8D,$8D),($E5,$87,$96),
     ($E5,$8D,$9E),($E5,$8D,$A9),($E5,$8D,$AE),($E5,$A4,$98),
     ($E5,$8D,$BB),($E5,$8D,$B7),($E5,$8E,$82),($E5,$8E,$96),
     ($E5,$8E,$A0),($E5,$8E,$A6),($E5,$8E,$A5),($E5,$8E,$AE),
     ($E5,$8E,$B0),($E5,$8E,$B6),($E5,$8F,$83),($E7,$B0,$92),
     ($E9,$9B,$99),($E5,$8F,$9F),($E6,$9B,$BC),($E7,$87,$AE),
     ($E5,$8F,$AE),($E5,$8F,$A8),($E5,$8F,$AD),($E5,$8F,$BA),
     ($E5,$90,$81),($E5,$90,$BD),($E5,$91,$80),($E5,$90,$AC),
     ($E5,$90,$AD),($E5,$90,$BC),($E5,$90,$AE),($E5,$90,$B6),
     ($E5,$90,$A9),($E5,$90,$9D),($E5,$91,$8E),($E5,$92,$8F),
     ($E5,$91,$B5),($E5,$92,$8E),($E5,$91,$9F),($E5,$91,$B1),
     ($E5,$91,$B7),($E5,$91,$B0),($E5,$92,$92),($E5,$91,$BB),
     ($E5,$92,$80),($E5,$91,$B6),($E5,$92,$84),($E5,$92,$90),
     ($E5,$92,$86),($E5,$93,$87),($E5,$92,$A2),($E5,$92,$B8),
     ($E5,$92,$A5),($E5,$92,$AC),($E5,$93,$84),($E5,$93,$88),
     ($E5,$92,$A8));

  S2U_9A: TSjisRowTable =
    (($E5,$92,$AB),($E5,$93,$82),($E5,$92,$A4),($E5,$92,$BE),
     ($E5,$92,$BC),($E5,$93,$98),($E5,$93,$A5),($E5,$93,$A6),
     ($E5,$94,$8F),($E5,$94,$94),($E5,$93,$BD),($E5,$93,$AE),
     ($E5,$93,$AD),($E5,$93,$BA),($E5,$93,$A2),($E5,$94,$B9),
     ($E5,$95,$80),($E5,$95,$A3),($E5,$95,$8C),($E5,$94,$AE),
     ($E5,$95,$9C),($E5,$95,$85),($E5,$95,$96),($E5,$95,$97),
     ($E5,$94,$B8),($E5,$94,$B3),($E5,$95,$9D),($E5,$96,$99),
     ($E5,$96,$80),($E5,$92,$AF),($E5,$96,$8A),($E5,$96,$9F),
     ($E5,$95,$BB),($E5,$95,$BE),($E5,$96,$98),($E5,$96,$9E),
     ($E5,$96,$AE),($E5,$95,$BC),($E5,$96,$83),($E5,$96,$A9),
     ($E5,$96,$87),($E5,$96,$A8),($E5,$97,$9A),($E5,$97,$85),
     ($E5,$97,$9F),($E5,$97,$84),($E5,$97,$9C),($E5,$97,$A4),
     ($E5,$97,$94),($E5,$98,$94),($E5,$97,$B7),($E5,$98,$96),
     ($E5,$97,$BE),($E5,$97,$BD),($E5,$98,$9B),($E5,$97,$B9),
     ($E5,$99,$8E),($E5,$99,$90),($E7,$87,$9F),($E5,$98,$B4),
     ($E5,$98,$B6),($E5,$98,$B2),($E5,$98,$B8),($00,$00,$00),
     ($E5,$99,$AB),($E5,$99,$A4),($E5,$98,$AF),($E5,$99,$AC),
     ($E5,$99,$AA),($E5,$9A,$86),($E5,$9A,$80),($E5,$9A,$8A),
     ($E5,$9A,$A0),($E5,$9A,$94),($E5,$9A,$8F),($E5,$9A,$A5),
     ($E5,$9A,$AE),($E5,$9A,$B6),($E5,$9A,$B4),($E5,$9B,$82),
     ($E5,$9A,$BC),($E5,$9B,$81),($E5,$9B,$83),($E5,$9B,$80),
     ($E5,$9B,$88),($E5,$9B,$8E),($E5,$9B,$91),($E5,$9B,$93),
     ($E5,$9B,$97),($E5,$9B,$AE),($E5,$9B,$B9),($E5,$9C,$80),
     ($E5,$9B,$BF),($E5,$9C,$84),($E5,$9C,$89),($E5,$9C,$88),
     ($E5,$9C,$8B),($E5,$9C,$8D),($E5,$9C,$93),($E5,$9C,$98),
     ($E5,$9C,$96),($E5,$97,$87),($E5,$9C,$9C),($E5,$9C,$A6),
     ($E5,$9C,$B7),($E5,$9C,$B8),($E5,$9D,$8E),($E5,$9C,$BB),
     ($E5,$9D,$80),($E5,$9D,$8F),($E5,$9D,$A9),($E5,$9F,$80),
     ($E5,$9E,$88),($E5,$9D,$A1),($E5,$9D,$BF),($E5,$9E,$89),
     ($E5,$9E,$93),($E5,$9E,$A0),($E5,$9E,$B3),($E5,$9E,$A4),
     ($E5,$9E,$AA),($E5,$9E,$B0),($E5,$9F,$83),($E5,$9F,$86),
     ($E5,$9F,$94),($E5,$9F,$92),($E5,$9F,$93),($E5,$A0,$8A),
     ($E5,$9F,$96),($E5,$9F,$A3),($E5,$A0,$8B),($E5,$A0,$99),
     ($E5,$A0,$9D),($E5,$A1,$B2),($E5,$A0,$A1),($E5,$A1,$A2),
     ($E5,$A1,$8B),($E5,$A1,$B0),($E6,$AF,$80),($E5,$A1,$92),
     ($E5,$A0,$BD),($E5,$A1,$B9),($E5,$A2,$85),($E5,$A2,$B9),
     ($E5,$A2,$9F),($E5,$A2,$AB),($E5,$A2,$BA),($E5,$A3,$9E),
     ($E5,$A2,$BB),($E5,$A2,$B8),($E5,$A2,$AE),($E5,$A3,$85),
     ($E5,$A3,$93),($E5,$A3,$91),($E5,$A3,$97),($E5,$A3,$99),
     ($E5,$A3,$98),($E5,$A3,$A5),($E5,$A3,$9C),($E5,$A3,$A4),
     ($E5,$A3,$9F),($E5,$A3,$AF),($E5,$A3,$BA),($E5,$A3,$B9),
     ($E5,$A3,$BB),($E5,$A3,$BC),($E5,$A3,$BD),($E5,$A4,$82),
     ($E5,$A4,$8A),($E5,$A4,$90),($E5,$A4,$9B),($E6,$A2,$A6),
     ($E5,$A4,$A5),($E5,$A4,$AC),($E5,$A4,$AD),($E5,$A4,$B2),
     ($E5,$A4,$B8),($E5,$A4,$BE),($E7,$AB,$92),($E5,$A5,$95),
     ($E5,$A5,$90),($E5,$A5,$8E),($E5,$A5,$9A),($E5,$A5,$98),
     ($E5,$A5,$A2),($E5,$A5,$A0),($E5,$A5,$A7),($E5,$A5,$AC),
     ($E5,$A5,$A9));

  S2U_9B: TSjisRowTable =
    (($E5,$A5,$B8),($E5,$A6,$81),($E5,$A6,$9D),($E4,$BD,$9E),
     ($E4,$BE,$AB),($E5,$A6,$A3),($E5,$A6,$B2),($E5,$A7,$86),
     ($E5,$A7,$A8),($E5,$A7,$9C),($E5,$A6,$8D),($E5,$A7,$99),
     ($E5,$A7,$9A),($E5,$A8,$A5),($E5,$A8,$9F),($E5,$A8,$91),
     ($E5,$A8,$9C),($E5,$A8,$89),($E5,$A8,$9A),($E5,$A9,$80),
     ($E5,$A9,$AC),($E5,$A9,$89),($E5,$A8,$B5),($E5,$A8,$B6),
     ($E5,$A9,$A2),($E5,$A9,$AA),($E5,$AA,$9A),($E5,$AA,$BC),
     ($E5,$AA,$BE),($E5,$AB,$8B),($E5,$AB,$82),($E5,$AA,$BD),
     ($E5,$AB,$A3),($E5,$AB,$97),($E5,$AB,$A6),($E5,$AB,$A9),
     ($E5,$AB,$96),($E5,$AB,$BA),($E5,$AB,$BB),($E5,$AC,$8C),
     ($E5,$AC,$8B),($E5,$AC,$96),($E5,$AC,$B2),($E5,$AB,$90),
     ($E5,$AC,$AA),($E5,$AC,$B6),($E5,$AC,$BE),($E5,$AD,$83),
     ($E5,$AD,$85),($E5,$AD,$80),($E5,$AD,$91),($E5,$AD,$95),
     ($E5,$AD,$9A),($E5,$AD,$9B),($E5,$AD,$A5),($E5,$AD,$A9),
     ($E5,$AD,$B0),($E5,$AD,$B3),($E5,$AD,$B5),($E5,$AD,$B8),
     ($E6,$96,$88),($E5,$AD,$BA),($E5,$AE,$80),($00,$00,$00),
     ($E5,$AE,$83),($E5,$AE,$A6),($E5,$AE,$B8),($E5,$AF,$83),
     ($E5,$AF,$87),($E5,$AF,$89),($E5,$AF,$94),($E5,$AF,$90),
     ($E5,$AF,$A4),($E5,$AF,$A6),($E5,$AF,$A2),($E5,$AF,$9E),
     ($E5,$AF,$A5),($E5,$AF,$AB),($E5,$AF,$B0),($E5,$AF,$B6),
     ($E5,$AF,$B3),($E5,$B0,$85),($E5,$B0,$87),($E5,$B0,$88),
     ($E5,$B0,$8D),($E5,$B0,$93),($E5,$B0,$A0),($E5,$B0,$A2),
     ($E5,$B0,$A8),($E5,$B0,$B8),($E5,$B0,$B9),($E5,$B1,$81),
     ($E5,$B1,$86),($E5,$B1,$8E),($E5,$B1,$93),($E5,$B1,$90),
     ($E5,$B1,$8F),($E5,$AD,$B1),($E5,$B1,$AC),($E5,$B1,$AE),
     ($E4,$B9,$A2),($E5,$B1,$B6),($E5,$B1,$B9),($E5,$B2,$8C),
     ($E5,$B2,$91),($E5,$B2,$94),($E5,$A6,$9B),($E5,$B2,$AB),
     ($E5,$B2,$BB),($E5,$B2,$B6),($E5,$B2,$BC),($E5,$B2,$B7),
     ($E5,$B3,$85),($E5,$B2,$BE),($E5,$B3,$87),($E5,$B3,$99),
     ($E5,$B3,$A9),($E5,$B3,$BD),($E5,$B3,$BA),($E5,$B3,$AD),
     ($E5,$B6,$8C),($E5,$B3,$AA),($E5,$B4,$8B),($E5,$B4,$95),
     ($E5,$B4,$97),($E5,$B5,$9C),($E5,$B4,$9F),($E5,$B4,$9B),
     ($E5,$B4,$91),($E5,$B4,$94),($E5,$B4,$A2),($E5,$B4,$9A),
     ($E5,$B4,$99),($E5,$B4,$98),($E5,$B5,$8C),($E5,$B5,$92),
     ($E5,$B5,$8E),($E5,$B5,$8B),($E5,$B5,$AC),($E5,$B5,$B3),
     ($E5,$B5,$B6),($E5,$B6,$87),($E5,$B6,$84),($E5,$B6,$82),
     ($E5,$B6,$A2),($E5,$B6,$9D),($E5,$B6,$AC),($E5,$B6,$AE),
     ($E5,$B6,$BD),($E5,$B6,$90),($E5,$B6,$B7),($E5,$B6,$BC),
     ($E5,$B7,$89),($E5,$B7,$8D),($E5,$B7,$93),($E5,$B7,$92),
     ($E5,$B7,$96),($E5,$B7,$9B),($E5,$B7,$AB),($E5,$B7,$B2),
     ($E5,$B7,$B5),($E5,$B8,$8B),($E5,$B8,$9A),($E5,$B8,$99),
     ($E5,$B8,$91),($E5,$B8,$9B),($E5,$B8,$B6),($E5,$B8,$B7),
     ($E5,$B9,$84),($E5,$B9,$83),($E5,$B9,$80),($E5,$B9,$8E),
     ($E5,$B9,$97),($E5,$B9,$94),($E5,$B9,$9F),($E5,$B9,$A2),
     ($E5,$B9,$A4),($E5,$B9,$87),($E5,$B9,$B5),($E5,$B9,$B6),
     ($E5,$B9,$BA),($E9,$BA,$BC),($E5,$B9,$BF),($E5,$BA,$A0),
     ($E5,$BB,$81),($E5,$BB,$82),($E5,$BB,$88),($E5,$BB,$90),
     ($E5,$BB,$8F));

  S2U_9C: TSjisRowTable =
    (($E5,$BB,$96),($E5,$BB,$A3),($E5,$BB,$9D),($E5,$BB,$9A),
     ($E5,$BB,$9B),($E5,$BB,$A2),($E5,$BB,$A1),($E5,$BB,$A8),
     ($E5,$BB,$A9),($E5,$BB,$AC),($E5,$BB,$B1),($E5,$BB,$B3),
     ($E5,$BB,$B0),($E5,$BB,$B4),($E5,$BB,$B8),($E5,$BB,$BE),
     ($E5,$BC,$83),($E5,$BC,$89),($E5,$BD,$9D),($E5,$BD,$9C),
     ($E5,$BC,$8B),($E5,$BC,$91),($E5,$BC,$96),($E5,$BC,$A9),
     ($E5,$BC,$AD),($E5,$BC,$B8),($E5,$BD,$81),($E5,$BD,$88),
     ($E5,$BD,$8C),($E5,$BD,$8E),($E5,$BC,$AF),($E5,$BD,$91),
     ($E5,$BD,$96),($E5,$BD,$97),($E5,$BD,$99),($E5,$BD,$A1),
     ($E5,$BD,$AD),($E5,$BD,$B3),($E5,$BD,$B7),($E5,$BE,$83),
     ($E5,$BE,$82),($E5,$BD,$BF),($E5,$BE,$8A),($E5,$BE,$88),
     ($E5,$BE,$91),($E5,$BE,$87),($E5,$BE,$9E),($E5,$BE,$99),
     ($E5,$BE,$98),($E5,$BE,$A0),($E5,$BE,$A8),($E5,$BE,$AD),
     ($E5,$BE,$BC),($E5,$BF,$96),($E5,$BF,$BB),($E5,$BF,$A4),
     ($E5,$BF,$B8),($E5,$BF,$B1),($E5,$BF,$9D),($E6,$82,$B3),
     ($E5,$BF,$BF),($E6,$80,$A1),($E6,$81,$A0),($00,$00,$00),
     ($E6,$80,$99),($E6,$80,$90),($E6,$80,$A9),($E6,$80,$8E),
     ($E6,$80,$B1),($E6,$80,$9B),($E6,$80,$95),($E6,$80,$AB),
     ($E6,$80,$A6),($E6,$80,$8F),($E6,$80,$BA),($E6,$81,$9A),
     ($E6,$81,$81),($E6,$81,$AA),($E6,$81,$B7),($E6,$81,$9F),
     ($E6,$81,$8A),($E6,$81,$86),($E6,$81,$8D),($E6,$81,$A3),
     ($E6,$81,$83),($E6,$81,$A4),($E6,$81,$82),($E6,$81,$AC),
     ($E6,$81,$AB),($E6,$81,$99),($E6,$82,$81),($E6,$82,$8D),
     ($E6,$83,$A7),($E6,$82,$83),($E6,$82,$9A),($E6,$82,$84),
     ($E6,$82,$9B),($E6,$82,$96),($E6,$82,$97),($E6,$82,$92),
     ($E6,$82,$A7),($E6,$82,$8B),($E6,$83,$A1),($E6,$82,$B8),
     ($E6,$83,$A0),($E6,$83,$93),($E6,$82,$B4),($E5,$BF,$B0),
     ($E6,$82,$BD),($E6,$83,$86),($E6,$82,$B5),($E6,$83,$98),
     ($E6,$85,$8D),($E6,$84,$95),($E6,$84,$86),($E6,$83,$B6),
     ($E6,$83,$B7),($E6,$84,$80),($E6,$83,$B4),($E6,$83,$BA),
     ($E6,$84,$83),($E6,$84,$A1),($E6,$83,$BB),($E6,$83,$B1),
     ($E6,$84,$8D),($E6,$84,$8E),($E6,$85,$87),($E6,$84,$BE),
     ($E6,$84,$A8),($E6,$84,$A7),($E6,$85,$8A),($E6,$84,$BF),
     ($E6,$84,$BC),($E6,$84,$AC),($E6,$84,$B4),($E6,$84,$BD),
     ($E6,$85,$82),($E6,$85,$84),($E6,$85,$B3),($E6,$85,$B7),
     ($E6,$85,$98),($E6,$85,$99),($E6,$85,$9A),($E6,$85,$AB),
     ($E6,$85,$B4),($E6,$85,$AF),($E6,$85,$A5),($E6,$85,$B1),
     ($E6,$85,$9F),($E6,$85,$9D),($E6,$85,$93),($E6,$85,$B5),
     ($E6,$86,$99),($E6,$86,$96),($E6,$86,$87),($E6,$86,$AC),
     ($E6,$86,$94),($E6,$86,$9A),($E6,$86,$8A),($E6,$86,$91),
     ($E6,$86,$AB),($E6,$86,$AE),($E6,$87,$8C),($E6,$87,$8A),
     ($E6,$87,$89),($E6,$87,$B7),($E6,$87,$88),($E6,$87,$83),
     ($E6,$87,$86),($E6,$86,$BA),($E6,$87,$8B),($E7,$BD,$B9),
     ($E6,$87,$8D),($E6,$87,$A6),($E6,$87,$A3),($E6,$87,$B6),
     ($E6,$87,$BA),($E6,$87,$B4),($E6,$87,$BF),($E6,$87,$BD),
     ($E6,$87,$BC),($E6,$87,$BE),($E6,$88,$80),($E6,$88,$88),
     ($E6,$88,$89),($E6,$88,$8D),($E6,$88,$8C),($E6,$88,$94),
     ($E6,$88,$9B));

  S2U_9D: TSjisRowTable =
    (($E6,$88,$9E),($E6,$88,$A1),($E6,$88,$AA),($E6,$88,$AE),
     ($E6,$88,$B0),($E6,$88,$B2),($E6,$88,$B3),($E6,$89,$81),
     ($E6,$89,$8E),($E6,$89,$9E),($E6,$89,$A3),($E6,$89,$9B),
     ($E6,$89,$A0),($E6,$89,$A8),($E6,$89,$BC),($E6,$8A,$82),
     ($E6,$8A,$89),($E6,$89,$BE),($E6,$8A,$92),($E6,$8A,$93),
     ($E6,$8A,$96),($E6,$8B,$94),($E6,$8A,$83),($E6,$8A,$94),
     ($E6,$8B,$97),($E6,$8B,$91),($E6,$8A,$BB),($E6,$8B,$8F),
     ($E6,$8B,$BF),($E6,$8B,$86),($E6,$93,$94),($E6,$8B,$88),
     ($E6,$8B,$9C),($E6,$8B,$8C),($E6,$8B,$8A),($E6,$8B,$82),
     ($E6,$8B,$87),($E6,$8A,$9B),($E6,$8B,$89),($E6,$8C,$8C),
     ($E6,$8B,$AE),($E6,$8B,$B1),($E6,$8C,$A7),($E6,$8C,$82),
     ($E6,$8C,$88),($E6,$8B,$AF),($E6,$8B,$B5),($E6,$8D,$90),
     ($E6,$8C,$BE),($E6,$8D,$8D),($E6,$90,$9C),($E6,$8D,$8F),
     ($E6,$8E,$96),($E6,$8E,$8E),($E6,$8E,$80),($E6,$8E,$AB),
     ($E6,$8D,$B6),($E6,$8E,$A3),($E6,$8E,$8F),($E6,$8E,$89),
     ($E6,$8E,$9F),($E6,$8E,$B5),($E6,$8D,$AB),($00,$00,$00),
     ($E6,$8D,$A9),($E6,$8E,$BE),($E6,$8F,$A9),($E6,$8F,$80),
     ($E6,$8F,$86),($E6,$8F,$A3),($E6,$8F,$89),($E6,$8F,$92),
     ($E6,$8F,$B6),($E6,$8F,$84),($E6,$90,$96),($E6,$90,$B4),
     ($E6,$90,$86),($E6,$90,$93),($E6,$90,$A6),($E6,$90,$B6),
     ($E6,$94,$9D),($E6,$90,$97),($E6,$90,$A8),($E6,$90,$8F),
     ($E6,$91,$A7),($E6,$91,$AF),($E6,$91,$B6),($E6,$91,$8E),
     ($E6,$94,$AA),($E6,$92,$95),($E6,$92,$93),($E6,$92,$A5),
     ($E6,$92,$A9),($E6,$92,$88),($E6,$92,$BC),($E6,$93,$9A),
     ($E6,$93,$92),($E6,$93,$85),($E6,$93,$87),($E6,$92,$BB),
     ($E6,$93,$98),($E6,$93,$82),($E6,$93,$B1),($E6,$93,$A7),
     ($E8,$88,$89),($E6,$93,$A0),($E6,$93,$A1),($E6,$8A,$AC),
     ($E6,$93,$A3),($E6,$93,$AF),($E6,$94,$AC),($E6,$93,$B6),
     ($E6,$93,$B4),($E6,$93,$B2),($E6,$93,$BA),($E6,$94,$80),
     ($E6,$93,$BD),($E6,$94,$98),($E6,$94,$9C),($E6,$94,$85),
     ($E6,$94,$A4),($E6,$94,$A3),($E6,$94,$AB),($E6,$94,$B4),
     ($E6,$94,$B5),($E6,$94,$B7),($E6,$94,$B6),($E6,$94,$B8),
     ($E7,$95,$8B),($E6,$95,$88),($E6,$95,$96),($E6,$95,$95),
     ($E6,$95,$8D),($E6,$95,$98),($E6,$95,$9E),($E6,$95,$9D),
     ($E6,$95,$B2),($E6,$95,$B8),($E6,$96,$82),($E6,$96,$83),
     ($E8,$AE,$8A),($E6,$96,$9B),($E6,$96,$9F),($E6,$96,$AB),
     ($E6,$96,$B7),($E6,$97,$83),($E6,$97,$86),($E6,$97,$81),
     ($E6,$97,$84),($E6,$97,$8C),($E6,$97,$92),($E6,$97,$9B),
     ($E6,$97,$99),($E6,$97,$A0),($E6,$97,$A1),($E6,$97,$B1),
     ($E6,$9D,$B2),($E6,$98,$8A),($E6,$98,$83),($E6,$97,$BB),
     ($E6,$9D,$B3),($E6,$98,$B5),($E6,$98,$B6),($E6,$98,$B4),
     ($E6,$98,$9C),($E6,$99,$8F),($E6,$99,$84),($E6,$99,$89),
     ($E6,$99,$81),($E6,$99,$9E),($E6,$99,$9D),($E6,$99,$A4),
     ($E6,$99,$A7),($E6,$99,$A8),($E6,$99,$9F),($E6,$99,$A2),
     ($E6,$99,$B0),($E6,$9A,$83),($E6,$9A,$88),($E6,$9A,$8E),
     ($E6,$9A,$89),($E6,$9A,$84),($E6,$9A,$98),($E6,$9A,$9D),
     ($E6,$9B,$81),($E6,$9A,$B9),($E6,$9B,$89),($E6,$9A,$BE),
     ($E6,$9A,$BC));

  S2U_9E: TSjisRowTable =
    (($E6,$9B,$84),($E6,$9A,$B8),($E6,$9B,$96),($E6,$9B,$9A),
     ($E6,$9B,$A0),($E6,$98,$BF),($E6,$9B,$A6),($E6,$9B,$A9),
     ($E6,$9B,$B0),($E6,$9B,$B5),($E6,$9B,$B7),($E6,$9C,$8F),
     ($E6,$9C,$96),($E6,$9C,$9E),($E6,$9C,$A6),($E6,$9C,$A7),
     ($E9,$9C,$B8),($E6,$9C,$AE),($E6,$9C,$BF),($E6,$9C,$B6),
     ($E6,$9D,$81),($E6,$9C,$B8),($E6,$9C,$B7),($E6,$9D,$86),
     ($E6,$9D,$9E),($E6,$9D,$A0),($E6,$9D,$99),($E6,$9D,$A3),
     ($E6,$9D,$A4),($E6,$9E,$89),($E6,$9D,$B0),($E6,$9E,$A9),
     ($E6,$9D,$BC),($E6,$9D,$AA),($E6,$9E,$8C),($E6,$9E,$8B),
     ($E6,$9E,$A6),($E6,$9E,$A1),($E6,$9E,$85),($E6,$9E,$B7),
     ($E6,$9F,$AF),($E6,$9E,$B4),($E6,$9F,$AC),($E6,$9E,$B3),
     ($E6,$9F,$A9),($E6,$9E,$B8),($E6,$9F,$A4),($E6,$9F,$9E),
     ($E6,$9F,$9D),($E6,$9F,$A2),($E6,$9F,$AE),($E6,$9E,$B9),
     ($E6,$9F,$8E),($E6,$9F,$86),($E6,$9F,$A7),($E6,$AA,$9C),
     ($E6,$A0,$9E),($E6,$A1,$86),($E6,$A0,$A9),($E6,$A1,$80),
     ($E6,$A1,$8D),($E6,$A0,$B2),($E6,$A1,$8E),($00,$00,$00),
     ($E6,$A2,$B3),($E6,$A0,$AB),($E6,$A1,$99),($E6,$A1,$A3),
     ($E6,$A1,$B7),($E6,$A1,$BF),($E6,$A2,$9F),($E6,$A2,$8F),
     ($E6,$A2,$AD),($E6,$A2,$94),($E6,$A2,$9D),($E6,$A2,$9B),
     ($E6,$A2,$83),($E6,$AA,$AE),($E6,$A2,$B9),($E6,$A1,$B4),
     ($E6,$A2,$B5),($E6,$A2,$A0),($E6,$A2,$BA),($E6,$A4,$8F),
     ($E6,$A2,$8D),($E6,$A1,$BE),($E6,$A4,$81),($E6,$A3,$8A),
     ($E6,$A4,$88),($E6,$A3,$98),($E6,$A4,$A2),($E6,$A4,$A6),
     ($E6,$A3,$A1),($E6,$A4,$8C),($E6,$A3,$8D),($E6,$A3,$94),
     ($E6,$A3,$A7),($E6,$A3,$95),($E6,$A4,$B6),($E6,$A4,$92),
     ($E6,$A4,$84),($E6,$A3,$97),($E6,$A3,$A3),($E6,$A4,$A5),
     ($E6,$A3,$B9),($E6,$A3,$A0),($E6,$A3,$AF),($E6,$A4,$A8),
     ($E6,$A4,$AA),($E6,$A4,$9A),($E6,$A4,$A3),($E6,$A4,$A1),
     ($E6,$A3,$86),($E6,$A5,$B9),($E6,$A5,$B7),($E6,$A5,$9C),
     ($E6,$A5,$B8),($E6,$A5,$AB),($E6,$A5,$94),($E6,$A5,$BE),
     ($E6,$A5,$AE),($E6,$A4,$B9),($E6,$A5,$B4),($E6,$A4,$BD),
     ($E6,$A5,$99),($E6,$A4,$B0),($E6,$A5,$A1),($E6,$A5,$9E),
     ($E6,$A5,$9D),($E6,$A6,$81),($E6,$A5,$AA),($E6,$A6,$B2),
     ($E6,$A6,$AE),($E6,$A7,$90),($E6,$A6,$BF),($E6,$A7,$81),
     ($E6,$A7,$93),($E6,$A6,$BE),($E6,$A7,$8E),($E5,$AF,$A8),
     ($E6,$A7,$8A),($E6,$A7,$9D),($E6,$A6,$BB),($E6,$A7,$83),
     ($E6,$A6,$A7),($E6,$A8,$AE),($E6,$A6,$91),($E6,$A6,$A0),
     ($E6,$A6,$9C),($E6,$A6,$95),($E6,$A6,$B4),($E6,$A7,$9E),
     ($E6,$A7,$A8),($E6,$A8,$82),($E6,$A8,$9B),($E6,$A7,$BF),
     ($E6,$AC,$8A),($E6,$A7,$B9),($E6,$A7,$B2),($E6,$A7,$A7),
     ($E6,$A8,$85),($E6,$A6,$B1),($E6,$A8,$9E),($E6,$A7,$AD),
     ($E6,$A8,$94),($E6,$A7,$AB),($E6,$A8,$8A),($E6,$A8,$92),
     ($E6,$AB,$81),($E6,$A8,$A3),($E6,$A8,$93),($E6,$A9,$84),
     ($E6,$A8,$8C),($E6,$A9,$B2),($E6,$A8,$B6),($E6,$A9,$B8),
     ($E6,$A9,$87),($E6,$A9,$A2),($E6,$A9,$99),($E6,$A9,$A6),
     ($E6,$A9,$88),($E6,$A8,$B8),($E6,$A8,$A2),($E6,$AA,$90),
     ($E6,$AA,$8D),($E6,$AA,$A0),($E6,$AA,$84),($E6,$AA,$A2),
     ($E6,$AA,$A3));

  S2U_9F: TSjisRowTable =
    (($E6,$AA,$97),($E8,$98,$97),($E6,$AA,$BB),($E6,$AB,$83),
     ($E6,$AB,$82),($E6,$AA,$B8),($E6,$AA,$B3),($E6,$AA,$AC),
     ($E6,$AB,$9E),($E6,$AB,$91),($E6,$AB,$9F),($E6,$AA,$AA),
     ($E6,$AB,$9A),($E6,$AB,$AA),($E6,$AB,$BB),($E6,$AC,$85),
     ($E8,$98,$96),($E6,$AB,$BA),($E6,$AC,$92),($E6,$AC,$96),
     ($E9,$AC,$B1),($E6,$AC,$9F),($E6,$AC,$B8),($E6,$AC,$B7),
     ($E7,$9B,$9C),($E6,$AC,$B9),($E9,$A3,$AE),($E6,$AD,$87),
     ($E6,$AD,$83),($E6,$AD,$89),($E6,$AD,$90),($E6,$AD,$99),
     ($E6,$AD,$94),($E6,$AD,$9B),($E6,$AD,$9F),($E6,$AD,$A1),
     ($E6,$AD,$B8),($E6,$AD,$B9),($E6,$AD,$BF),($E6,$AE,$80),
     ($E6,$AE,$84),($E6,$AE,$83),($E6,$AE,$8D),($E6,$AE,$98),
     ($E6,$AE,$95),($E6,$AE,$9E),($E6,$AE,$A4),($E6,$AE,$AA),
     ($E6,$AE,$AB),($E6,$AE,$AF),($E6,$AE,$B2),($E6,$AE,$B1),
     ($E6,$AE,$B3),($E6,$AE,$B7),($E6,$AE,$BC),($E6,$AF,$86),
     ($E6,$AF,$8B),($E6,$AF,$93),($E6,$AF,$9F),($E6,$AF,$AC),
     ($E6,$AF,$AB),($E6,$AF,$B3),($E6,$AF,$AF),($00,$00,$00),
     ($E9,$BA,$BE),($E6,$B0,$88),($E6,$B0,$93),($E6,$B0,$94),
     ($E6,$B0,$9B),($E6,$B0,$A4),($E6,$B0,$A3),($E6,$B1,$9E),
     ($E6,$B1,$95),($E6,$B1,$A2),($E6,$B1,$AA),($E6,$B2,$82),
     ($E6,$B2,$8D),($E6,$B2,$9A),($E6,$B2,$81),($E6,$B2,$9B),
     ($E6,$B1,$BE),($E6,$B1,$A8),($E6,$B1,$B3),($E6,$B2,$92),
     ($E6,$B2,$90),($E6,$B3,$84),($E6,$B3,$B1),($E6,$B3,$93),
     ($E6,$B2,$BD),($E6,$B3,$97),($E6,$B3,$85),($E6,$B3,$9D),
     ($E6,$B2,$AE),($E6,$B2,$B1),($E6,$B2,$BE),($E6,$B2,$BA),
     ($E6,$B3,$9B),($E6,$B3,$AF),($E6,$B3,$99),($E6,$B3,$AA),
     ($E6,$B4,$9F),($E8,$A1,$8D),($E6,$B4,$B6),($E6,$B4,$AB),
     ($E6,$B4,$BD),($E6,$B4,$B8),($E6,$B4,$99),($E6,$B4,$B5),
     ($E6,$B4,$B3),($E6,$B4,$92),($E6,$B4,$8C),($E6,$B5,$A3),
     ($E6,$B6,$93),($E6,$B5,$A4),($E6,$B5,$9A),($E6,$B5,$B9),
     ($E6,$B5,$99),($E6,$B6,$8E),($E6,$B6,$95),($E6,$BF,$A4),
     ($E6,$B6,$85),($E6,$B7,$B9),($E6,$B8,$95),($E6,$B8,$8A),
     ($E6,$B6,$B5),($E6,$B7,$87),($E6,$B7,$A6),($E6,$B6,$B8),
     ($E6,$B7,$86),($E6,$B7,$AC),($E6,$B7,$9E),($E6,$B7,$8C),
     ($E6,$B7,$A8),($E6,$B7,$92),($E6,$B7,$85),($E6,$B7,$BA),
     ($E6,$B7,$99),($E6,$B7,$A4),($E6,$B7,$95),($E6,$B7,$AA),
     ($E6,$B7,$AE),($E6,$B8,$AD),($E6,$B9,$AE),($E6,$B8,$AE),
     ($E6,$B8,$99),($E6,$B9,$B2),($E6,$B9,$9F),($E6,$B8,$BE),
     ($E6,$B8,$A3),($E6,$B9,$AB),($E6,$B8,$AB),($E6,$B9,$B6),
     ($E6,$B9,$8D),($E6,$B8,$9F),($E6,$B9,$83),($E6,$B8,$BA),
     ($E6,$B9,$8E),($E6,$B8,$A4),($E6,$BB,$BF),($E6,$B8,$9D),
     ($E6,$B8,$B8),($E6,$BA,$82),($E6,$BA,$AA),($E6,$BA,$98),
     ($E6,$BB,$89),($E6,$BA,$B7),($E6,$BB,$93),($E6,$BA,$BD),
     ($E6,$BA,$AF),($E6,$BB,$84),($E6,$BA,$B2),($E6,$BB,$94),
     ($E6,$BB,$95),($E6,$BA,$8F),($E6,$BA,$A5),($E6,$BB,$82),
     ($E6,$BA,$9F),($E6,$BD,$81),($E6,$BC,$91),($E7,$81,$8C),
     ($E6,$BB,$AC),($E6,$BB,$B8),($E6,$BB,$BE),($E6,$BC,$BF),
     ($E6,$BB,$B2),($E6,$BC,$B1),($E6,$BB,$AF),($E6,$BC,$B2),
     ($E6,$BB,$8C));

  S2U_E0: TSjisRowTable =
    (($E6,$BC,$BE),($E6,$BC,$93),($E6,$BB,$B7),($E6,$BE,$86),
     ($E6,$BD,$BA),($E6,$BD,$B8),($E6,$BE,$81),($E6,$BE,$80),
     ($E6,$BD,$AF),($E6,$BD,$9B),($E6,$BF,$B3),($E6,$BD,$AD),
     ($E6,$BE,$82),($E6,$BD,$BC),($E6,$BD,$98),($E6,$BE,$8E),
     ($E6,$BE,$91),($E6,$BF,$82),($E6,$BD,$A6),($E6,$BE,$B3),
     ($E6,$BE,$A3),($E6,$BE,$A1),($E6,$BE,$A4),($E6,$BE,$B9),
     ($E6,$BF,$86),($E6,$BE,$AA),($E6,$BF,$9F),($E6,$BF,$95),
     ($E6,$BF,$AC),($E6,$BF,$94),($E6,$BF,$98),($E6,$BF,$B1),
     ($E6,$BF,$AE),($E6,$BF,$9B),($E7,$80,$89),($E7,$80,$8B),
     ($E6,$BF,$BA),($E7,$80,$91),($E7,$80,$81),($E7,$80,$8F),
     ($E6,$BF,$BE),($E7,$80,$9B),($E7,$80,$9A),($E6,$BD,$B4),
     ($E7,$80,$9D),($E7,$80,$98),($E7,$80,$9F),($E7,$80,$B0),
     ($E7,$80,$BE),($E7,$80,$B2),($E7,$81,$91),($E7,$81,$A3),
     ($E7,$82,$99),($E7,$82,$92),($E7,$82,$AF),($E7,$83,$B1),
     ($E7,$82,$AC),($E7,$82,$B8),($E7,$82,$B3),($E7,$82,$AE),
     ($E7,$83,$9F),($E7,$83,$8B),($E7,$83,$9D),($00,$00,$00),
     ($E7,$83,$99),($E7,$84,$89),($E7,$83,$BD),($E7,$84,$9C),
     ($E7,$84,$99),($E7,$85,$A5),($E7,$85,$95),($E7,$86,$88),
     ($E7,$85,$A6),($E7,$85,$A2),($E7,$85,$8C),($E7,$85,$96),
     ($E7,$85,$AC),($E7,$86,$8F),($E7,$87,$BB),($E7,$86,$84),
     ($E7,$86,$95),($E7,$86,$A8),($E7,$86,$AC),($E7,$87,$97),
     ($E7,$86,$B9),($E7,$86,$BE),($E7,$87,$92),($E7,$87,$89),
     ($E7,$87,$94),($E7,$87,$8E),($E7,$87,$A0),($E7,$87,$AC),
     ($E7,$87,$A7),($E7,$87,$B5),($E7,$87,$BC),($E7,$87,$B9),
     ($E7,$87,$BF),($E7,$88,$8D),($E7,$88,$90),($E7,$88,$9B),
     ($E7,$88,$A8),($E7,$88,$AD),($E7,$88,$AC),($E7,$88,$B0),
     ($E7,$88,$B2),($E7,$88,$BB),($E7,$88,$BC),($E7,$88,$BF),
     ($E7,$89,$80),($E7,$89,$86),($E7,$89,$8B),($E7,$89,$98),
     ($E7,$89,$B4),($E7,$89,$BE),($E7,$8A,$82),($E7,$8A,$81),
     ($E7,$8A,$87),($E7,$8A,$92),($E7,$8A,$96),($E7,$8A,$A2),
     ($E7,$8A,$A7),($E7,$8A,$B9),($E7,$8A,$B2),($E7,$8B,$83),
     ($E7,$8B,$86),($E7,$8B,$84),($E7,$8B,$8E),($E7,$8B,$92),
     ($E7,$8B,$A2),($E7,$8B,$A0),($E7,$8B,$A1),($E7,$8B,$B9),
     ($E7,$8B,$B7),($E5,$80,$8F),($E7,$8C,$97),($E7,$8C,$8A),
     ($E7,$8C,$9C),($E7,$8C,$96),($E7,$8C,$9D),($E7,$8C,$B4),
     ($E7,$8C,$AF),($E7,$8C,$A9),($E7,$8C,$A5),($E7,$8C,$BE),
     ($E7,$8D,$8E),($E7,$8D,$8F),($E9,$BB,$98),($E7,$8D,$97),
     ($E7,$8D,$AA),($E7,$8D,$A8),($E7,$8D,$B0),($E7,$8D,$B8),
     ($E7,$8D,$B5),($E7,$8D,$BB),($E7,$8D,$BA),($E7,$8F,$88),
     ($E7,$8E,$B3),($E7,$8F,$8E),($E7,$8E,$BB),($E7,$8F,$80),
     ($E7,$8F,$A5),($E7,$8F,$AE),($E7,$8F,$9E),($E7,$92,$A2),
     ($E7,$90,$85),($E7,$91,$AF),($E7,$90,$A5),($E7,$8F,$B8),
     ($E7,$90,$B2),($E7,$90,$BA),($E7,$91,$95),($E7,$90,$BF),
     ($E7,$91,$9F),($E7,$91,$99),($E7,$91,$81),($E7,$91,$9C),
     ($E7,$91,$A9),($E7,$91,$B0),($E7,$91,$A3),($E7,$91,$AA),
     ($E7,$91,$B6),($E7,$91,$BE),($E7,$92,$8B),($E7,$92,$9E),
     ($E7,$92,$A7),($E7,$93,$8A),($E7,$93,$8F),($E7,$93,$94),
     ($E7,$8F,$B1));

  S2U_E1: TSjisRowTable =
    (($E7,$93,$A0),($E7,$93,$A3),($E7,$93,$A7),($E7,$93,$A9),
     ($E7,$93,$AE),($E7,$93,$B2),($E7,$93,$B0),($E7,$93,$B1),
     ($E7,$93,$B8),($E7,$93,$B7),($E7,$94,$84),($E7,$94,$83),
     ($E7,$94,$85),($E7,$94,$8C),($E7,$94,$8E),($E7,$94,$8D),
     ($E7,$94,$95),($E7,$94,$93),($E7,$94,$9E),($E7,$94,$A6),
     ($E7,$94,$AC),($E7,$94,$BC),($E7,$95,$84),($E7,$95,$8D),
     ($E7,$95,$8A),($E7,$95,$89),($E7,$95,$9B),($E7,$95,$86),
     ($E7,$95,$9A),($E7,$95,$A9),($E7,$95,$A4),($E7,$95,$A7),
     ($E7,$95,$AB),($E7,$95,$AD),($E7,$95,$B8),($E7,$95,$B6),
     ($E7,$96,$86),($E7,$96,$87),($E7,$95,$B4),($E7,$96,$8A),
     ($E7,$96,$89),($E7,$96,$82),($E7,$96,$94),($E7,$96,$9A),
     ($E7,$96,$9D),($E7,$96,$A5),($E7,$96,$A3),($E7,$97,$82),
     ($E7,$96,$B3),($E7,$97,$83),($E7,$96,$B5),($E7,$96,$BD),
     ($E7,$96,$B8),($E7,$96,$BC),($E7,$96,$B1),($E7,$97,$8D),
     ($E7,$97,$8A),($E7,$97,$92),($E7,$97,$99),($E7,$97,$A3),
     ($E7,$97,$9E),($E7,$97,$BE),($E7,$97,$BF),($00,$00,$00),
     ($E7,$97,$BC),($E7,$98,$81),($E7,$97,$B0),($E7,$97,$BA),
     ($E7,$97,$B2),($E7,$97,$B3),($E7,$98,$8B),($E7,$98,$8D),
     ($E7,$98,$89),($E7,$98,$9F),($E7,$98,$A7),($E7,$98,$A0),
     ($E7,$98,$A1),($E7,$98,$A2),($E7,$98,$A4),($E7,$98,$B4),
     ($E7,$98,$B0),($E7,$98,$BB),($E7,$99,$87),($E7,$99,$88),
     ($E7,$99,$86),($E7,$99,$9C),($E7,$99,$98),($E7,$99,$A1),
     ($E7,$99,$A2),($E7,$99,$A8),($E7,$99,$A9),($E7,$99,$AA),
     ($E7,$99,$A7),($E7,$99,$AC),($E7,$99,$B0),($E7,$99,$B2),
     ($E7,$99,$B6),($E7,$99,$B8),($E7,$99,$BC),($E7,$9A,$80),
     ($E7,$9A,$83),($E7,$9A,$88),($E7,$9A,$8B),($E7,$9A,$8E),
     ($E7,$9A,$96),($E7,$9A,$93),($E7,$9A,$99),($E7,$9A,$9A),
     ($E7,$9A,$B0),($E7,$9A,$B4),($E7,$9A,$B8),($E7,$9A,$B9),
     ($E7,$9A,$BA),($E7,$9B,$82),($E7,$9B,$8D),($E7,$9B,$96),
     ($E7,$9B,$92),($E7,$9B,$9E),($E7,$9B,$A1),($E7,$9B,$A5),
     ($E7,$9B,$A7),($E7,$9B,$AA),($E8,$98,$AF),($E7,$9B,$BB),
     ($E7,$9C,$88),($E7,$9C,$87),($E7,$9C,$84),($E7,$9C,$A9),
     ($E7,$9C,$A4),($E7,$9C,$9E),($E7,$9C,$A5),($E7,$9C,$A6),
     ($E7,$9C,$9B),($E7,$9C,$B7),($E7,$9C,$B8),($E7,$9D,$87),
     ($E7,$9D,$9A),($E7,$9D,$A8),($E7,$9D,$AB),($E7,$9D,$9B),
     ($E7,$9D,$A5),($E7,$9D,$BF),($E7,$9D,$BE),($E7,$9D,$B9),
     ($E7,$9E,$8E),($E7,$9E,$8B),($E7,$9E,$91),($E7,$9E,$A0),
     ($E7,$9E,$9E),($E7,$9E,$B0),($E7,$9E,$B6),($E7,$9E,$B9),
     ($E7,$9E,$BF),($E7,$9E,$BC),($E7,$9E,$BD),($E7,$9E,$BB),
     ($E7,$9F,$87),($E7,$9F,$8D),($E7,$9F,$97),($E7,$9F,$9A),
     ($E7,$9F,$9C),($E7,$9F,$A3),($E7,$9F,$AE),($E7,$9F,$BC),
     ($E7,$A0,$8C),($E7,$A0,$92),($E7,$A4,$A6),($E7,$A0,$A0),
     ($E7,$A4,$AA),($E7,$A1,$85),($E7,$A2,$8E),($E7,$A1,$B4),
     ($E7,$A2,$86),($E7,$A1,$BC),($E7,$A2,$9A),($E7,$A2,$8C),
     ($E7,$A2,$A3),($E7,$A2,$B5),($E7,$A2,$AA),($E7,$A2,$AF),
     ($E7,$A3,$91),($E7,$A3,$86),($E7,$A3,$8B),($E7,$A3,$94),
     ($E7,$A2,$BE),($E7,$A2,$BC),($E7,$A3,$85),($E7,$A3,$8A),
     ($E7,$A3,$AC));

  S2U_E2: TSjisRowTable =
    (($E7,$A3,$A7),($E7,$A3,$9A),($E7,$A3,$BD),($E7,$A3,$B4),
     ($E7,$A4,$87),($E7,$A4,$92),($E7,$A4,$91),($E7,$A4,$99),
     ($E7,$A4,$AC),($E7,$A4,$AB),($E7,$A5,$80),($E7,$A5,$A0),
     ($E7,$A5,$97),($E7,$A5,$9F),($E7,$A5,$9A),($E7,$A5,$95),
     ($E7,$A5,$93),($E7,$A5,$BA),($E7,$A5,$BF),($E7,$A6,$8A),
     ($E7,$A6,$9D),($E7,$A6,$A7),($E9,$BD,$8B),($E7,$A6,$AA),
     ($E7,$A6,$AE),($E7,$A6,$B3),($E7,$A6,$B9),($E7,$A6,$BA),
     ($E7,$A7,$89),($E7,$A7,$95),($E7,$A7,$A7),($E7,$A7,$AC),
     ($E7,$A7,$A1),($E7,$A7,$A3),($E7,$A8,$88),($E7,$A8,$8D),
     ($E7,$A8,$98),($E7,$A8,$99),($E7,$A8,$A0),($E7,$A8,$9F),
     ($E7,$A6,$80),($E7,$A8,$B1),($E7,$A8,$BB),($E7,$A8,$BE),
     ($E7,$A8,$B7),($E7,$A9,$83),($E7,$A9,$97),($E7,$A9,$89),
     ($E7,$A9,$A1),($E7,$A9,$A2),($E7,$A9,$A9),($E9,$BE,$9D),
     ($E7,$A9,$B0),($E7,$A9,$B9),($E7,$A9,$BD),($E7,$AA,$88),
     ($E7,$AA,$97),($E7,$AA,$95),($E7,$AA,$98),($E7,$AA,$96),
     ($E7,$AA,$A9),($E7,$AB,$88),($E7,$AA,$B0),($00,$00,$00),
     ($E7,$AA,$B6),($E7,$AB,$85),($E7,$AB,$84),($E7,$AA,$BF),
     ($E9,$82,$83),($E7,$AB,$87),($E7,$AB,$8A),($E7,$AB,$8D),
     ($E7,$AB,$8F),($E7,$AB,$95),($E7,$AB,$93),($E7,$AB,$99),
     ($E7,$AB,$9A),($E7,$AB,$9D),($E7,$AB,$A1),($E7,$AB,$A2),
     ($E7,$AB,$A6),($E7,$AB,$AD),($E7,$AB,$B0),($E7,$AC,$82),
     ($E7,$AC,$8F),($E7,$AC,$8A),($E7,$AC,$86),($E7,$AC,$B3),
     ($E7,$AC,$98),($E7,$AC,$99),($E7,$AC,$9E),($E7,$AC,$B5),
     ($E7,$AC,$A8),($E7,$AC,$B6),($E7,$AD,$90),($E7,$AD,$BA),
     ($E7,$AC,$84),($E7,$AD,$8D),($E7,$AC,$8B),($E7,$AD,$8C),
     ($E7,$AD,$85),($E7,$AD,$B5),($E7,$AD,$A5),($E7,$AD,$B4),
     ($E7,$AD,$A7),($E7,$AD,$B0),($E7,$AD,$B1),($E7,$AD,$AC),
     ($E7,$AD,$AE),($E7,$AE,$9D),($E7,$AE,$98),($E7,$AE,$9F),
     ($E7,$AE,$8D),($E7,$AE,$9C),($E7,$AE,$9A),($E7,$AE,$8B),
     ($E7,$AE,$92),($E7,$AE,$8F),($E7,$AD,$9D),($E7,$AE,$99),
     ($E7,$AF,$8B),($E7,$AF,$81),($E7,$AF,$8C),($E7,$AF,$8F),
     ($E7,$AE,$B4),($E7,$AF,$86),($E7,$AF,$9D),($E7,$AF,$A9),
     ($E7,$B0,$91),($E7,$B0,$94),($E7,$AF,$A6),($E7,$AF,$A5),
     ($E7,$B1,$A0),($E7,$B0,$80),($E7,$B0,$87),($E7,$B0,$93),
     ($E7,$AF,$B3),($E7,$AF,$B7),($E7,$B0,$97),($E7,$B0,$8D),
     ($E7,$AF,$B6),($E7,$B0,$A3),($E7,$B0,$A7),($E7,$B0,$AA),
     ($E7,$B0,$9F),($E7,$B0,$B7),($E7,$B0,$AB),($E7,$B0,$BD),
     ($E7,$B1,$8C),($E7,$B1,$83),($E7,$B1,$94),($E7,$B1,$8F),
     ($E7,$B1,$80),($E7,$B1,$90),($E7,$B1,$98),($E7,$B1,$9F),
     ($E7,$B1,$A4),($E7,$B1,$96),($E7,$B1,$A5),($E7,$B1,$AC),
     ($E7,$B1,$B5),($E7,$B2,$83),($E7,$B2,$90),($E7,$B2,$A4),
     ($E7,$B2,$AD),($E7,$B2,$A2),($E7,$B2,$AB),($E7,$B2,$A1),
     ($E7,$B2,$A8),($E7,$B2,$B3),($E7,$B2,$B2),($E7,$B2,$B1),
     ($E7,$B2,$AE),($E7,$B2,$B9),($E7,$B2,$BD),($E7,$B3,$80),
     ($E7,$B3,$85),($E7,$B3,$82),($E7,$B3,$98),($E7,$B3,$92),
     ($E7,$B3,$9C),($E7,$B3,$A2),($E9,$AC,$BB),($E7,$B3,$AF),
     ($E7,$B3,$B2),($E7,$B3,$B4),($E7,$B3,$B6),($E7,$B3,$BA),
     ($E7,$B4,$86));

  S2U_E3: TSjisRowTable =
    (($E7,$B4,$82),($E7,$B4,$9C),($E7,$B4,$95),($E7,$B4,$8A),
     ($E7,$B5,$85),($E7,$B5,$8B),($E7,$B4,$AE),($E7,$B4,$B2),
     ($E7,$B4,$BF),($E7,$B4,$B5),($E7,$B5,$86),($E7,$B5,$B3),
     ($E7,$B5,$96),($E7,$B5,$8E),($E7,$B5,$B2),($E7,$B5,$A8),
     ($E7,$B5,$AE),($E7,$B5,$8F),($E7,$B5,$A3),($E7,$B6,$93),
     ($E7,$B6,$89),($E7,$B5,$9B),($E7,$B6,$8F),($E7,$B5,$BD),
     ($E7,$B6,$9B),($E7,$B6,$BA),($E7,$B6,$AE),($E7,$B6,$A3),
     ($E7,$B6,$B5),($E7,$B7,$87),($E7,$B6,$BD),($E7,$B6,$AB),
     ($E7,$B8,$BD),($E7,$B6,$A2),($E7,$B6,$AF),($E7,$B7,$9C),
     ($E7,$B6,$B8),($E7,$B6,$9F),($E7,$B6,$B0),($E7,$B7,$98),
     ($E7,$B7,$9D),($E7,$B7,$A4),($E7,$B7,$9E),($E7,$B7,$BB),
     ($E7,$B7,$B2),($E7,$B7,$A1),($E7,$B8,$85),($E7,$B8,$8A),
     ($E7,$B8,$A3),($E7,$B8,$A1),($E7,$B8,$92),($E7,$B8,$B1),
     ($E7,$B8,$9F),($E7,$B8,$89),($E7,$B8,$8B),($E7,$B8,$A2),
     ($E7,$B9,$86),($E7,$B9,$A6),($E7,$B8,$BB),($E7,$B8,$B5),
     ($E7,$B8,$B9),($E7,$B9,$83),($E7,$B8,$B7),($00,$00,$00),
     ($E7,$B8,$B2),($E7,$B8,$BA),($E7,$B9,$A7),($E7,$B9,$9D),
     ($E7,$B9,$96),($E7,$B9,$9E),($E7,$B9,$99),($E7,$B9,$9A),
     ($E7,$B9,$B9),($E7,$B9,$AA),($E7,$B9,$A9),($E7,$B9,$BC),
     ($E7,$B9,$BB),($E7,$BA,$83),($E7,$B7,$95),($E7,$B9,$BD),
     ($E8,$BE,$AE),($E7,$B9,$BF),($E7,$BA,$88),($E7,$BA,$89),
     ($E7,$BA,$8C),($E7,$BA,$92),($E7,$BA,$90),($E7,$BA,$93),
     ($E7,$BA,$94),($E7,$BA,$96),($E7,$BA,$8E),($E7,$BA,$9B),
     ($E7,$BA,$9C),($E7,$BC,$B8),($E7,$BC,$BA),($E7,$BD,$85),
     ($E7,$BD,$8C),($E7,$BD,$8D),($E7,$BD,$8E),($E7,$BD,$90),
     ($E7,$BD,$91),($E7,$BD,$95),($E7,$BD,$94),($E7,$BD,$98),
     ($E7,$BD,$9F),($E7,$BD,$A0),($E7,$BD,$A8),($E7,$BD,$A9),
     ($E7,$BD,$A7),($E7,$BD,$B8),($E7,$BE,$82),($E7,$BE,$86),
     ($E7,$BE,$83),($E7,$BE,$88),($E7,$BE,$87),($E7,$BE,$8C),
     ($E7,$BE,$94),($E7,$BE,$9E),($E7,$BE,$9D),($E7,$BE,$9A),
     ($E7,$BE,$A3),($E7,$BE,$AF),($E7,$BE,$B2),($E7,$BE,$B9),
     ($E7,$BE,$AE),($E7,$BE,$B6),($E7,$BE,$B8),($E8,$AD,$B1),
     ($E7,$BF,$85),($E7,$BF,$86),($E7,$BF,$8A),($E7,$BF,$95),
     ($E7,$BF,$94),($E7,$BF,$A1),($E7,$BF,$A6),($E7,$BF,$A9),
     ($E7,$BF,$B3),($E7,$BF,$B9),($E9,$A3,$9C),($E8,$80,$86),
     ($E8,$80,$84),($E8,$80,$8B),($E8,$80,$92),($E8,$80,$98),
     ($E8,$80,$99),($E8,$80,$9C),($E8,$80,$A1),($E8,$80,$A8),
     ($E8,$80,$BF),($E8,$80,$BB),($E8,$81,$8A),($E8,$81,$86),
     ($E8,$81,$92),($E8,$81,$98),($E8,$81,$9A),($E8,$81,$9F),
     ($E8,$81,$A2),($E8,$81,$A8),($E8,$81,$B3),($E8,$81,$B2),
     ($E8,$81,$B0),($E8,$81,$B6),($E8,$81,$B9),($E8,$81,$BD),
     ($E8,$81,$BF),($E8,$82,$84),($E8,$82,$86),($E8,$82,$85),
     ($E8,$82,$9B),($E8,$82,$93),($E8,$82,$9A),($E8,$82,$AD),
     ($E5,$86,$90),($E8,$82,$AC),($E8,$83,$9B),($E8,$83,$A5),
     ($E8,$83,$99),($E8,$83,$9D),($E8,$83,$84),($E8,$83,$9A),
     ($E8,$83,$96),($E8,$84,$89),($E8,$83,$AF),($E8,$83,$B1),
     ($E8,$84,$9B),($E8,$84,$A9),($E8,$84,$A3),($E8,$84,$AF),
     ($E8,$85,$8B));

  S2U_E4: TSjisRowTable =
    (($E9,$9A,$8B),($E8,$85,$86),($E8,$84,$BE),($E8,$85,$93),
     ($E8,$85,$91),($E8,$83,$BC),($E8,$85,$B1),($E8,$85,$AE),
     ($E8,$85,$A5),($E8,$85,$A6),($E8,$85,$B4),($E8,$86,$83),
     ($E8,$86,$88),($E8,$86,$8A),($E8,$86,$80),($E8,$86,$82),
     ($E8,$86,$A0),($E8,$86,$95),($E8,$86,$A4),($E8,$86,$A3),
     ($E8,$85,$9F),($E8,$86,$93),($E8,$86,$A9),($E8,$86,$B0),
     ($E8,$86,$B5),($E8,$86,$BE),($E8,$86,$B8),($E8,$86,$BD),
     ($E8,$87,$80),($E8,$87,$82),($E8,$86,$BA),($E8,$87,$89),
     ($E8,$87,$8D),($E8,$87,$91),($E8,$87,$99),($E8,$87,$98),
     ($E8,$87,$88),($E8,$87,$9A),($E8,$87,$9F),($E8,$87,$A0),
     ($E8,$87,$A7),($E8,$87,$BA),($E8,$87,$BB),($E8,$87,$BE),
     ($E8,$88,$81),($E8,$88,$82),($E8,$88,$85),($E8,$88,$87),
     ($E8,$88,$8A),($E8,$88,$8D),($E8,$88,$90),($E8,$88,$96),
     ($E8,$88,$A9),($E8,$88,$AB),($E8,$88,$B8),($E8,$88,$B3),
     ($E8,$89,$80),($E8,$89,$99),($E8,$89,$98),($E8,$89,$9D),
     ($E8,$89,$9A),($E8,$89,$9F),($E8,$89,$A4),($00,$00,$00),
     ($E8,$89,$A2),($E8,$89,$A8),($E8,$89,$AA),($E8,$89,$AB),
     ($E8,$88,$AE),($E8,$89,$B1),($E8,$89,$B7),($E8,$89,$B8),
     ($E8,$89,$BE),($E8,$8A,$8D),($E8,$8A,$92),($E8,$8A,$AB),
     ($E8,$8A,$9F),($E8,$8A,$BB),($E8,$8A,$AC),($E8,$8B,$A1),
     ($E8,$8B,$A3),($E8,$8B,$9F),($E8,$8B,$92),($E8,$8B,$B4),
     ($E8,$8B,$B3),($E8,$8B,$BA),($E8,$8E,$93),($E8,$8C,$83),
     ($E8,$8B,$BB),($E8,$8B,$B9),($E8,$8B,$9E),($E8,$8C,$86),
     ($E8,$8B,$9C),($E8,$8C,$89),($E8,$8B,$99),($E8,$8C,$B5),
     ($E8,$8C,$B4),($E8,$8C,$96),($E8,$8C,$B2),($E8,$8C,$B1),
     ($E8,$8D,$80),($E8,$8C,$B9),($E8,$8D,$90),($E8,$8D,$85),
     ($E8,$8C,$AF),($E8,$8C,$AB),($E8,$8C,$97),($E8,$8C,$98),
     ($E8,$8E,$85),($E8,$8E,$9A),($E8,$8E,$AA),($E8,$8E,$9F),
     ($E8,$8E,$A2),($E8,$8E,$96),($E8,$8C,$A3),($E8,$8E,$8E),
     ($E8,$8E,$87),($E8,$8E,$8A),($E8,$8D,$BC),($E8,$8E,$B5),
     ($E8,$8D,$B3),($E8,$8D,$B5),($E8,$8E,$A0),($E8,$8E,$89),
     ($E8,$8E,$A8),($E8,$8F,$B4),($E8,$90,$93),($E8,$8F,$AB),
     ($E8,$8F,$8E),($E8,$8F,$BD),($E8,$90,$83),($E8,$8F,$98),
     ($E8,$90,$8B),($E8,$8F,$81),($E8,$8F,$B7),($E8,$90,$87),
     ($E8,$8F,$A0),($E8,$8F,$B2),($E8,$90,$8D),($E8,$90,$A2),
     ($E8,$90,$A0),($E8,$8E,$BD),($E8,$90,$B8),($E8,$94,$86),
     ($E8,$8F,$BB),($E8,$91,$AD),($E8,$90,$AA),($E8,$90,$BC),
     ($E8,$95,$9A),($E8,$92,$84),($E8,$91,$B7),($E8,$91,$AB),
     ($E8,$92,$AD),($E8,$91,$AE),($E8,$92,$82),($E8,$91,$A9),
     ($E8,$91,$86),($E8,$90,$AC),($E8,$91,$AF),($E8,$91,$B9),
     ($E8,$90,$B5),($E8,$93,$8A),($E8,$91,$A2),($E8,$92,$B9),
     ($E8,$92,$BF),($E8,$92,$9F),($E8,$93,$99),($E8,$93,$8D),
     ($E8,$92,$BB),($E8,$93,$9A),($E8,$93,$90),($E8,$93,$81),
     ($E8,$93,$86),($E8,$93,$96),($E8,$92,$A1),($E8,$94,$A1),
     ($E8,$93,$BF),($E8,$93,$B4),($E8,$94,$97),($E8,$94,$98),
     ($E8,$94,$AC),($E8,$94,$9F),($E8,$94,$95),($E8,$94,$94),
     ($E8,$93,$BC),($E8,$95,$80),($E8,$95,$A3),($E8,$95,$98),
     ($E8,$95,$88));

  S2U_E5: TSjisRowTable =
    (($E8,$95,$81),($E8,$98,$82),($E8,$95,$8B),($E8,$95,$95),
     ($E8,$96,$80),($E8,$96,$A4),($E8,$96,$88),($E8,$96,$91),
     ($E8,$96,$8A),($E8,$96,$A8),($E8,$95,$AD),($E8,$96,$94),
     ($E8,$96,$9B),($E8,$97,$AA),($E8,$96,$87),($E8,$96,$9C),
     ($E8,$95,$B7),($E8,$95,$BE),($E8,$96,$90),($E8,$97,$89),
     ($E8,$96,$BA),($E8,$97,$8F),($E8,$96,$B9),($E8,$97,$90),
     ($E8,$97,$95),($E8,$97,$9D),($E8,$97,$A5),($E8,$97,$9C),
     ($E8,$97,$B9),($E8,$98,$8A),($E8,$98,$93),($E8,$98,$8B),
     ($E8,$97,$BE),($E8,$97,$BA),($E8,$98,$86),($E8,$98,$A2),
     ($E8,$98,$9A),($E8,$98,$B0),($E8,$98,$BF),($E8,$99,$8D),
     ($E4,$B9,$95),($E8,$99,$94),($E8,$99,$9F),($E8,$99,$A7),
     ($E8,$99,$B1),($E8,$9A,$93),($E8,$9A,$A3),($E8,$9A,$A9),
     ($E8,$9A,$AA),($E8,$9A,$8B),($E8,$9A,$8C),($E8,$9A,$B6),
     ($E8,$9A,$AF),($E8,$9B,$84),($E8,$9B,$86),($E8,$9A,$B0),
     ($E8,$9B,$89),($E8,$A0,$A3),($E8,$9A,$AB),($E8,$9B,$94),
     ($E8,$9B,$9E),($E8,$9B,$A9),($E8,$9B,$AC),($00,$00,$00),
     ($E8,$9B,$9F),($E8,$9B,$9B),($E8,$9B,$AF),($E8,$9C,$92),
     ($E8,$9C,$86),($E8,$9C,$88),($E8,$9C,$80),($E8,$9C,$83),
     ($E8,$9B,$BB),($E8,$9C,$91),($E8,$9C,$89),($E8,$9C,$8D),
     ($E8,$9B,$B9),($E8,$9C,$8A),($E8,$9C,$B4),($E8,$9C,$BF),
     ($E8,$9C,$B7),($E8,$9C,$BB),($E8,$9C,$A5),($E8,$9C,$A9),
     ($E8,$9C,$9A),($E8,$9D,$A0),($E8,$9D,$9F),($E8,$9D,$B8),
     ($E8,$9D,$8C),($E8,$9D,$8E),($E8,$9D,$B4),($E8,$9D,$97),
     ($E8,$9D,$A8),($E8,$9D,$AE),($E8,$9D,$99),($E8,$9D,$93),
     ($E8,$9D,$A3),($E8,$9D,$AA),($E8,$A0,$85),($E8,$9E,$A2),
     ($E8,$9E,$9F),($E8,$9E,$82),($E8,$9E,$AF),($E8,$9F,$8B),
     ($E8,$9E,$BD),($E8,$9F,$80),($E8,$9F,$90),($E9,$9B,$96),
     ($E8,$9E,$AB),($E8,$9F,$84),($E8,$9E,$B3),($E8,$9F,$87),
     ($E8,$9F,$86),($E8,$9E,$BB),($E8,$9F,$AF),($E8,$9F,$B2),
     ($E8,$9F,$A0),($E8,$A0,$8F),($E8,$A0,$8D),($E8,$9F,$BE),
     ($E8,$9F,$B6),($E8,$9F,$B7),($E8,$A0,$8E),($E8,$9F,$92),
     ($E8,$A0,$91),($E8,$A0,$96),($E8,$A0,$95),($E8,$A0,$A2),
     ($E8,$A0,$A1),($E8,$A0,$B1),($E8,$A0,$B6),($E8,$A0,$B9),
     ($E8,$A0,$A7),($E8,$A0,$BB),($E8,$A1,$84),($E8,$A1,$82),
     ($E8,$A1,$92),($E8,$A1,$99),($E8,$A1,$9E),($E8,$A1,$A2),
     ($E8,$A1,$AB),($E8,$A2,$81),($E8,$A1,$BE),($E8,$A2,$9E),
     ($E8,$A1,$B5),($E8,$A1,$BD),($E8,$A2,$B5),($E8,$A1,$B2),
     ($E8,$A2,$82),($E8,$A2,$97),($E8,$A2,$92),($E8,$A2,$AE),
     ($E8,$A2,$99),($E8,$A2,$A2),($E8,$A2,$8D),($E8,$A2,$A4),
     ($E8,$A2,$B0),($E8,$A2,$BF),($E8,$A2,$B1),($E8,$A3,$83),
     ($E8,$A3,$84),($E8,$A3,$94),($E8,$A3,$98),($E8,$A3,$99),
     ($E8,$A3,$9D),($E8,$A3,$B9),($E8,$A4,$82),($E8,$A3,$BC),
     ($E8,$A3,$B4),($E8,$A3,$A8),($E8,$A3,$B2),($E8,$A4,$84),
     ($E8,$A4,$8C),($E8,$A4,$8A),($E8,$A4,$93),($E8,$A5,$83),
     ($E8,$A4,$9E),($E8,$A4,$A5),($E8,$A4,$AA),($E8,$A4,$AB),
     ($E8,$A5,$81),($E8,$A5,$84),($E8,$A4,$BB),($E8,$A4,$B6),
     ($E8,$A4,$B8),($E8,$A5,$8C),($E8,$A4,$9D),($E8,$A5,$A0),
     ($E8,$A5,$9E));

  S2U_E6: TSjisRowTable =
    (($E8,$A5,$A6),($E8,$A5,$A4),($E8,$A5,$AD),($E8,$A5,$AA),
     ($E8,$A5,$AF),($E8,$A5,$B4),($E8,$A5,$B7),($E8,$A5,$BE),
     ($E8,$A6,$83),($E8,$A6,$88),($E8,$A6,$8A),($E8,$A6,$93),
     ($E8,$A6,$98),($E8,$A6,$A1),($E8,$A6,$A9),($E8,$A6,$A6),
     ($E8,$A6,$AC),($E8,$A6,$AF),($E8,$A6,$B2),($E8,$A6,$BA),
     ($E8,$A6,$BD),($E8,$A6,$BF),($E8,$A7,$80),($E8,$A7,$9A),
     ($E8,$A7,$9C),($E8,$A7,$9D),($E8,$A7,$A7),($E8,$A7,$B4),
     ($E8,$A7,$B8),($E8,$A8,$83),($E8,$A8,$96),($E8,$A8,$90),
     ($E8,$A8,$8C),($E8,$A8,$9B),($E8,$A8,$9D),($E8,$A8,$A5),
     ($E8,$A8,$B6),($E8,$A9,$81),($E8,$A9,$9B),($E8,$A9,$92),
     ($E8,$A9,$86),($E8,$A9,$88),($E8,$A9,$BC),($E8,$A9,$AD),
     ($E8,$A9,$AC),($E8,$A9,$A2),($E8,$AA,$85),($E8,$AA,$82),
     ($E8,$AA,$84),($E8,$AA,$A8),($E8,$AA,$A1),($E8,$AA,$91),
     ($E8,$AA,$A5),($E8,$AA,$A6),($E8,$AA,$9A),($E8,$AA,$A3),
     ($E8,$AB,$84),($E8,$AB,$8D),($E8,$AB,$82),($E8,$AB,$9A),
     ($E8,$AB,$AB),($E8,$AB,$B3),($E8,$AB,$A7),($00,$00,$00),
     ($E8,$AB,$A4),($E8,$AB,$B1),($E8,$AC,$94),($E8,$AB,$A0),
     ($E8,$AB,$A2),($E8,$AB,$B7),($E8,$AB,$9E),($E8,$AB,$9B),
     ($E8,$AC,$8C),($E8,$AC,$87),($E8,$AC,$9A),($E8,$AB,$A1),
     ($E8,$AC,$96),($E8,$AC,$90),($E8,$AC,$97),($E8,$AC,$A0),
     ($E8,$AC,$B3),($E9,$9E,$AB),($E8,$AC,$A6),($E8,$AC,$AB),
     ($E8,$AC,$BE),($E8,$AC,$A8),($E8,$AD,$81),($E8,$AD,$8C),
     ($E8,$AD,$8F),($E8,$AD,$8E),($E8,$AD,$89),($E8,$AD,$96),
     ($E8,$AD,$9B),($E8,$AD,$9A),($E8,$AD,$AB),($E8,$AD,$9F),
     ($E8,$AD,$AC),($E8,$AD,$AF),($E8,$AD,$B4),($E8,$AD,$BD),
     ($E8,$AE,$80),($E8,$AE,$8C),($E8,$AE,$8E),($E8,$AE,$92),
     ($E8,$AE,$93),($E8,$AE,$96),($E8,$AE,$99),($E8,$AE,$9A),
     ($E8,$B0,$BA),($E8,$B1,$81),($E8,$B0,$BF),($E8,$B1,$88),
     ($E8,$B1,$8C),($E8,$B1,$8E),($E8,$B1,$90),($E8,$B1,$95),
     ($E8,$B1,$A2),($E8,$B1,$AC),($E8,$B1,$B8),($E8,$B1,$BA),
     ($E8,$B2,$82),($E8,$B2,$89),($E8,$B2,$85),($E8,$B2,$8A),
     ($E8,$B2,$8D),($E8,$B2,$8E),($E8,$B2,$94),($E8,$B1,$BC),
     ($E8,$B2,$98),($E6,$88,$9D),($E8,$B2,$AD),($E8,$B2,$AA),
     ($E8,$B2,$BD),($E8,$B2,$B2),($E8,$B2,$B3),($E8,$B2,$AE),
     ($E8,$B2,$B6),($E8,$B3,$88),($E8,$B3,$81),($E8,$B3,$A4),
     ($E8,$B3,$A3),($E8,$B3,$9A),($E8,$B3,$BD),($E8,$B3,$BA),
     ($E8,$B3,$BB),($E8,$B4,$84),($E8,$B4,$85),($E8,$B4,$8A),
     ($E8,$B4,$87),($E8,$B4,$8F),($E8,$B4,$8D),($E8,$B4,$90),
     ($E9,$BD,$8E),($E8,$B4,$93),($E8,$B3,$8D),($E8,$B4,$94),
     ($E8,$B4,$96),($E8,$B5,$A7),($E8,$B5,$AD),($E8,$B5,$B1),
     ($E8,$B5,$B3),($E8,$B6,$81),($E8,$B6,$99),($E8,$B7,$82),
     ($E8,$B6,$BE),($E8,$B6,$BA),($E8,$B7,$8F),($E8,$B7,$9A),
     ($E8,$B7,$96),($E8,$B7,$8C),($E8,$B7,$9B),($E8,$B7,$8B),
     ($E8,$B7,$AA),($E8,$B7,$AB),($E8,$B7,$9F),($E8,$B7,$A3),
     ($E8,$B7,$BC),($E8,$B8,$88),($E8,$B8,$89),($E8,$B7,$BF),
     ($E8,$B8,$9D),($E8,$B8,$9E),($E8,$B8,$90),($E8,$B8,$9F),
     ($E8,$B9,$82),($E8,$B8,$B5),($E8,$B8,$B0),($E8,$B8,$B4),
     ($E8,$B9,$8A));

  S2U_E7: TSjisRowTable =
    (($E8,$B9,$87),($E8,$B9,$89),($E8,$B9,$8C),($E8,$B9,$90),
     ($E8,$B9,$88),($E8,$B9,$99),($E8,$B9,$A4),($E8,$B9,$A0),
     ($E8,$B8,$AA),($E8,$B9,$A3),($E8,$B9,$95),($E8,$B9,$B6),
     ($E8,$B9,$B2),($E8,$B9,$BC),($E8,$BA,$81),($E8,$BA,$87),
     ($E8,$BA,$85),($E8,$BA,$84),($E8,$BA,$8B),($E8,$BA,$8A),
     ($E8,$BA,$93),($E8,$BA,$91),($E8,$BA,$94),($E8,$BA,$99),
     ($E8,$BA,$AA),($E8,$BA,$A1),($E8,$BA,$AC),($E8,$BA,$B0),
     ($E8,$BB,$86),($E8,$BA,$B1),($E8,$BA,$BE),($E8,$BB,$85),
     ($E8,$BB,$88),($E8,$BB,$8B),($E8,$BB,$9B),($E8,$BB,$A3),
     ($E8,$BB,$BC),($E8,$BB,$BB),($E8,$BB,$AB),($E8,$BB,$BE),
     ($E8,$BC,$8A),($E8,$BC,$85),($E8,$BC,$95),($E8,$BC,$92),
     ($E8,$BC,$99),($E8,$BC,$93),($E8,$BC,$9C),($E8,$BC,$9F),
     ($E8,$BC,$9B),($E8,$BC,$8C),($E8,$BC,$A6),($E8,$BC,$B3),
     ($E8,$BC,$BB),($E8,$BC,$B9),($E8,$BD,$85),($E8,$BD,$82),
     ($E8,$BC,$BE),($E8,$BD,$8C),($E8,$BD,$89),($E8,$BD,$86),
     ($E8,$BD,$8E),($E8,$BD,$97),($E8,$BD,$9C),($00,$00,$00),
     ($E8,$BD,$A2),($E8,$BD,$A3),($E8,$BD,$A4),($E8,$BE,$9C),
     ($E8,$BE,$9F),($E8,$BE,$A3),($E8,$BE,$AD),($E8,$BE,$AF),
     ($E8,$BE,$B7),($E8,$BF,$9A),($E8,$BF,$A5),($E8,$BF,$A2),
     ($E8,$BF,$AA),($E8,$BF,$AF),($E9,$82,$87),($E8,$BF,$B4),
     ($E9,$80,$85),($E8,$BF,$B9),($E8,$BF,$BA),($E9,$80,$91),
     ($E9,$80,$95),($E9,$80,$A1),($E9,$80,$8D),($E9,$80,$9E),
     ($E9,$80,$96),($E9,$80,$8B),($E9,$80,$A7),($E9,$80,$B6),
     ($E9,$80,$B5),($E9,$80,$B9),($E8,$BF,$B8),($E9,$81,$8F),
     ($E9,$81,$90),($E9,$81,$91),($E9,$81,$92),($E9,$80,$8E),
     ($E9,$81,$89),($E9,$80,$BE),($E9,$81,$96),($E9,$81,$98),
     ($E9,$81,$9E),($E9,$81,$A8),($E9,$81,$AF),($E9,$81,$B6),
     ($E9,$9A,$A8),($E9,$81,$B2),($E9,$82,$82),($E9,$81,$BD),
     ($E9,$82,$81),($E9,$82,$80),($E9,$82,$8A),($E9,$82,$89),
     ($E9,$82,$8F),($E9,$82,$A8),($E9,$82,$AF),($E9,$82,$B1),
     ($E9,$82,$B5),($E9,$83,$A2),($E9,$83,$A4),($E6,$89,$88),
     ($E9,$83,$9B),($E9,$84,$82),($E9,$84,$92),($E9,$84,$99),
     ($E9,$84,$B2),($E9,$84,$B0),($E9,$85,$8A),($E9,$85,$96),
     ($E9,$85,$98),($E9,$85,$A3),($E9,$85,$A5),($E9,$85,$A9),
     ($E9,$85,$B3),($E9,$85,$B2),($E9,$86,$8B),($E9,$86,$89),
     ($E9,$86,$82),($E9,$86,$A2),($E9,$86,$AB),($E9,$86,$AF),
     ($E9,$86,$AA),($E9,$86,$B5),($E9,$86,$B4),($E9,$86,$BA),
     ($E9,$87,$80),($E9,$87,$81),($E9,$87,$89),($E9,$87,$8B),
     ($E9,$87,$90),($E9,$87,$96),($E9,$87,$9F),($E9,$87,$A1),
     ($E9,$87,$9B),($E9,$87,$BC),($E9,$87,$B5),($E9,$87,$B6),
     ($E9,$88,$9E),($E9,$87,$BF),($E9,$88,$94),($E9,$88,$AC),
     ($E9,$88,$95),($E9,$88,$91),($E9,$89,$9E),($E9,$89,$97),
     ($E9,$89,$85),($E9,$89,$89),($E9,$89,$A4),($E9,$89,$88),
     ($E9,$8A,$95),($E9,$88,$BF),($E9,$89,$8B),($E9,$89,$90),
     ($E9,$8A,$9C),($E9,$8A,$96),($E9,$8A,$93),($E9,$8A,$9B),
     ($E9,$89,$9A),($E9,$8B,$8F),($E9,$8A,$B9),($E9,$8A,$B7),
     ($E9,$8B,$A9),($E9,$8C,$8F),($E9,$8B,$BA),($E9,$8D,$84),
     ($E9,$8C,$AE));

  S2U_E8: TSjisRowTable =
    (($E9,$8C,$99),($E9,$8C,$A2),($E9,$8C,$9A),($E9,$8C,$A3),
     ($E9,$8C,$BA),($E9,$8C,$B5),($E9,$8C,$BB),($E9,$8D,$9C),
     ($E9,$8D,$A0),($E9,$8D,$BC),($E9,$8D,$AE),($E9,$8D,$96),
     ($E9,$8E,$B0),($E9,$8E,$AC),($E9,$8E,$AD),($E9,$8E,$94),
     ($E9,$8E,$B9),($E9,$8F,$96),($E9,$8F,$97),($E9,$8F,$A8),
     ($E9,$8F,$A5),($E9,$8F,$98),($E9,$8F,$83),($E9,$8F,$9D),
     ($E9,$8F,$90),($E9,$8F,$88),($E9,$8F,$A4),($E9,$90,$9A),
     ($E9,$90,$94),($E9,$90,$93),($E9,$90,$83),($E9,$90,$87),
     ($E9,$90,$90),($E9,$90,$B6),($E9,$90,$AB),($E9,$90,$B5),
     ($E9,$90,$A1),($E9,$90,$BA),($E9,$91,$81),($E9,$91,$92),
     ($E9,$91,$84),($E9,$91,$9B),($E9,$91,$A0),($E9,$91,$A2),
     ($E9,$91,$9E),($E9,$91,$AA),($E9,$88,$A9),($E9,$91,$B0),
     ($E9,$91,$B5),($E9,$91,$B7),($E9,$91,$BD),($E9,$91,$9A),
     ($E9,$91,$BC),($E9,$91,$BE),($E9,$92,$81),($E9,$91,$BF),
     ($E9,$96,$82),($E9,$96,$87),($E9,$96,$8A),($E9,$96,$94),
     ($E9,$96,$96),($E9,$96,$98),($E9,$96,$99),($00,$00,$00),
     ($E9,$96,$A0),($E9,$96,$A8),($E9,$96,$A7),($E9,$96,$AD),
     ($E9,$96,$BC),($E9,$96,$BB),($E9,$96,$B9),($E9,$96,$BE),
     ($E9,$97,$8A),($E6,$BF,$B6),($E9,$97,$83),($E9,$97,$8D),
     ($E9,$97,$8C),($E9,$97,$95),($E9,$97,$94),($E9,$97,$96),
     ($E9,$97,$9C),($E9,$97,$A1),($E9,$97,$A5),($E9,$97,$A2),
     ($E9,$98,$A1),($E9,$98,$A8),($E9,$98,$AE),($E9,$98,$AF),
     ($E9,$99,$82),($E9,$99,$8C),($E9,$99,$8F),($E9,$99,$8B),
     ($E9,$99,$B7),($E9,$99,$9C),($E9,$99,$9E),($E9,$99,$9D),
     ($E9,$99,$9F),($E9,$99,$A6),($E9,$99,$B2),($E9,$99,$AC),
     ($E9,$9A,$8D),($E9,$9A,$98),($E9,$9A,$95),($E9,$9A,$97),
     ($E9,$9A,$AA),($E9,$9A,$A7),($E9,$9A,$B1),($E9,$9A,$B2),
     ($E9,$9A,$B0),($E9,$9A,$B4),($E9,$9A,$B6),($E9,$9A,$B8),
     ($E9,$9A,$B9),($E9,$9B,$8E),($E9,$9B,$8B),($E9,$9B,$89),
     ($E9,$9B,$8D),($E8,$A5,$8D),($E9,$9B,$9C),($E9,$9C,$8D),
     ($E9,$9B,$95),($E9,$9B,$B9),($E9,$9C,$84),($E9,$9C,$86),
     ($E9,$9C,$88),($E9,$9C,$93),($E9,$9C,$8E),($E9,$9C,$91),
     ($E9,$9C,$8F),($E9,$9C,$96),($E9,$9C,$99),($E9,$9C,$A4),
     ($E9,$9C,$AA),($E9,$9C,$B0),($E9,$9C,$B9),($E9,$9C,$BD),
     ($E9,$9C,$BE),($E9,$9D,$84),($E9,$9D,$86),($E9,$9D,$88),
     ($E9,$9D,$82),($E9,$9D,$89),($E9,$9D,$9C),($E9,$9D,$A0),
     ($E9,$9D,$A4),($E9,$9D,$A6),($E9,$9D,$A8),($E5,$8B,$92),
     ($E9,$9D,$AB),($E9,$9D,$B1),($E9,$9D,$B9),($E9,$9E,$85),
     ($E9,$9D,$BC),($E9,$9E,$81),($E9,$9D,$BA),($E9,$9E,$86),
     ($E9,$9E,$8B),($E9,$9E,$8F),($E9,$9E,$90),($E9,$9E,$9C),
     ($E9,$9E,$A8),($E9,$9E,$A6),($E9,$9E,$A3),($E9,$9E,$B3),
     ($E9,$9E,$B4),($E9,$9F,$83),($E9,$9F,$86),($E9,$9F,$88),
     ($E9,$9F,$8B),($E9,$9F,$9C),($E9,$9F,$AD),($E9,$BD,$8F),
     ($E9,$9F,$B2),($E7,$AB,$9F),($E9,$9F,$B6),($E9,$9F,$B5),
     ($E9,$A0,$8F),($E9,$A0,$8C),($E9,$A0,$B8),($E9,$A0,$A4),
     ($E9,$A0,$A1),($E9,$A0,$B7),($E9,$A0,$BD),($E9,$A1,$86),
     ($E9,$A1,$8F),($E9,$A1,$8B),($E9,$A1,$AB),($E9,$A1,$AF),
     ($E9,$A1,$B0));

  S2U_E9: TSjisRowTable =
    (($E9,$A1,$B1),($E9,$A1,$B4),($E9,$A1,$B3),($E9,$A2,$AA),
     ($E9,$A2,$AF),($E9,$A2,$B1),($E9,$A2,$B6),($E9,$A3,$84),
     ($E9,$A3,$83),($E9,$A3,$86),($E9,$A3,$A9),($E9,$A3,$AB),
     ($E9,$A4,$83),($E9,$A4,$89),($E9,$A4,$92),($E9,$A4,$94),
     ($E9,$A4,$98),($E9,$A4,$A1),($E9,$A4,$9D),($E9,$A4,$9E),
     ($E9,$A4,$A4),($E9,$A4,$A0),($E9,$A4,$AC),($E9,$A4,$AE),
     ($E9,$A4,$BD),($E9,$A4,$BE),($E9,$A5,$82),($E9,$A5,$89),
     ($E9,$A5,$85),($E9,$A5,$90),($E9,$A5,$8B),($E9,$A5,$91),
     ($E9,$A5,$92),($E9,$A5,$8C),($E9,$A5,$95),($E9,$A6,$97),
     ($E9,$A6,$98),($E9,$A6,$A5),($E9,$A6,$AD),($E9,$A6,$AE),
     ($E9,$A6,$BC),($E9,$A7,$9F),($E9,$A7,$9B),($E9,$A7,$9D),
     ($E9,$A7,$98),($E9,$A7,$91),($E9,$A7,$AD),($E9,$A7,$AE),
     ($E9,$A7,$B1),($E9,$A7,$B2),($E9,$A7,$BB),($E9,$A7,$B8),
     ($E9,$A8,$81),($E9,$A8,$8F),($E9,$A8,$85),($E9,$A7,$A2),
     ($E9,$A8,$99),($E9,$A8,$AB),($E9,$A8,$B7),($E9,$A9,$85),
     ($E9,$A9,$82),($E9,$A9,$80),($E9,$A9,$83),($00,$00,$00),
     ($E9,$A8,$BE),($E9,$A9,$95),($E9,$A9,$8D),($E9,$A9,$9B),
     ($E9,$A9,$97),($E9,$A9,$9F),($E9,$A9,$A2),($E9,$A9,$A5),
     ($E9,$A9,$A4),($E9,$A9,$A9),($E9,$A9,$AB),($E9,$A9,$AA),
     ($E9,$AA,$AD),($E9,$AA,$B0),($E9,$AA,$BC),($E9,$AB,$80),
     ($E9,$AB,$8F),($E9,$AB,$91),($E9,$AB,$93),($E9,$AB,$94),
     ($E9,$AB,$9E),($E9,$AB,$9F),($E9,$AB,$A2),($E9,$AB,$A3),
     ($E9,$AB,$A6),($E9,$AB,$AF),($E9,$AB,$AB),($E9,$AB,$AE),
     ($E9,$AB,$B4),($E9,$AB,$B1),($E9,$AB,$B7),($E9,$AB,$BB),
     ($E9,$AC,$86),($E9,$AC,$98),($E9,$AC,$9A),($E9,$AC,$9F),
     ($E9,$AC,$A2),($E9,$AC,$A3),($E9,$AC,$A5),($E9,$AC,$A7),
     ($E9,$AC,$A8),($E9,$AC,$A9),($E9,$AC,$AA),($E9,$AC,$AE),
     ($E9,$AC,$AF),($E9,$AC,$B2),($E9,$AD,$84),($E9,$AD,$83),
     ($E9,$AD,$8F),($E9,$AD,$8D),($E9,$AD,$8E),($E9,$AD,$91),
     ($E9,$AD,$98),($E9,$AD,$B4),($E9,$AE,$93),($E9,$AE,$83),
     ($E9,$AE,$91),($E9,$AE,$96),($E9,$AE,$97),($E9,$AE,$9F),
     ($E9,$AE,$A0),($E9,$AE,$A8),($E9,$AE,$B4),($E9,$AF,$80),
     ($E9,$AF,$8A),($E9,$AE,$B9),($E9,$AF,$86),($E9,$AF,$8F),
     ($E9,$AF,$91),($E9,$AF,$92),($E9,$AF,$A3),($E9,$AF,$A2),
     ($E9,$AF,$A4),($E9,$AF,$94),($E9,$AF,$A1),($E9,$B0,$BA),
     ($E9,$AF,$B2),($E9,$AF,$B1),($E9,$AF,$B0),($E9,$B0,$95),
     ($E9,$B0,$94),($E9,$B0,$89),($E9,$B0,$93),($E9,$B0,$8C),
     ($E9,$B0,$86),($E9,$B0,$88),($E9,$B0,$92),($E9,$B0,$8A),
     ($E9,$B0,$84),($E9,$B0,$AE),($E9,$B0,$9B),($E9,$B0,$A5),
     ($E9,$B0,$A4),($E9,$B0,$A1),($E9,$B0,$B0),($E9,$B1,$87),
     ($E9,$B0,$B2),($E9,$B1,$86),($E9,$B0,$BE),($E9,$B1,$9A),
     ($E9,$B1,$A0),($E9,$B1,$A7),($E9,$B1,$B6),($E9,$B1,$B8),
     ($E9,$B3,$A7),($E9,$B3,$AC),($E9,$B3,$B0),($E9,$B4,$89),
     ($E9,$B4,$88),($E9,$B3,$AB),($E9,$B4,$83),($E9,$B4,$86),
     ($E9,$B4,$AA),($E9,$B4,$A6),($E9,$B6,$AF),($E9,$B4,$A3),
     ($E9,$B4,$9F),($E9,$B5,$84),($E9,$B4,$95),($E9,$B4,$92),
     ($E9,$B5,$81),($E9,$B4,$BF),($E9,$B4,$BE),($E9,$B5,$86),
     ($E9,$B5,$88));

  S2U_EA: TSjisRowTable =
    (($E9,$B5,$9D),($E9,$B5,$9E),($E9,$B5,$A4),($E9,$B5,$91),
     ($E9,$B5,$90),($E9,$B5,$99),($E9,$B5,$B2),($E9,$B6,$89),
     ($E9,$B6,$87),($E9,$B6,$AB),($E9,$B5,$AF),($E9,$B5,$BA),
     ($E9,$B6,$9A),($E9,$B6,$A4),($E9,$B6,$A9),($E9,$B6,$B2),
     ($E9,$B7,$84),($E9,$B7,$81),($E9,$B6,$BB),($E9,$B6,$B8),
     ($E9,$B6,$BA),($E9,$B7,$86),($E9,$B7,$8F),($E9,$B7,$82),
     ($E9,$B7,$99),($E9,$B7,$93),($E9,$B7,$B8),($E9,$B7,$A6),
     ($E9,$B7,$AD),($E9,$B7,$AF),($E9,$B7,$BD),($E9,$B8,$9A),
     ($E9,$B8,$9B),($E9,$B8,$9E),($E9,$B9,$B5),($E9,$B9,$B9),
     ($E9,$B9,$BD),($E9,$BA,$81),($E9,$BA,$88),($E9,$BA,$8B),
     ($E9,$BA,$8C),($E9,$BA,$92),($E9,$BA,$95),($E9,$BA,$91),
     ($E9,$BA,$9D),($E9,$BA,$A5),($E9,$BA,$A9),($E9,$BA,$B8),
     ($E9,$BA,$AA),($E9,$BA,$AD),($E9,$9D,$A1),($E9,$BB,$8C),
     ($E9,$BB,$8E),($E9,$BB,$8F),($E9,$BB,$90),($E9,$BB,$94),
     ($E9,$BB,$9C),($E9,$BB,$9E),($E9,$BB,$9D),($E9,$BB,$A0),
     ($E9,$BB,$A5),($E9,$BB,$A8),($E9,$BB,$AF),($00,$00,$00),
     ($E9,$BB,$B4),($E9,$BB,$B6),($E9,$BB,$B7),($E9,$BB,$B9),
     ($E9,$BB,$BB),($E9,$BB,$BC),($E9,$BB,$BD),($E9,$BC,$87),
     ($E9,$BC,$88),($E7,$9A,$B7),($E9,$BC,$95),($E9,$BC,$A1),
     ($E9,$BC,$AC),($E9,$BC,$BE),($E9,$BD,$8A),($E9,$BD,$92),
     ($E9,$BD,$94),($E9,$BD,$A3),($E9,$BD,$9F),($E9,$BD,$A0),
     ($E9,$BD,$A1),($E9,$BD,$A6),($E9,$BD,$A7),($E9,$BD,$AC),
     ($E9,$BD,$AA),($E9,$BD,$B7),($E9,$BD,$B2),($E9,$BD,$B6),
     ($E9,$BE,$95),($E9,$BE,$9C),($E9,$BE,$A0),($E5,$A0,$AF),
     ($E6,$A7,$87),($E9,$81,$99),($E7,$91,$A4),($E5,$87,$9C),
     ($E7,$86,$99),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00),($00,$00,$00),($00,$00,$00),($00,$00,$00),
     ($00,$00,$00));

function SjisToUtf8(Value: String): String;
var
  Index, Len: Integer;
  Eos: Boolean;
  function Pop(): Byte;
  begin
    Result := 0;
    Inc(Index);
    if Len < Index then
    begin
      Eos := True;
      exit;
    end;
    Result := Ord(Value[Index]);
  end;
var
  Ch1: Byte;
  function GetUtf8(): String;
  var
    Ch2: Byte;
    UC: TUtfChar;
    i: Integer;
  begin
    Result := '';
    Ch2 := Pop(); if Eos then exit;
    case Ch2 of
      $40..$FC: ; // Go next step
    else
      exit; // Invalid Shift_jis range
    end;
    case Ch1 of
      $81..$84: UC := Sjis2Utf8Table_81_84[Ch1][Ch2];
      $88..$9F: UC := Sjis2Utf8Table_88_9F[Ch1][Ch2];
      $E0..$EA: UC := Sjis2Utf8Table_E0_EA[Ch1][Ch2];
    else // Not JIS X 0208
      exit; // Just ignored
    end;
    for i := 0 to 2 do
      if 0 < UC[i] then Result := Result + Chr(UC[i])
      else              break;
  end;
begin
  Result := '';
  Index := 0;
  Len := Length(Value);
  Eos := False;
  while True do
  begin
    Ch1 := Pop(); if Eos then break;
    case Ch1 of
      $00..$7E: // ASCII
        Result := Result + Chr(Ch1);
      $A0..$DF: // Hankaku kana
        ; // Just ignored
    else // Kanji 1
      Result := Result + GetUtf8();
    end;
  end;
end;

function Utf8ToSjis(Value: String): String;
var
  Index, Len: Integer;
  Eos: Boolean;
  function Pop(): Byte;
  begin
    Result := 0;
    Inc(Index);
    if Len < Index then
    begin
      Eos := True;
      exit;
    end;
    Result := Ord(Value[Index]);
  end;
var
  Ch1, Ch2, Ch3: Byte;
  function GetSjis(): String;
  var
    Row, Col: Integer;
    UC: TUtfChar;
  begin
    Result := '';
    for Row := $81 to $84 do
      for Col := $40 to $FC do
      begin
        UC := Sjis2Utf8Table_81_84[Row][Col];
        if (Ch1 = UC[0]) and (Ch2 = UC[1]) and (Ch3 = UC[2]) then
        begin
          Result := Chr(Row) + Chr(Col);
          exit;
        end;
      end;
    for Row := $88 to $9F do
      for Col := $40 to $FC do
      begin
        UC := Sjis2Utf8Table_88_9F[Row][Col];
        if (Ch1 = UC[0]) and (Ch2 = UC[1]) and (Ch3 = UC[2]) then
        begin
          Result := Chr(Row) + Chr(Col);
          exit;
        end;
      end;
    for Row := $E0 to $EA do
      for Col := $40 to $FC do
      begin
        UC := Sjis2Utf8Table_E0_EA[Row][Col];
        if (Ch1 = UC[0]) and (Ch2 = UC[1]) and (Ch3 = UC[2]) then
        begin
          Result := Chr(Row) + Chr(Col);
          exit;
        end;
      end;
  end;
begin
  Result := '';
  Index := 0;
  Len := Length(Value);
  Eos := False;
  while True do
  begin
    Ch1 := Pop(); if Eos then break;
    if      $00 = ($80 and Ch1) then // ASCII
      Result := Result + Chr(Ch1)
    else if $C0 = ($E0 and Ch1) then // 2 bytes
    begin
      Ch2 := Pop(); if Eos then break;
      Ch3 := $00;
      Result := Result + GetSjis();
    end
    else if $E0 = ($F0 and Ch1) then // 3 bytes
    begin
      Ch2 := Pop(); if Eos then break;
      Ch3 := Pop(); if Eos then break;
      Result := Result + GetSjis();
    end
    else if $F0 = ($F8 and Ch1) then // 4 bytes; Not Japanese charactor
    begin
      Pop(); if Eos then break;
      Pop(); if Eos then break;
      Pop(); if Eos then break;
    end
    else // 4 bytes over or other invalidation
      raise ERangeError.Create('Invalid UTF-8 encoding');
  end;
end;

initialization

  Sjis2Utf8Table_81_84[$81] := S2U_81;
  Sjis2Utf8Table_81_84[$82] := S2U_82;
  Sjis2Utf8Table_81_84[$83] := S2U_83;
  Sjis2Utf8Table_81_84[$84] := S2U_84;
  Sjis2Utf8Table_88_9F[$88] := S2U_88;
  Sjis2Utf8Table_88_9F[$89] := S2U_89;
  Sjis2Utf8Table_88_9F[$8A] := S2U_8A;
  Sjis2Utf8Table_88_9F[$8B] := S2U_8B;
  Sjis2Utf8Table_88_9F[$8C] := S2U_8C;
  Sjis2Utf8Table_88_9F[$8D] := S2U_8D;
  Sjis2Utf8Table_88_9F[$8E] := S2U_8E;
  Sjis2Utf8Table_88_9F[$8F] := S2U_8F;
  Sjis2Utf8Table_88_9F[$90] := S2U_90;
  Sjis2Utf8Table_88_9F[$91] := S2U_91;
  Sjis2Utf8Table_88_9F[$92] := S2U_92;
  Sjis2Utf8Table_88_9F[$93] := S2U_93;
  Sjis2Utf8Table_88_9F[$94] := S2U_94;
  Sjis2Utf8Table_88_9F[$95] := S2U_95;
  Sjis2Utf8Table_88_9F[$96] := S2U_96;
  Sjis2Utf8Table_88_9F[$97] := S2U_97;
  Sjis2Utf8Table_88_9F[$98] := S2U_98;
  Sjis2Utf8Table_88_9F[$99] := S2U_99;
  Sjis2Utf8Table_88_9F[$9A] := S2U_9A;
  Sjis2Utf8Table_88_9F[$9B] := S2U_9B;
  Sjis2Utf8Table_88_9F[$9C] := S2U_9C;
  Sjis2Utf8Table_88_9F[$9D] := S2U_9D;
  Sjis2Utf8Table_88_9F[$9E] := S2U_9E;
  Sjis2Utf8Table_88_9F[$9F] := S2U_9F;
  Sjis2Utf8Table_E0_EA[$E0] := S2U_E0;
  Sjis2Utf8Table_E0_EA[$E1] := S2U_E1;
  Sjis2Utf8Table_E0_EA[$E2] := S2U_E2;
  Sjis2Utf8Table_E0_EA[$E3] := S2U_E3;
  Sjis2Utf8Table_E0_EA[$E4] := S2U_E4;
  Sjis2Utf8Table_E0_EA[$E5] := S2U_E5;
  Sjis2Utf8Table_E0_EA[$E6] := S2U_E6;
  Sjis2Utf8Table_E0_EA[$E7] := S2U_E7;
  Sjis2Utf8Table_E0_EA[$E8] := S2U_E8;
  Sjis2Utf8Table_E0_EA[$E9] := S2U_E9;
  Sjis2Utf8Table_E0_EA[$EA] := S2U_EA;

end.
